-- Copyright (C) 2009, CA, Inc. All rights reserved.
--
-- This file contains the schema for all of the 12.5 SP1
-- tables for snapshot
--
--
-------------------------------------------------------------------

-- INNODB is required for FOREIGN KEY CONSTRAINTS and AUTO_INCREMENT.
SET storage_engine=INNODB;


-- Add the Version table
CREATE TABLE imreport_version (
    imr_key     VARCHAR(255) NOT NULL,
    imr_value   VARCHAR(255) NOT NULL
);

INSERT INTO imreport_version(imr_key,imr_value) VALUES ('version','12.5.7.0.0');


--
-- IM Report6
--
CREATE TABLE imreport6 (
    imr_reportid            INT NOT NULL,
    imr_reportname          VARCHAR(255) NOT NULL,
    imr_reportdescription   VARCHAR(628) NULL,
    -- default date/time only supported for TIMESTAMP column, else have to write trigger
    imr_reporttime          TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    imr_reportuser          VARCHAR(628) NOT NULL,
    imr_envoid              VARCHAR(64) NOT NULL,
    imr_envname             VARCHAR(255) NULL,
    imr_keepsnapshot        INT,
    imr_snapshotsource      VARCHAR(255),
    imr_snapshotstatus      VARCHAR(64)
);

ALTER TABLE imreport6 ADD
    CONSTRAINT PK_01_IMREPORT6 PRIMARY KEY (
        imr_reportid
);

ALTER TABLE imreport6 MODIFY
        imr_reportid INT NOT NULL AUTO_INCREMENT;

CREATE INDEX imreport6_index_1 ON imreport6 (imr_reporttime);

CREATE INDEX imreport6_index_2 ON imreport6 (imr_reportid, imr_envoid);

CREATE INDEX imreport6_index_3 ON imreport6 (imr_reportid, imr_envname);


--
-- SP to insert into the report table
--
DROP PROCEDURE IF EXISTS insertintoimreport;

DELIMITER $$

CREATE PROCEDURE insertintoimreport (
    IN reportname           VARCHAR(255),
    IN reportdescription    VARCHAR(628),
    IN reportuser           VARCHAR(628),
    IN envoid               VARCHAR(64),
    IN envname              VARCHAR(255),
    IN keepsnapshot         INT,
    IN snapshotsource       VARCHAR(255),
    IN snapshotstatus       VARCHAR(64),
    OUT current_id          INT
)
BEGIN
    INSERT INTO imreport6 (
        imr_reportname ,
        imr_reportdescription,
        imr_reportuser ,
        imr_envoid ,
        imr_envname ,
        imr_keepsnapshot ,
        imr_snapshotsource ,
        imr_snapshotstatus
    )
    VALUES (
        reportname ,
        reportdescription ,
        reportuser ,
        envoid ,
        envname ,
        keepsnapshot ,
        snapshotsource ,
        snapshotstatus
    );
    SELECT LAST_INSERT_ID() INTO current_id;
END$$

DELIMITER ;


--
-- IMR ROLE6
--
CREATE TABLE imrrole6 (
    imr_reportid            INT NOT NULL,
    imr_oid                 VARCHAR(64) NOT NULL,
    imr_name                VARCHAR(255)NULL,
    imr_description         VARCHAR(255) NULL,
    imr_type                VARCHAR(64) NOT NULL,
    imr_isenabled           INTEGER NULL,
    imr_isassignable        INTEGER NULL,
    imr_isadminassignable   INTEGER NULL
);

ALTER TABLE imrrole6 ADD
    CONSTRAINT PK_01_IMRROLE6 PRIMARY KEY (
        imr_reportid,
        imr_oid
);

CREATE INDEX imrrole6_index_1 ON imrrole6 (imr_reportid, imr_name);


-- IMR ROLE RULES 6
-- member, administrator and owner rules
-- add and remove actions
--
CREATE TABLE imrrolerule6 (
    imr_reportid        INT NOT NULL,
    imr_roleoid         VARCHAR(64) NOT NULL,
    imr_ruleorder       INT NOT NULL DEFAULT 0,
    imr_ruletype        VARCHAR(64) NULL,
    imr_constraintvalue VARCHAR(3800) NULL
);

ALTER TABLE imrrolerule6 ADD
    CONSTRAINT PK_01_IMRROLERULE6 PRIMARY KEY (
        imr_reportid,
        imr_roleoid,
        imr_ruleorder
);

CREATE INDEX imrrolerule6_index_1 ON imrrolerule6 (imr_reportid, imr_roleoid);


--
-- IMR ROLE SCOPE RULES 6
-- scope rules associated with role rules (admin, member and owner rules)
--
CREATE TABLE imrrolescoperule6 (
    imr_reportid        INT NOT NULL,
    imr_roleoid         VARCHAR(64) NOT NULL,
    imr_roleruleorder   INT NOT NULL DEFAULT 0,
    imr_scopeobjecttype VARCHAR(64) NOT NULL,
    imr_constraintvalue VARCHAR(3800) NULL
);

ALTER TABLE imrrolescoperule6 ADD
    CONSTRAINT PK_01_IMRROLESCOPERULE6 PRIMARY KEY (
        imr_reportid,
        imr_roleoid,
        imr_roleruleorder,
        imr_scopeobjecttype
);

CREATE INDEX imrrolescoperule6_index_1 ON imrrolescoperule6 (imr_reportid, imr_roleoid, imr_roleruleorder);


--
-- IMR ROLE TASK 6
--
CREATE TABLE imrroletask6 (
    imr_reportid    INT NOT NULL,
    imr_roleoid     VARCHAR(64) NOT NULL,
    imr_taskoid     VARCHAR(64) NOT NULL
);

ALTER TABLE imrroletask6 ADD
    CONSTRAINT PK_01_IMRROLETASK6 PRIMARY KEY (
        imr_reportid,
        imr_roleoid,
        imr_taskoid
);

CREATE INDEX imrroletask6_index_1 ON imrroletask6 (imr_reportid, imr_roleoid);


--
-- IMR TASK 6
--
CREATE TABLE imrtask6 (
    imr_reportid INTEGER NOT NULL,
    imr_oid             VARCHAR(64) NOT NULL,
    imr_name            VARCHAR(255) NULL,
    imr_description     VARCHAR(255) NULL,
    imr_type            VARCHAR(64) NOT NULL,
    imr_category        VARCHAR(255) NULL,
    imr_workflowprocess VARCHAR(255) NULL,
    imr_application     VARCHAR(255) NULL,
    imr_externalurl     VARCHAR(255) NULL,
    imr_tasktag         VARCHAR(255) NULL,
    imr_reserved1       VARCHAR(255) NULL,
    imr_reserved2       VARCHAR(255) NULL,
    imr_reserved3       VARCHAR(255) NULL,
    imr_reserved4       VARCHAR(255) NULL,
    imr_reserved5       VARCHAR(255) NULL,
    imr_reserved6       VARCHAR(255) NULL,
    imr_reserved7       VARCHAR(255) NULL,
    imr_reserved8       VARCHAR(255) NULL,
    imr_reserved9       VARCHAR(255) NULL,
    imr_reserved10      VARCHAR(255) NULL
);

ALTER TABLE imrtask6 ADD
    CONSTRAINT PK_01_IMRTASK6 PRIMARY KEY (
        imr_reportid,
        imr_oid
);


--
-- IMR ROLE MEMBER 6
--
CREATE TABLE imrrolemember6 (
    imr_reportid    INT NOT NULL,
    imr_roleoid     VARCHAR(64) NOT NULL,
    imr_userdn      VARCHAR(628) NOT NULL
);

CREATE INDEX imrrolemember6_index_1 ON imrrolemember6 (imr_reportid, imr_roleoid);

CREATE INDEX imrrolemember6_index_2 ON imrrolemember6 (imr_reportid, imr_userdn);


--
-- IMR ROLEADMIN6
--
CREATE TABLE imrroleadmin6 (
    imr_reportid    INT NOT NULL,
    imr_roleoid     VARCHAR(64) NOT NULL,
    imr_userdn      VARCHAR(628) NOT NULL
 );

CREATE INDEX imrroleadmin6_index_1 ON imrroleadmin6 (imr_reportid, imr_roleoid);

CREATE INDEX imrroleadmin6_index_2 ON imrroleadmin6 (imr_reportid, imr_userdn);


--
-- IMR ROLEOWNER6
--
CREATE TABLE imrroleowner6 (
    imr_reportid    INT NOT NULL,
    imr_roleoid     VARCHAR(64) NOT NULL,
    imr_userdn      VARCHAR(628) NOT NULL
);

CREATE INDEX imrroleowner6_index_1 ON imrroleowner6 (imr_reportid, imr_roleoid);

CREATE INDEX imrroleowner6_index_2 ON imrroleowner6 (imr_reportid, imr_userdn);


--
-- SP to insert members into role tables
--

DROP PROCEDURE IF EXISTS insertusermembertable;

DELIMITER $$

CREATE PROCEDURE insertusermembertable (
    tablename   VARCHAR(128),
    reportid    INT,
    roleoid     VARCHAR(64),
    userdn      VARCHAR(628))
BEGIN
    DECLARE rowcount INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    IF (tablename = 'imrrolemember6') THEN
        SELECT COUNT(*) INTO rowcount FROM imrrolemember6 WHERE
                imr_reportid = reportid AND
                imr_roleoid  = roleoid AND
                imr_userdn   = userdn;
    ELSEIF (tablename = 'imrroleadmin6') THEN
        SELECT COUNT(*) INTO rowcount FROM imrroleadmin6 WHERE
                imr_reportid = reportid AND
                imr_roleoid  = roleoid AND
                imr_userdn   = userdn;
    ELSEIF (tablename = 'imrroleowner6') then
        SELECT COUNT(*) INTO rowcount FROM imrroleowner6 WHERE
                imr_reportid = reportid AND
                imr_roleoid  = roleoid AND
                imr_userdn   = userdn;
    END IF;

    IF rowcount = 0 THEN
        IF (tablename = 'imrrolemember6') THEN
            INSERT INTO imrrolemember6 (imr_reportid, imr_roleoid, imr_userdn)
                    VALUES (reportid, roleoid, userdn);
        ELSEIF (tablename = 'imrroleadmin6') THEN
            INSERT INTO imrroleadmin6 (imr_reportid, imr_roleoid, imr_userdn)
                    VALUES (reportid, roleoid, userdn);
        ELSEIF (tablename = 'imrroleowner6') THEN
            INSERT INTO imrroleowner6 (imr_reportid ,imr_roleoid ,imr_userdn)
                    VALUES (reportid, roleoid, userdn);
        END IF;
    END IF;
END$$

DELIMITER ;


--
-- IMR ORG6
--
CREATE TABLE imrorg6 (
    imr_reportid    INT NOT NULL,
    imr_orgdn       VARCHAR(628) NOT NULL,
    imr_orgname     VARCHAR(255) NULL,
    imr_parentorgdn VARCHAR(628) NULL
);

CREATE INDEX imrorg6_index_1 ON imrorg6 (imr_reportid, imr_parentorgdn);


-- Insert into imrorg6
DROP PROCEDURE IF EXISTS insertintoorgtable;

DELIMITER $$

CREATE PROCEDURE insertintoorgtable (
    reportid    INT,
    orgdn       VARCHAR(628),
    orgname     VARCHAR(255),
    parentorgdn VARCHAR(628))
BEGIN
    DECLARE rowcount INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    SELECT COUNT(*) INTO rowcount FROM imrorg6 WHERE
            imr_reportid = reportid AND imr_orgdn = orgdn;
    IF rowcount = 0 THEN
        INSERT INTO imrorg6 (imr_reportid, imr_orgdn, imr_orgname, imr_parentorgdn)
                VALUES (reportid, orgdn, orgname, parentorgdn);
    END IF;
END$$

DELIMITER ;


--
-- IMR OrgAttr6
--

CREATE TABLE imrorgattr6 (
    imr_reportid    INT NOT NULL,
    imr_orgdn       VARCHAR(628) NOT NULL,
    imr_attrname    VARCHAR(255) NOT NULL,
    imr_attrvalue   VARCHAR(628) NULL
);

CREATE INDEX imrorgattr6_index_2 ON imrorgattr6 (imr_reportid, imr_attrname, imr_attrvalue);


--
-- IMR GROUP6
--
CREATE TABLE imrgroup6 (
    imr_reportid                INT NOT NULL,
    imr_groupdn                 VARCHAR(628) NOT NULL,
    imr_name                    VARCHAR(255) NULL,
    imr_description             VARCHAR(255) NULL,
    imr_parentorgdn             VARCHAR(628) NOT NULL,
    imr_selfsubscribing         VARCHAR(64) NULL,
    imr_dynamicgroupmembership  VARCHAR(255) NULL
);

CREATE INDEX imrgroup6_index_1 ON imrgroup6 (imr_reportid, imr_parentorgdn);


--
-- Insert into imrgroup6
--
DROP PROCEDURE IF EXISTS insertintogrouptable;

DELIMITER $$

CREATE PROCEDURE insertintogrouptable (
    reportid                INT,
    groupdn                 VARCHAR(628),
    name                    VARCHAR(255),
    description             VARCHAR(255),
    parentorgdn             VARCHAR(628),
    selfsubscribing         VARCHAR(64),
    dynamicgroupmembership  VARCHAR(255))
BEGIN
    DECLARE rowcount INT DEFAULT 0;
    
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    SELECT COUNT(*) INTO rowcount from imrgroup6 WHERE
            imr_reportid = reportid AND imr_groupdn = groupdn;

    IF rowcount = 0  THEN
        INSERT INTO imrgroup6 (
                imr_reportid, imr_groupdn, imr_name, imr_description,
                imr_parentorgdn, imr_selfsubscribing, imr_dynamicgroupmembership)
        VALUES (
                reportid, groupdn, name, description,
                parentorgdn, selfsubscribing, dynamicgroupmembership);
    END IF;
END$$

DELIMITER ;

--
-- IMR GroupAttr6
--
CREATE TABLE imrgroupattr6 (
    imr_reportid    INT NOT NULL,
    imr_groupdn     VARCHAR(628) NOT NULL,
    imr_attrname    VARCHAR(255) NOT NULL,
    imr_attrvalue   VARCHAR(628) NULL
);

CREATE INDEX imrgroupattr6_index_2 ON imrgroupattr6 (imr_reportid, imr_attrname, imr_attrvalue);


--
-- IMR GroupMember6
--
CREATE TABLE imrgroupmember6 (
    imr_reportid    INT NOT NULL,
    imr_groupdn     VARCHAR(628) NOT NULL,
    imr_memberdn    VARCHAR(628) NOT NULL
);

CREATE INDEX imrgroupmember6_index_2 ON imrgroupmember6 (imr_reportid, imr_memberdn);


--
-- IMR GroupAdmin6
--
CREATE TABLE imrgroupadmin6 (
    imr_reportid    INT NOT NULL,
    imr_groupdn     VARCHAR(628) NOT NULL,
    imr_admindn     VARCHAR(628) NOT NULL
);

CREATE INDEX imrgroupadmin6_index_2 ON imrgroupadmin6 (imr_reportid, imr_admindn);


--
-- IMR User 6
--
CREATE TABLE imruser6 (
    imr_reportid    INT NOT NULL,
    imr_userdn      VARCHAR(628) NOT NULL,
    imr_isenabled   INT NULL DEFAULT 1,
    imr_userid      VARCHAR(255) NULL,
    imr_fullname    VARCHAR(255) NULL,
    imr_lastname    VARCHAR(255) NULL,
    imr_firstname   VARCHAR(255) NULL,
    imr_parentorgdn VARCHAR(628) NULL,
    imr_email       VARCHAR(255) NULL
);

CREATE INDEX imruser6_index_1 ON imruser6 (imr_reportid, imr_parentorgdn);

CREATE INDEX imruser6_index_2 ON imruser6 (imr_reportid, imr_userid);

CREATE INDEX imruser6_index_3 ON imruser6 (imr_reportid, imr_userdn);
--
-- Insert into user table
--
DROP PROCEDURE IF EXISTS insertintousertable;

DELIMITER $$

CREATE PROCEDURE insertintousertable (
    reportid    INT,
    userdn      VARCHAR(628),
    isenabled   INT,
    userid      VARCHAR(255),
    fullname    VARCHAR(255),
    lastname    VARCHAR(255),
    firstname   VARCHAR(255),
    parentorgdn VARCHAR(628),
    email       VARCHAR(255))
BEGIN
    DECLARE rowcount INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    SELECT COUNT(*) INTO rowcount from imruser6 WHERE
            imruser6.imr_reportid = reportid AND imruser6.imr_userdn = userdn;

    IF rowcount = 0  THEN
        INSERT INTO imruser6 (
                imr_reportid, imr_userdn, imr_isenabled, imr_userid, imr_fullname,
                imr_lastname, imr_firstname, imr_parentorgdn, imr_email)
        VALUES (
                reportid, userdn, isenabled, userid, fullname,
                lastname, firstname, parentorgdn, email);
    END IF;
END$$

DELIMITER ;


--
-- IMR userattr6
--
CREATE TABLE imruserattr6 (
    imr_reportid    INT NOT NULL,
    imr_userdn      VARCHAR(628) NOT NULL,
    imr_attrname    VARCHAR(255) NOT NULL,
    imr_attrvalue   VARCHAR(628) NULL
);

CREATE  INDEX imruserattr6_index_2 ON imruserattr6 (imr_reportid, imr_attrname, imr_userdn);


--
-- IM imridentitypolicy6
--
CREATE TABLE imridentitypolicy6 (
    imr_reportid            INT NOT NULL,
    imr_policyid            VARCHAR(64) NOT NULL,
    imr_policyname          VARCHAR(255) NOT NULL,
    imr_policyindex         INT NOT NULL,
    imr_apptype             VARCHAR(64) NOT NULL,
    imr_compliance          VARCHAR(64) NOT NULL,
    imr_ruleevaluationtype  VARCHAR(64) NOT NULL,
    imr_rule                VARCHAR(3000) NOT NULL,
    imr_policysetid         VARCHAR(64) NOT NULL
);

ALTER TABLE imridentitypolicy6 ADD
        CONSTRAINT PK_01_imridentitypolicy6 PRIMARY KEY (
                imr_reportid,
                imr_policyid,
                imr_policysetid
);

CREATE INDEX imridentitypolicy6_index_1 ON imridentitypolicy6 (imr_reportid, imr_policyid);

CREATE INDEX imridentitypolicy6_index_2 ON imridentitypolicy6 (imr_reportid, imr_policysetid);


--
-- IMR imridentitypolicyset6
--
CREATE TABLE imridentitypolicyset6 (
    imr_reportid        INT NOT NULL,
    imr_policysetid     VARCHAR(64) NOT NULL,
    imr_policysetname   VARCHAR(255) NOT NULL,
    imr_category        VARCHAR(255) NOT NULL
);

ALTER TABLE imridentitypolicyset6 ADD
        CONSTRAINT PK_01_imridentitypolicyset6 PRIMARY KEY (
                imr_reportid,
                imr_policysetid
);


--
-- IM imridentitypolicyactions6
--
CREATE TABLE imridentitypolicyactions6 (
    imr_reportid                INT NOT NULL,
    imr_policyid                VARCHAR(64) NOT NULL,
    imr_actiontype              VARCHAR(64) NOT NULL,
    imr_actionindex             INT NOT NULL,
    imr_actionevaluationtype    VARCHAR(64) NOT NULL,
    imr_action                  VARCHAR(3000) NOT NULL
);

ALTER TABLE imridentitypolicyactions6 ADD
        CONSTRAINT PK_01_imrpolicyactions6 PRIMARY KEY (
                imr_reportid,
                imr_policyid,
                imr_actiontype,
                imr_actionindex
);

CREATE INDEX imrpolicyactions6_index_1 ON imridentitypolicyactions6 (imr_reportid, imr_policyid);


--
-- IM imridentitypolicyusersync6
--
CREATE TABLE imridentitypolicyusersync6 (
    imr_reportid        INT NOT NULL,
    imr_policysetname   VARCHAR(255) NOT NULL,
    imr_policyid        VARCHAR(64) NOT NULL,
    imr_userdn          VARCHAR(255) NOT NULL,
    imr_syncstatus      VARCHAR(255) NOT NULL
);

ALTER TABLE imridentitypolicyusersync6 ADD
        CONSTRAINT PK_01_imrpolicyusersync6 PRIMARY KEY (
                imr_reportid,
                imr_policyid,
                imr_userdn
);

CREATE INDEX imrpolicyusersync6_index_1 ON imridentitypolicyusersync6 (imr_reportid, imr_userdn);

CREATE INDEX imrpolicyusersync6_index_2 ON imridentitypolicyusersync6 (imr_reportid, imr_policyid);


CREATE OR REPLACE VIEW imrroleinfo6
AS
    SELECT imrrole6.imr_reportid, imrrole6.imr_oid, imrrole6.imr_name, imrrole6.imr_type,
        imruser6.imr_userdn,
        imrrolemember6.imr_userdn AS member,
        imrroleowner6.imr_userdn AS owner,
        imrroleadmin6.imr_userdn AS admin
    FROM imrrole6 INNER JOIN
        imruser6 ON imrrole6.imr_reportid = imruser6.imr_reportid AND
        imrrole6.imr_reportid = imruser6.imr_reportid LEFT OUTER JOIN
        imrroleowner6 ON imrrole6.imr_reportid = imrroleowner6.imr_reportid AND
        imrroleowner6.imr_userdn = imruser6.imr_userdn AND imrroleowner6.imr_reportid = imruser6.imr_reportid AND
        imrroleowner6.imr_roleoid = imrrole6.imr_oid LEFT OUTER JOIN
        imrrolemember6 ON imrrole6.imr_reportid = imrrolemember6.imr_reportid AND
        imrrolemember6.imr_userdn = imruser6.imr_userdn AND
        imrrolemember6.imr_reportid = imruser6.imr_reportid AND
        imrrolemember6.imr_roleoid = imrrole6.imr_oid LEFT OUTER JOIN
        imrroleadmin6 ON imrrole6.imr_reportid = imrroleowner6.imr_reportid AND
        imrroleadmin6.imr_userdn = imruser6.imr_userdn AND imrroleadmin6.imr_reportid = imruser6.imr_reportid AND
        imrroleadmin6.imr_roleoid = imrrole6.imr_oid
    WHERE (NOT (imrrolemember6.imr_userdn IS NULL AND imrroleowner6.imr_userdn IS NULL AND
        imrroleadmin6.imr_userdn IS NULL));


--
-- IM imrptsnapshots12
--
CREATE TABLE imrptsnapshots12 (
    imr_rptname             VARCHAR(255) NOT NULL,
    imr_snapshotname        VARCHAR(255) NOT NULL,
    imr_envname             VARCHAR(255) NOT NULL,
    IMR_RPT_TASK_TAG_NAME   VARCHAR(255) NOT NULL
);

CREATE INDEX imrptsnapshots12_index_1 ON imrptsnapshots12 (imr_rptname, imr_snapshotname, imr_envname);


------------------------------------------------------
-- New r12.5 support.
------------------------------------------------------

CREATE TABLE imrAttribute (
    imr_reportid    INT NOT NULL,
    imr_objectid    INT NOT NULL,
    imr_attrinfoid  INT NOT NULL,
    imr_value       VARCHAR(4000),
    imr_objectrefid INT
);

CREATE INDEX imrattribute_index_1 ON imrattribute (imr_reportid, imr_attrinfoid, imr_objectid);

CREATE INDEX imrattribute_index_2 ON imrattribute (imr_reportid, imr_objectid);

CREATE INDEX imrattribute_index_3 ON imrattribute (imr_reportid, imr_value);


CREATE TABLE imrAttributeInfo (
    imr_reportid        INT NOT NULL,
    imr_attrinfoid      INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    imr_syntaxid        INT NOT NULL,
    imr_name            VARCHAR(64) NOT NULL,
    imr_objectinfoid    INT NOT NULL,
    imr_isbasetype      VARCHAR(256) NULL
);

CREATE INDEX imrattributeinfo_index_1 ON imrattributeinfo (imr_reportid, imr_name);


CREATE TABLE imrObject (
    imr_reportid        INT NOT NULL,
    imr_friendlyname    VARCHAR(255) NOT NULL,
    imr_objectdn        VARCHAR(628) NOT NULL,
    imr_objectinfoname  VARCHAR(255) NOT NULL,
    imr_objectid        INT NOT NULL AUTO_INCREMENT PRIMARY KEY
);

CREATE INDEX imrobject_index_1 ON imrobject (imr_reportid, imr_friendlyname);

CREATE INDEX imrobject_index_2 ON imrobject (imr_reportid, imr_objectdn);


CREATE TABLE imrObjectCategory (
    imr_reportid    INT NOT NULL,
    imr_name        VARCHAR(255) NOT NULL
);

CREATE INDEX imrobjectcategory_index_1 ON imrobjectcategory (imr_reportid, imr_name);


CREATE TABLE imrObjectInfo (
    imr_reportid        INT NOT NULL,
    imr_objectid        INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    imr_name            VARCHAR(255) NOT NULL,
    imr_objectcatname   VARCHAR(255) NOT NULL
);

CREATE INDEX imrobjectinfo_index_1 ON imrobjectinfo (imr_reportid, imr_name);


CREATE TABLE imrProvRolesRelationship (
    imr_reportid    INT NOT NULL,
    imr_parentoid   VARCHAR(64) NOT NULL,
    imr_childoid    VARCHAR(64) NOT NULL
);

CREATE INDEX imrprovrolesrel_index_1 ON imrProvRolesRelationship (imr_reportid, imr_parentoid, imr_childoid);


CREATE TABLE imrProvRoleTemplRelationship (
    imr_reportid    INT NOT NULL,
    imr_roleoid     VARCHAR(64) NOT NULL,
    imr_roledn      VARCHAR(628) NOT NULL,
    imr_templatedn  VARCHAR(628) NOT NULL
);

CREATE INDEX imrprovroletemplrel_index_1 ON imrprovroletemplrelationship (imr_reportid, imr_roleoid, imr_templatedn, imr_roledn);


CREATE TABLE imrUserAccountRelationship (
    imr_reportid        INT NOT NULL,
    imr_useraccountdn   VARCHAR(1024) NOT NULL,
    imr_provuserdn      VARCHAR(1024) NOT NULL,
    imr_userid          VARCHAR(1024) NOT NULL
);

CREATE INDEX imruseraccountrel_index_1 ON imruseraccountrelationship (imr_reportid, imr_userid);

CREATE INDEX imruseraccountrel_index_2 ON imruseraccountrelationship (imr_reportid, imr_provuserdn);

CREATE INDEX imruseraccountrel_index_3 ON imruseraccountrelationship (imr_reportid, imr_useraccountdn);


--
-- IM imrTemplEndpointRelationship
--
CREATE TABLE imrTemplEndpointRelationship (
    imr_reportid        INT NOT NULL,
    imr_templatedn      VARCHAR(628),
    imr_endpointname    VARCHAR(628),
    imr_endpointdn      VARCHAR(628)
);

CREATE INDEX imrtemplendpointrel_index_1 ON imrtemplendpointrelationship (imr_reportid, imr_templatedn, imr_endpointname);


--
-- IM imrAccount
--
CREATE TABLE imrAccountTemplateRel (
    imr_reportid        INT NOT NULL,
    imr_name            VARCHAR(628) NOT NULL,
    imr_templatedn      VARCHAR(628),
    imr_endpointtype    VARCHAR(628)
);

CREATE INDEX imrAccountTemplateRel_index_1 ON imrAccountTemplateRel (imr_reportid, imr_name, imr_templatedn, imr_endpointtype);


--
-- IM imrCapabilityAttrRelationship
--
CREATE TABLE imrCapabilityAttrRel (
    imr_reportid                INT NOT NULL,
    imr_endpointtype            VARCHAR(628) NOT NULL,
    imr_objecttypename          VARCHAR(628) NOT NULL,
    imr_jiamobjecttypename      VARCHAR(628) NOT NULL,
    imr_attrid                  INT,
    imr_capabilityobjectname    VARCHAR(628) NOT NULL,
    imr_capabilityobjectdn      VARCHAR(628) NOT NULL,
    imr_endpointobjid           INT,
    imr_name                    VARCHAR(64),
    imr_value                   VARCHAR(4000)
);

CREATE INDEX imrcapabilityattrrel_index_1 ON imrcapabilityattrrel (imr_reportid, imr_endpointtype, imr_endpointobjid, imr_objecttypename, imr_capabilityobjectdn, imr_name);


--
-- IM imrroleattr12
--
CREATE TABLE imrroleattr12 (
    imr_reportid    INT NOT NULL,
    imr_roleoid     VARCHAR(64) NOT NULL,
    imr_name        VARCHAR(628) NOT NULL,
    imr_value       VARCHAR(4000) NULL
);

CREATE INDEX imrroleattr12_index_1 ON imrroleattr12 (imr_reportid, imr_roleoid, imr_name);


DROP PROCEDURE IF EXISTS insertCapabilityAttrRel;

DELIMITER $$

CREATE PROCEDURE insertCapabilityAttrRel(
    reportid                INT,
    endpointtype            VARCHAR(628),
    objecttypename          VARCHAR(628),
    jiamobjecttypename      VARCHAR(628),
    attrid                  INT,
    capabilityobjectname    VARCHAR(628),
    capabilityobjectdn      VARCHAR(628),
    endpointobjid           INT,
    attrname                VARCHAR(64),
    attrvalue               VARCHAR(4000))
BEGIN
    DECLARE rowcount INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    SELECT COUNT(*) INTO rowcount FROM imrCapabilityAttrRel WHERE
            imr_reportid = reportid AND
            imr_endpointtype = endpointtype AND
            imr_objecttypename = objecttypename AND
            imr_endpointtype = endpointtype AND
            imr_jiamobjecttypename = jiamobjecttypename AND
            imr_attrid = attrid AND
            imr_capabilityobjectname = capabilityobjectname AND
            imr_capabilityobjectdn = capabilityobjectdn AND
            imr_endpointobjid = endpointobjid AND
            imr_name = attrname;
    
    IF rowcount = 0 THEN
        INSERT INTO imrCapabilityAttrRel(
                imr_reportid, imr_endpointtype, imr_objecttypename, imr_jiamobjecttypename,
                imr_attrid, imr_capabilityobjectname, imr_capabilityobjectdn, imr_endpointobjid,
                imr_name, imr_value)
        VALUES (
                reportid, endpointtype, objecttypename, jiamobjecttypename,
                attrid, capabilityobjectname, capabilityobjectdn, endpointobjid,
                attrname, attrvalue);
    END IF;
END$$


DROP PROCEDURE IF EXISTS insertTemplEndpointRel$$

CREATE PROCEDURE insertTemplEndpointRel(
    reportid        INT,
    templatedn      VARCHAR(628),
    endpointname    VARCHAR(628),
    endpointdn      VARCHAR(628))
BEGIN
    DECLARE rowcount INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    SELECT COUNT(*) INTO rowcount FROM imrTemplEndpointRelationship WHERE
            imr_reportid = reportid AND
            imr_templatedn = templatedn AND
            imr_endpointname = endpointname AND
            imr_endpointdn = endpointdn;
    
    IF rowcount = 0 THEN
        INSERT INTO imrTemplEndpointRelationship (imr_reportid, imr_templatedn, imr_endpointname, imr_endpointdn)
        VALUES (reportid, templatedn, endpointname, endpointdn);
    END IF;
END$$


--
-- IM insertAccountTemplateRel
--
CREATE PROCEDURE insertAccountTemplateRel(
    reportid        INT,
    name            VARCHAR(628),
    templatedn      VARCHAR(628),
    endpointtype    VARCHAR(628))
BEGIN
    DECLARE rowcount INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    SELECT COUNT(*) INTO rowcount FROM imrAccountTemplateRel WHERE
            imr_reportid = reportid AND
            imr_name = name AND
            imr_templatedn = templatedn AND
            imr_endpointtype = endpointtype;

    IF rowcount = 0 THEN
        INSERT INTO imrAccountTemplateRel (imr_reportid, imr_name, imr_templatedn, imr_endpointtype)
        VALUES (reportid, name, templatedn, endpointtype);
    END IF;
END$$

DELIMITER ;


------------------------
-- Views
------------------------

CREATE OR REPLACE VIEW imruseraccounts6
AS
    SELECT
        accountObject.imr_reportid AS imr_reportid,
        imruser.imr_userid AS imr_userid,
        accountObject.imr_friendlyname AS imr_accountname,
        accountObject.imr_objectdn AS imr_accountdn,
        SUBSTRING(accountObject.imr_objectdn,
            INSTR(accountObject.imr_objectdn,',EndPoint=') + LENGTH(',EndPoint='),
            CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0 THEN
                INSTR(accountObject.imr_objectdn,',Namespace=') -
                        INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
            ELSE
                0
            END) as imr_endpoint,
        accountObject.imr_objectinfoname as imr_accounttype,
        accountAttr1.imr_value as imr_createtime
    FROM
        imrObject accountObject,
        imrUserAccountRelationship userAccountRelationship,
        imrAttributeInfo accountAttrInfo1,
        imrAttribute accountAttr1,
        imruser6 imruser
    WHERE
        imruser.imr_userid = userAccountRelationship.imr_userid AND
        accountObject.imr_objectdn = userAccountRelationship.imr_useraccountdn AND
        accountAttr1.imr_attrinfoid = accountAttrInfo1.imr_attrinfoid AND
        accountAttrInfo1.imr_name='createDate' AND
        accountObject.imr_objectid = accountAttr1.imr_objectid AND
        accountObject.imr_reportid=imruser.imr_reportid AND
        userAccountRelationship.imr_reportid = imruser.imr_reportid AND
        accountAttrInfo1.imr_reportid = imruser.imr_reportid AND
        accountAttr1.imr_reportid = imruser.imr_reportid;


CREATE OR REPLACE VIEW imraccountattrs6
AS
    SELECT
        accountObject.imr_reportid AS imr_reportid,
        accountObject.imr_objectid AS accountObjid,
        accountObject.imr_friendlyname AS imr_accountname,
        accountObject.imr_objectdn AS imr_accountdn,
        SUBSTRING(accountObject.imr_objectdn,
            INSTR(accountObject.imr_objectdn,',EndPoint=') + LENGTH(',EndPoint='),
            CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0 THEN
                INSTR(accountObject.imr_objectdn,',Namespace=') -
                        INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
            ELSE
                0
            END) AS imr_endpoint,
        accountObjectInfo.imr_name AS imr_endpointtype,
        resourceAttrInfo.imr_name AS imr_accountattr,
        resourceAttr.imr_value AS imr_accountattrvalue,
        resourceAttr.imr_objectrefid AS imr_objectrefid,
        resourceAttrInfo.imr_isbasetype AS imr_isbasetype
    FROM
        imrObjectCategory accountCategory,
        imrObjectInfo accountObjectInfo,
        imrObject accountObject,
        imrAttributeInfo resourceAttrInfo,
        imrAttribute resourceAttr
    WHERE
        (resourceAttr.imr_value IS NOT NULL OR resourceAttr.imr_objectrefid IS NOT NULL) AND
        resourceAttr.imr_objectid = accountObject.imr_objectid AND
        resourceAttrInfo.imr_attrinfoid = resourceAttr.imr_attrinfoid AND
        accountObject.imr_objectinfoname = accountObjectInfo.imr_name AND
        accountObjectInfo.imr_objectcatname = accountCategory.imr_name AND
        accountCategory.imr_name = 'Account' AND
        accountCategory.imr_reportid=accountObject.imr_reportid AND
        accountObjectInfo.imr_reportid=accountObject.imr_reportid AND
        resourceAttrInfo.imr_reportid=accountObject.imr_reportid AND
        resourceAttr.imr_reportid=accountObject.imr_reportid;


CREATE OR REPLACE VIEW endpointaccountattrs6
AS
    SELECT
        accountObject.imr_reportid AS imr_reportid,
        accountObject.imr_objectid AS imr_accountid,
        accountObject.imr_friendlyname AS imr_accountname,
        accountObject.imr_objectdn AS imr_accountdn,
        SUBSTRING(accountObject.imr_objectdn,
            INSTR(accountObject.imr_objectdn,',EndPoint=') + LENGTH(',EndPoint='),
            CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0 THEN
                INSTR(accountObject.imr_objectdn,',Namespace=') -
                        INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
            ELSE
                0
            END) AS imr_endpoint,
        accountObjectInfo.imr_name AS imr_endpointtype,
        attr1.imr_value AS imr_createtime,
        attr3.imr_value AS imr_accountlastmodified
    FROM
        imrObjectInfo accountObjectInfo,
        imrObject accountObject,
        imrAttributeInfo attrInfo1,
        imrAttribute attr1,
        imrAttributeInfo attrInfo3,
        imrAttribute attr3
    WHERE
        ((attr1.imr_value IS NOT NULL ) OR
        (attr3.imr_value IS NOT NULL )) AND
        attr1.imr_objectid = accountObject.imr_objectid AND
        attr3.imr_objectid = accountObject.imr_objectid AND
        accountObject.imr_objectinfoname = accountObjectInfo.imr_name AND
        accountObjectInfo.imr_objectcatname = 'Account' AND
        (attr1.imr_attrinfoid = attrInfo1.imr_attrinfoid AND
        attrInfo1.imr_name = 'createDate') AND
        (attr3.imr_attrinfoid = attrInfo3.imr_attrinfoid AND
        attrInfo3.imr_name = 'updateDate') AND
        accountObjectInfo.imr_reportid=accountObject.imr_reportid AND
        attrInfo1.imr_reportid=accountObject.imr_reportid AND
        attr1.imr_reportid=accountObject.imr_reportid AND
        attrInfo3.imr_reportid=accountObject.imr_reportid AND
        attr3.imr_reportid=accountObject.imr_reportid;


CREATE OR REPLACE VIEW endpointuseraccounts6
AS
    SELECT
        a.imr_reportid imr_reportid,
        b.imr_userid imr_userid,
        a.imr_accountname imr_accountname,
        a.imr_accountdn imr_accountdn,
        a.imr_endpoint imr_endpoint,
        a.imr_endpointtype imr_accounttype,
        a.imr_createtime imr_createtime,
        a.imr_accountlastmodified imr_accountlastmodified,
        c.imr_firstname imr_firstname,
        c.imr_lastname imr_lastname,
        c.imr_fullname imr_fullname
    FROM
        endpointaccountattrs6 a,
        imrUserAccountRelationship b,
        imruser6 c
    WHERE
        b.imr_useraccountdn = a.imr_accountdn AND
        b.imr_reportid = a.imr_reportid AND
        c.imr_reportid = a.imr_reportid AND
        c.imr_userid=b.imr_userid;


---
--- Procedure for counting the number of exception accounts
---

-- Procedures to insert in to the provisioning tables r12.1 additions

DROP PROCEDURE IF EXISTS insertimrAttrInfo;

DELIMITER $$

CREATE PROCEDURE insertimrAttrInfo(
    reportid        INT,
    syntaxid        INT,
    name            VARCHAR(64),
    objectinfoid    INT,
    isbasetype      CHAR(1),
    OUT infoid      INT)
BEGIN
    DECLARE infoid INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET infoid = 0;
    SELECT imr_attrinfoid INTO infoid FROM imrAttributeInfo WHERE
        imr_reportid = reportid AND
        imr_name = name AND
        imr_objectinfoid = objectinfoid;

    IF infoid = 0 THEN
        INSERT INTO imrAttributeInfo (
                imr_reportid, IMR_ATTRINFOID, imr_syntaxid, imr_name, imr_objectinfoid, imr_isbasetype)
        VALUES (
                reportid, infoid, syntaxid, name, objectinfoid, isbasetype );
        SELECT LAST_INSERT_ID() INTO infoid;
    END IF;
END$$


DROP PROCEDURE IF EXISTS insertimrobject$$

CREATE PROCEDURE insertimrobject(
    reportid        INT,
    friendlyname    VARCHAR(255),
    objectdn        VARCHAR(628),
    objectinfoname  VARCHAR(255),
    OUT objectid    INT)
BEGIN
    DECLARE objectid INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET objectid = 0;
    SELECT imr_objectid INTO objectid FROM imrObject WHERE
        imr_reportid = reportid AND
        imr_objectdn = objectdn;

    IF objectid = 0 THEN
        INSERT INTO imrObject (imr_reportid, imr_friendlyname, imr_objectdn, imr_objectinfoname, IMR_OBJECTID)
        VALUES (reportid, friendlyname, objectdn, objectinfoname, objectid);
        SELECT LAST_INSERT_ID() INTO objectid;
    END IF;
END$$


DROP PROCEDURE IF EXISTS insertimrobjectcategory$$

CREATE PROCEDURE insertimrobjectcategory(
    reportid    INT,
    name        VARCHAR(255))
BEGIN
    DECLARE rowcount INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    SELECT COUNT(*) INTO rowcount FROM imrObjectCategory WHERE
        imr_reportid = reportid AND
        imr_name = name;

    IF rowcount = 0 THEN
        INSERT INTO imrObjectCategory (imr_reportid, imr_name)
        VALUES (reportid, name);
    END IF;
END$$


DROP PROCEDURE IF EXISTS insertimrobjectinfo$$

CREATE PROCEDURE insertimrobjectinfo(
    reportid        INT,
    name            VARCHAR(255),
    objectcatname   VARCHAR(255),
    OUT infoid      INT)
BEGIN
    DECLARE infoid INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET infoid = 0;
    SELECT imr_objectid INTO infoid FROM imrObjectInfo WHERE
        imr_reportid = reportid AND
        imr_name = name AND
        imr_objectcatname = objectcatname;

    IF infoid = 0 THEN
        INSERT INTO imrObjectInfo (imr_reportid, imr_objectid, imr_name, imr_objectcatname)
        VALUES (reportid, infoid, name, objectcatname);
        SELECT LAST_INSERT_ID() INTO infoid;
    END IF;
END$$


DROP PROCEDURE IF EXISTS insertimrroletemplate$$

CREATE PROCEDURE insertimrroletemplate(
    reportid    INT,
    roleoid     VARCHAR(64),
    roledn      VARCHAR(628),
    templatedn  VARCHAR(628))
BEGIN
    DECLARE rowcount INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    SELECT COUNT(*) INTO rowcount FROM imrProvRoleTemplRelationship WHERE
        imr_reportid = reportid AND
        imr_roleoid = roleoid AND
        imr_roledn = roledn AND
        imr_templatedn = templatedn;

    IF rowcount = 0 THEN
        INSERT INTO imrProvRoleTemplRelationship (imr_reportid, imr_roleoid, imr_roledn, imr_templatedn)
        VALUES (reportid, roleoid, roledn, templatedn);
    END IF;
END$$


DROP PROCEDURE IF EXISTS insertProvRolesRel$$

CREATE PROCEDURE insertProvRolesRel(
    reportid    INT,
    parentoid   VARCHAR(64),
    childoid    VARCHAR(64))
BEGIN
    DECLARE rowcount INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET rowcount = 0;
    SELECT COUNT(imr_reportid) INTO rowcount FROM imrProvRolesRelationship WHERE
            imr_reportid = reportid AND
            imr_parentoid = parentoid AND
            imr_childoid = childoid;

    IF rowcount = 0 THEN
        INSERT INTO imrProvRolesRelationship(imr_reportid, imr_parentoid, imr_childoid)
        VALUES (reportid, parentoid, childoid);
    END IF;
END$$

DELIMITER ;


CREATE OR REPLACE VIEW endPointAttributes
AS
    SELECT
        Attribute.imr_reportid AS imr_reportid,
        Attribute.imr_objectid AS imr_endpointid,
        Attribute.imr_attrInfoID AS AttributeInfoID,
        AttributeInfo.imr_name AS Name,
        Attribute.imr_value AS AttributeValue
    FROM
        imrattribute Attribute,
        imrattributeinfo AttributeInfo
    WHERE
        imr_objectid IN (
            SELECT
                endpointObject.imr_objectid
            FROM
                imrObject endpointObject,
                imrObjectCategory endpointCategory,
                imrObjectInfo endpointObjectInfo
            WHERE
                endpointObject.imr_objectinfoname = endpointObjectInfo.imr_name AND
                endpointObjectInfo.imr_objectcatname = endpointCategory.imr_name AND
                endpointCategory.imr_name = 'EndPoint' AND
                endpointCategory.imr_reportid = endpointObject.imr_reportid AND
                endpointObjectInfo.imr_reportid = endpointCategory.imr_reportid AND
                endpointObject.imr_reportid = endpointCategory.imr_reportid) AND
            AttributeInfo.imr_attrinfoid = Attribute.imr_attrinfoid;


CREATE OR REPLACE VIEW accttmplattribute
AS
    SELECT
        acctTmplObject.imr_reportid AS imr_reportid,
        acctTmplObject.imr_friendlyname AS imr_accttmplname,
        acctTmplObject.imr_objectdn AS imr_accttmpldn,
        acctTmplObjectInfo.imr_name AS imr_endpointtype,
        resourceAttrInfo.imr_name AS imr_attrname,
        resourceAttr.imr_value AS imr_attrvalue,
        resourceAttr.imr_objectrefid AS imr_objectrefid,
        resourceAttrInfo.imr_isbasetype AS imr_isbasetype
    FROM
        imrObjectCategory acctTmplCategory,
        imrObjectInfo acctTmplObjectInfo,
        imrObject acctTmplObject,
        imrAttributeInfo resourceAttrInfo,
        imrAttribute resourceAttr
    WHERE
        -- NULL attr values means attr is a subattr
        (resourceAttr.imr_value IS NOT NULL OR
        resourceAttr.imr_objectrefid IS NOT NULL) AND
        resourceAttr.imr_objectid = acctTmplObject.imr_objectid AND
        resourceAttrInfo.imr_attrinfoid = resourceAttr.imr_attrinfoid AND
        acctTmplObject.imr_objectinfoname = acctTmplObjectInfo.imr_name AND
        acctTmplObjectInfo.imr_objectcatname = acctTmplCategory.imr_name AND
        acctTmplCategory.imr_name = 'Policy' AND
        acctTmplCategory.imr_reportid=acctTmplObject.imr_reportid AND
        acctTmplObjectInfo.imr_reportid = acctTmplObject.imr_reportid AND
        resourceAttrInfo.imr_reportid = acctTmplObject.imr_reportid AND
        resourceAttr.imr_reportid = acctTmplObject.imr_reportid;


CREATE OR REPLACE VIEW endpointaccount
AS
    SELECT
        accountObject.imr_reportid AS imr_reportid,
        SUBSTRING(accountObject.imr_objectdn,
            INSTR(accountObject.imr_objectdn, ',EndPoint=') + LENGTH(',EndPoint='),
            CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0 THEN
                INSTR(accountObject.imr_objectdn,',Namespace=') -
                        INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
            ELSE
                0
            END) AS imr_endpoint,
        accountObjectInfo.imr_name AS imr_endpointtype,
        accountObject.imr_friendlyname AS imr_accountname,
        accountObject.imr_objectdn AS imr_accountdn
    FROM
        imrObject accountObject,
        imrObjectInfo accountObjectInfo
    WHERE
        accountObject.imr_objectinfoname = accountObjectInfo.imr_name AND
        accountObjectInfo.imr_objectcatname = 'Account' AND
        accountObject.imr_objectdn LIKE 'Account=%' AND
        accountObject.imr_reportid=accountObjectInfo.imr_reportid;


CREATE OR REPLACE VIEW endpointview
AS
    SELECT
        endpointObject.imr_reportid AS imr_reportid,
        endpointObject.imr_friendlyname AS imr_endpoint,
        endpointObject.imr_objectdn AS imr_endpointdn,
        endpointObject.imr_objectid AS imr_endpointid,
        endpointObjectInfo.imr_name AS imr_endpointtype
    FROM
        imrObject endpointObject,
        imrObjectCategory endpointCategory,
        imrObjectInfo endpointObjectInfo
    WHERE
        endpointObject.imr_objectinfoname = endpointObjectInfo.imr_name AND
        endpointObjectInfo.imr_objectcatname = endpointCategory.imr_name AND
        endpointCategory.imr_name = 'EndPoint' AND
        endpointObject.imr_objectdn LIKE 'EndPoint=%' AND
        endpointCategory.imr_reportid = endpointObject.imr_reportid AND
        endpointObjectInfo.imr_reportid = endpointObject.imr_reportid;


CREATE OR REPLACE VIEW provrolesrelationshipview
AS
    SELECT
        provRolesRelationship.imr_reportid AS imr_reportid,
        provRolesRelationship.imr_parentoid AS imr_parentoid,
        provRolesRelationship.imr_childoid AS imr_childoid
    FROM
        imrProvRolesRelationship provRolesRelationship;


CREATE OR REPLACE VIEW accttemplEndpointRelView
AS
    SELECT
        imrObjectInfo.imr_name AS TemplateType,
        imrObject.imr_objectdn AS TemplateDN,
        imrObject.imr_friendlyname AS TemplateName,
        imrTemplEndpointRelationship.imr_endpointname AS EndpointName,
        imrTemplEndpointRelationship.imr_endpointdn AS EndpointDN,
        imrObject.imr_reportid
    FROM
        imrTemplEndpointRelationship RIGHT OUTER JOIN imrObject ON
        imrTemplEndpointRelationship.imr_reportid = imrObject.imr_reportid AND
        imrTemplEndpointRelationship.imr_templatedn = imrObject.imr_objectdn LEFT OUTER JOIN
        imrObjectCategory INNER JOIN imrObjectInfo ON
        imrObjectCategory.imr_name = imrObjectInfo.imr_objectcatname ON
        imrObject.imr_reportid = imrObjectCategory.imr_reportid AND
        imrObject.imr_reportid = imrObjectInfo.imr_reportid AND
        imrObject.imr_objectinfoname = imrObjectInfo.imr_name
    WHERE
        imrObjectCategory.imr_name = 'Policy';


CREATE OR REPLACE VIEW capabilityattrsview
AS
    SELECT
        imrCapabilityAttrRel.imr_reportid,
        imrCapabilityAttrRel.imr_endpointtype AS EndpointType,
        imrObject.imr_friendlyname AS EndpointName,
        imrObject.imr_objectdn AS EndpointDN,
        imrCapabilityAttrRel.imr_objecttypename,
        imrCapabilityAttrRel.imr_jiamobjecttypename,
        imrCapabilityAttrRel.imr_capabilityobjectname,
        imrCapabilityAttrRel.imr_capabilityobjectdn,
        imrCapabilityAttrRel.imr_endpointobjid AS endpointid,
        imrCapabilityAttrRel.imr_attrid AS Attributeid,
        imrCapabilityAttrRel.imr_name,
        imrCapabilityAttrRel.imr_value
    FROM
        imrCapabilityAttrRel,
        imrObject
    WHERE
        imrCapabilityAttrRel.imr_reportid = imrObject.imr_reportid AND
        imrCapabilityAttrRel.imr_endpointobjid = imrObject.imr_objectid;


CREATE OR REPLACE VIEW provroleaccttmplendptrelview
AS
    SELECT
        imrProvRoleTemplRelationship.imr_reportid,
        imrProvRoleTemplRelationship.imr_roleoid,
        imrProvRoleTemplRelationship.imr_roledn,
        accttemplEndpointRelView.TemplateType,
        accttemplEndpointRelView.TemplateDN,
        accttemplEndpointRelView.TemplateName,
        accttemplEndpointRelView.EndpointName,
        accttemplEndpointRelView.EndpointDN
    FROM
        imrProvRoleTemplRelationship LEFT OUTER JOIN accttemplEndpointRelView ON
        imrProvRoleTemplRelationship.imr_reportid = accttemplEndpointRelView.imr_reportid AND
        imrProvRoleTemplRelationship.imr_templatedn = accttemplEndpointRelView.TemplateDN;


------------------------------------------------------
-- New r12.5 SP1 support.
------------------------------------------------------

--
-- IMR imrpolicyxpress125
--
CREATE TABLE imrpolicyxpress125 (
    imr_reportid        INT NOT NULL,
    imr_policyid        VARCHAR(64) NOT NULL,
    imr_policyname      VARCHAR(255) NULL,
    imr_enabled         VARCHAR(5) NULL,
    imr_category        VARCHAR(255) NULL,
    imr_description     VARCHAR(1023) NULL,
    imr_runonce         VARCHAR(5) NULL,
    imr_priority        INT NULL,
    imr_type            VARCHAR(255) NULL,
    imr_policyevents    LONGTEXT NULL,
    imr_policydata      LONGTEXT NULL,
    imr_entryrules      LONGTEXT NULL,
    imr_actionrules     LONGTEXT NULL
);

ALTER TABLE imrpolicyxpress125 ADD
    CONSTRAINT PK_01_imrpolicyxpress125 PRIMARY KEY (
        imr_reportid,
        imr_policyid
);


--
-- IMR imrreversesyncmodifyaccount125
--
CREATE TABLE imrreversesyncmodifyaccount125 (
    imr_reportid        INT NOT NULL,
    imr_policyid        VARCHAR(64) NOT NULL,
    imr_policyname      VARCHAR(255) NULL,
    imr_enabled         VARCHAR(5) NULL,
    imr_description     VARCHAR(1023) NULL,
    imr_priority        INT NULL,
    imr_endpointtype    VARCHAR(255) NULL,
    imr_endpointname    VARCHAR(1023) NULL,
    imr_container       VARCHAR(255) NULL,
    imr_attributename   VARCHAR(255) NULL,
    imr_attributevalue  VARCHAR(1023) NULL,
    imr_action          VARCHAR(255) NULL
);

ALTER TABLE imrreversesyncmodifyaccount125 ADD
    CONSTRAINT PK_01_imrreversesyncmodify125 PRIMARY KEY (
        imr_reportid,
        imr_policyid
);


--
-- IMR imrreversesyncnewaccount125
--
CREATE TABLE imrreversesyncnewaccount125 (
    imr_reportid            INT NOT NULL,
    imr_policyid            VARCHAR(64) NOT NULL,
    imr_policyname          VARCHAR(255) NULL,
    imr_enabled             VARCHAR(5) NULL,
    imr_description         VARCHAR(1023) NULL,
    imr_priority            INT NULL,
    imr_endpointtype        VARCHAR(255) NULL,
    imr_endpointname        VARCHAR(1023) NULL,
    imr_container           VARCHAR(255) NULL,
    imr_targetuserstatus    VARCHAR(255) NULL,
    imr_action              VARCHAR(255) NULL
);

ALTER TABLE imrreversesyncnewaccount125 ADD
    CONSTRAINT PK_01_imrreversesyncnew125 PRIMARY KEY (
        imr_reportid,
        imr_policyid
);


--
-- IMR imremailmanagement125
--
CREATE TABLE imremailmanagement125 (
    imr_reportid    INT NOT NULL,
    imr_emailid     VARCHAR(64) NOT NULL,
    imr_emailname   VARCHAR(255) NULL,
    imr_enabled     VARCHAR(5) NULL,
    imr_category    VARCHAR(255) NULL,
    imr_description VARCHAR(1023) NULL,
    imr_subject     VARCHAR(1023) NULL
);

ALTER TABLE imremailmanagement125 ADD
    CONSTRAINT PK_01_imremailmanagement125 PRIMARY KEY (
        imr_reportid,
        imr_emailid
);


--
-- IMR imrbulktasks125
--
CREATE TABLE imrbulktasks125 (
    imr_reportid        INT NOT NULL,
    imr_bulktaskid      VARCHAR(64) NOT NULL,
    imr_bulktaskname    VARCHAR(255) NULL,
    imr_description     VARCHAR(1023) NULL,
    imr_tasktag         VARCHAR(255) NULL,
    imr_objecttype      VARCHAR(255) NULL,
    imr_initiator       VARCHAR(255) NULL,
    imr_dateformat      VARCHAR(255) NULL,
    imr_populationrule  LONGTEXT NULL,
    imr_datefilter      LONGTEXT NULL
);

ALTER TABLE imrbulktasks125 ADD
    CONSTRAINT PK_01_imrbulktasks125 PRIMARY KEY (
        imr_reportid,
        imr_bulktaskid
);


------------------------------------------------------
-- End of r12.5 SP1 support.
------------------------------------------------------

------------------------------------------------------
-- New r12.5 SP7 support.
------------------------------------------------------

CREATE TABLE imrNonStandardAccts(
    imr_reportid        INT NOT NULL,
    imr_endpointtype    VARCHAR(64) NULL,
    imr_endpoint        VARCHAR(64) NULL,
    imr_accounttype     VARCHAR(628) NULL,
    imr_totalcount      INT NULL
);

CREATE INDEX imrNonStandardAccts_index_1 ON imrNonStandardAccts (imr_reportid, imr_endpointtype, imr_endpoint);


CREATE OR REPLACE VIEW exceptionaccountsview
AS
    SELECT
        accountObject.imr_reportid AS imr_reportid,
        accountObject.imr_objectid AS imr_accountid,
        accountObject.imr_friendlyname AS imr_accountname,
        accountObject.imr_objectdn AS imr_accountdn,
        (SELECT
            imr_value
        FROM
            imrAttribute endPointAttr,
            imrAttributeInfo endPointAttrInfo
        WHERE
            accountObject.imr_objectid = endPointAttr.imr_objectid AND
            endPointAttr.imr_attrinfoid = endPointAttrInfo.imr_attrinfoid AND
            endPointAttrInfo.imr_name = 'endPointName') AS imr_endpoint,
            accountObjectInfo.imr_name AS imr_endpointtype,
            attr1.imr_value AS imr_createtime,
            attr2.imr_value AS imr_accountlastmodified,
            attr3.imr_value AS exceptionaccount
    FROM
        imrObjectCategory accountCategory,
        imrObjectInfo accountObjectInfo,
        imrObject accountObject,
        imrAttributeInfo attrInfo1,
        imrAttribute attr1,
        imrAttributeInfo attrInfo2,
        imrAttribute attr2,
        imrAttributeInfo attrInfo3,
        imrAttribute attr3
    WHERE
        (attr1.imr_value IS NOT NULL OR
        attr1.imr_objectrefid IS NOT NULL OR
        attr2.imr_value IS NOT NULL OR
        attr2.imr_objectrefid IS NOT NULL OR
        attr3.imr_value IS NOT NULL OR
        attr3.imr_objectrefid IS NOT NULL) AND
        attr1.imr_objectid = accountObject.imr_objectid AND
        attr2.imr_objectid = accountObject.imr_objectid AND
        attr3.imr_objectid = accountObject.imr_objectid AND
        accountObject.imr_objectinfoname = accountObjectInfo.imr_name AND
        accountObjectInfo.imr_objectcatname = accountCategory.imr_name AND
        accountCategory.imr_name = 'Account' AND
        (attr1.imr_attrinfoid = attrInfo1.imr_attrinfoid AND
        attrInfo1.imr_name = 'createDate') AND
        (attr2.imr_attrinfoid = attrInfo2.imr_attrinfoid AND
        attrInfo2.imr_name = 'updateDate') AND
        (attr3.imr_attrinfoid = attrInfo3.imr_attrinfoid AND
        attrInfo3.imr_name = 'exceptionAccount' AND attr3.imr_value = 'true') AND
        accountCategory.imr_reportid=accountObject.imr_reportid AND
        accountObjectInfo.imr_reportid=accountObject.imr_reportid AND
        attrInfo1.imr_reportid=accountObject.imr_reportid AND
        attr1.imr_reportid=accountObject.imr_reportid AND
        attrInfo2.imr_reportid=accountObject.imr_reportid AND
        attr2.imr_reportid=accountObject.imr_reportid AND
        attrInfo3.imr_reportid=accountObject.imr_reportid AND
        attr3.imr_reportid=accountObject.imr_reportid;


CREATE OR REPLACE VIEW endpointexceptionuseraccounts6
AS
    SELECT
        a.imr_reportid imr_reportid,
        b.imr_userid imr_userid,
        a.imr_accountname imr_accountname,
        a.imr_accountdn imr_accountdn,
        a.imr_endpoint AS imr_endpoint,
        a.imr_endpointtype imr_accounttype,
        a.imr_createtime imr_createtime,
        a.imr_accountlastmodified imr_accountlastmodified,
        c.imr_firstname imr_firstname,
        c.imr_lastname imr_lastname,
        c.imr_fullname imr_fullname
    FROM
        exceptionaccountsview a,
        imrUserAccountRelationship b,
        imruser6 c
    WHERE
        b.imr_useraccountdn = a.imr_accountdn AND
        b.imr_reportid = a.imr_reportid AND
        c.imr_reportid = a.imr_reportid AND
        c.imr_userid=b.imr_userid;


CREATE OR REPLACE VIEW totalExceptionAccounts
AS
    SELECT
        imr_reportid AS REPORT_ID,
        imr_accounttype AS END_POINT_TYPE,
        imr_endpoint AS END_POINT,
        'Exception' AS EXCEPTION_TYPE,
        imr_accountname AS ACCOUNT_NAME,
        imr_createtime AS CREATION_DATE,
        imr_accountlastmodified AS LAST_MODIFIED
    FROM
        endpointexceptionuseraccounts6
    WHERE
        imr_userid NOT IN ( '[default user]','[system user]')
        UNION
            -- ADD ORPHAN ACCOUNTS
            SELECT
                imr_reportid AS REPORT_ID,
                imr_accounttype AS END_POINT_TYPE,
                imr_endpoint AS END_POINT,
                'Orphan' AS EXCEPTION_TYPE,
                imr_accountname AS ACCOUNT_NAME,
                imr_createtime AS CREATION_DATE,
                imr_accountlastmodified AS LAST_MODIFIED
            FROM
                endpointuseraccounts6
            WHERE
                imr_userid IN('[default user]')
                UNION
                    -- ADD SYSTEM ACCOUNTS
                    SELECT
                        imr_reportid AS REPORT_ID,
                        imr_accounttype AS END_POINT_TYPE,
                        imr_endpoint AS END_POINT,
                        'System' AS EXCEPTION_TYPE,
                        imr_accountname AS ACCOUNT_NAME,
                        imr_createtime AS CREATION_DATE,
                        imr_accountlastmodified AS LAST_MODIFIED
                    FROM
                        endpointuseraccounts6
                    WHERE
                        imr_userid  IN('[system user]');


------------------------------------------------------
-- End of r12.5 SP7 support.
------------------------------------------------------
----------
-- Cleanup Stuff
----------


--
-- PROCEDURE IMRCleanupProc6 For cleaning reports older than a month
-- *********DEPRICATED DO NOT USE
--
DROP PROCEDURE IF EXISTS IMRCleanupProc6;


--
-- Procedure for removing a snapshot from all the tables it exist
--
DROP PROCEDURE IF EXISTS IMRRemoveSnapshot;

DELIMITER $$

CREATE PROCEDURE IMRRemoveSnapshot (
    id INT)
BEGIN
    DELETE FROM imrrole6 WHERE imr_reportid = id;
    DELETE FROM imrrolemember6 WHERE imr_reportid = id;
    DELETE FROM imrroleadmin6 WHERE imr_reportid = id;
    DELETE FROM imrroleowner6 WHERE imr_reportid = id;
    DELETE FROM imrrolerule6 WHERE imr_reportid = id;
    DELETE FROM imrrolescoperule6 WHERE imr_reportid = id;

    DELETE FROM imrroletask6 WHERE imr_reportid = id;
    DELETE FROM imrtask6 WHERE imr_reportid = id;

    DELETE FROM imrorg6 WHERE imr_reportid = id;
    DELETE FROM imrorgattr6 WHERE imr_reportid = id;

    DELETE FROM imruser6 WHERE imr_reportid = id;
    DELETE FROM imruserattr6 WHERE imr_reportid = id;

    DELETE FROM imrgroup6 WHERE imr_reportid = id;
    DELETE FROM imrgroupattr6 WHERE imr_reportid = id;
    DELETE FROM imrgroupmember6 WHERE imr_reportid = id;
    DELETE FROM imrgroupadmin6 WHERE imr_reportid = id;

    DELETE FROM imridentitypolicy6 WHERE imr_reportid = id;
    DELETE FROM imridentitypolicyset6 WHERE imr_reportid = id;
    DELETE FROM imridentitypolicyusersync6 WHERE imr_reportid = id;
    DELETE FROM imridentitypolicyactions6 WHERE imr_reportid = id;

    DELETE FROM imrAttribute WHERE imr_reportid = id;
    DELETE FROM imrAttributeInfo WHERE imr_reportid = id;
    DELETE FROM imrObject WHERE imr_reportid = id;
    DELETE FROM imrObjectCategory WHERE imr_reportid = id;
    DELETE FROM imrObjectInfo WHERE imr_reportid = id;
    DELETE FROM imrProvRolesRelationship WHERE imr_reportid = id;
    DELETE FROM imrProvRoleTemplRelationship WHERE imr_reportid = id;
    DELETE FROM imrUserAccountRelationship WHERE imr_reportid = id;
    DELETE FROM imrTemplEndpointRelationship WHERE imr_reportid = id;
    DELETE FROM imrAccountTemplateRel WHERE imr_reportid = id;
    DELETE FROM imrCapabilityAttrRel WHERE imr_reportid = id;
    DELETE FROM imrroleattr12 WHERE imr_reportid = id;

    -- New 12.5 SP1 suport
    DELETE FROM imrpolicyxpress125 WHERE imr_reportid = id;
    DELETE FROM imrreversesyncnewaccount125 WHERE imr_reportid = id;
    DELETE FROM imrreversesyncmodifyaccount125 WHERE imr_reportid = id;
    DELETE FROM imremailmanagement125 WHERE imr_reportid = id;
    DELETE FROM imrbulktasks125 WHERE imr_reportid = id;
    -- End of 12.5 SP1 suport

    DELETE FROM imreport6 WHERE imr_reportid = id;
END$$


--
-- Procedure for keeping only top X snapshots that we want, and remove the rest.
--

DROP PROCEDURE IF EXISTS IMRCleanOldSnapshotsProc$$

CREATE PROCEDURE IMRCleanOldSnapshotsProc (
    IN keep    INT,
    IN name    VARCHAR(255),
    IN source  VARCHAR(255))
BEGIN
    -- make temporary table to keep the snapshots ids we want to clean
    CREATE TEMPORARY TABLE IF NOT EXISTS retired_snapshots (id INT);
    TRUNCATE TABLE retired_snapshots;

    -- In the MySQL DECLARE CURSOR statement the FOR SELECT... bit is not optional
    -- so you can't SET the cursor later as you can in MS SQL. This means you must
    -- create the temp table in an outer block and the cursor in an inner block.
    BEGIN
        DECLARE reportId INT;
        DECLARE done INT DEFAULT 0;
        DECLARE runCursor CURSOR FOR SELECT id FROM retired_snapshots;
        DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

        -- I couldn't get this statement to work across multiple lines so I've
        -- split it up using variables which I then concatenate. Strange.
        SET @SQLString1 = 'INSERT INTO retired_snapshots (SELECT imr_reportid FROM imreport6 WHERE imr_reportname=? ';
        SET @SQLString2 = 'AND imr_snapshotsource=? AND imr_snapshotstatus = ''Completed '' AND imr_reportid NOT IN ';
        SET @SQLString3 = '(SELECT * FROM (SELECT imr_reportid FROM imreport6 WHERE imr_reportname=? ';
        -- ERROR 1235 (42000): This version of MySQL doesn't yet support 'LIMIT & IN/ALL/ANY/SOME subquery'
        -- A work around is to use an anonymous 'alias' for the subquery that includes the LIMIT...
        SET @SQLString4 = 'AND imr_snapshotsource=? AND imr_snapshotstatus= ''Completed '' ORDER BY imr_reportid DESC LIMIT ?) alias))';
        SET @SQLString = CONCAT(@SQLString1,@SQLString2,@SQLString3,@SQLString4);
        
        PREPARE retireSnapshotIds FROM @SQLString;
        SET @n = name;
        SET @s = source;
        SET @k = keep;
        EXECUTE retireSnapshotIds USING @n, @s, @n, @s, @k;

        OPEN runCursor;
        run_loop: LOOP
            FETCH runCursor INTO reportId;
            IF done THEN
                LEAVE run_loop;  -- exit loop if condition is true
            END IF;
            CALL IMRRemoveSnapshot(@reportId);
        END LOOP;
        CLOSE runCursor;

        DEALLOCATE PREPARE retireSnapshotIds;
    END;

    DROP TEMPORARY TABLE retired_snapshots;
END$$

DELIMITER ;

