-- Copyright (C) 2009, CA, Inc. All rights reserved.
--
-- This file contains the schema for all of the 12.5 SP1
-- tables for snapshot
--
--
-------------------------------------------------------------------

-- Add the Version table
CREATE TABLE imreport_version (
    imr_key varchar2(255) NOT NULL,
    imr_value varchar2(255) NOT NULL
    );

insert into imreport_version(imr_key,imr_value) values ('version','12.6.2.0.0');

--
-- SEQUENCE SEQ_IDreport For ims_reportid generation
--
create sequence SEQ_IDREPORT     increment by 1  start with 1 ;


--
-- IM Report6
--
CREATE TABLE imreport6 (
	imr_reportid INTEGER  NOT NULL,
	imr_reportname VARCHAR2(255) NOT NULL,
	imr_reportdescription VARCHAR2(628) NULL,
  	imr_reporttime DATE DEFAULT SYSDATE NOT NULL,
  	imr_reportuser  VARCHAR2(628) NOT NULL,
  	imr_envoid     	VARCHAR2(64) NOT NULL,
  	imr_envname  	VARCHAR2(255) NULL,
  	imr_keepsnapshot INTEGER ,
  	imr_snapshotsource VARCHAR2 (255),
  	imr_snapshotstatus VARCHAR2 (64),
  	CONSTRAINT PK_01_IMREPORT6 PRIMARY KEY  (imr_reportid)
);

CREATE INDEX imreport6_index_1 ON imreport6 (imr_reporttime);

CREATE INDEX imreport6_index_2 ON imreport6 (imr_reportid, imr_envoid);

CREATE INDEX imreport6_index_3 ON imreport6 (imr_reportid, imr_envname);

--
-- SP to insert into the report table
--
CREATE OR REPLACE PROCEDURE insertintoimreport (
    reportname  VARCHAR2 ,
    reportdescription  VARCHAR2 ,
    reportuser  VARCHAR2 ,
    envoid      VARCHAR2 ,
    envname     VARCHAR2 ,
	keepsnapshot INTEGER ,
	snapshotsource VARCHAR2 ,
	snapshotstatus VARCHAR2 ,
    current_id      OUT INTEGER)
AS
BEGIN
SELECT SEQ_IDREPORT.NEXTVAL
    INTO   current_id
    FROM   Dual;

 INSERT INTO imreport6 (
	 	imr_reportid,
        imr_reportname ,
        imr_reportdescription,
        imr_reportuser ,
        imr_envoid ,
        imr_envname ,
		imr_keepsnapshot ,
		imr_snapshotsource ,
		imr_snapshotstatus)
            VALUES (
        current_id ,
        reportname ,
        reportdescription ,
        reportuser ,
        envoid ,
        envname ,
		keepsnapshot ,
		snapshotsource ,
		snapshotstatus);
RETURN;
END;
/

--
-- IMR ROLE6
--
CREATE TABLE imrrole6 (
	imr_reportid INTEGER NOT NULL,
	imr_oid    VARCHAR2(64) NOT NULL,
	imr_name   VARCHAR2(255)NULL,
	imr_description   VARCHAR2(255) NULL,
	imr_type  VARCHAR2(64) NOT NULL,
        imr_isenabled  INTEGER  NULL,
        imr_isassignable INTEGER NULL,
        imr_isadminassignable INTEGER NULL,
  	CONSTRAINT PK_01_IMRROLE6 PRIMARY KEY  (imr_reportid,imr_oid)
);

CREATE INDEX imrrole6_index_1 ON imrrole6 (imr_reportid,imr_name);

-- IMR ROLE RULES 6
-- member, administrator and owner rules
-- add and remove actions
--
CREATE TABLE imrrolerule6 (
    imr_reportid INTEGER  NOT NULL,
    imr_roleoid    VARCHAR2(64) NOT NULL,
    imr_ruleorder  INTEGER DEFAULT (0) NOT NULL,
    imr_ruletype   VARCHAR2(64) NULL,
    imr_constraintvalue VARCHAR2(3800) NULL,
    CONSTRAINT PK_01_IMRROLERULE6 PRIMARY KEY  (imr_reportid,imr_roleoid,imr_ruleorder)
);

CREATE INDEX imrrolerule6_index_1 ON imrrolerule6 (imr_reportid,imr_roleoid);

--
-- IMR ROLE SCOPE RULES 6
-- scope rules associated with role rules (admin, member and owner rules)
--
CREATE TABLE imrrolescoperule6 (
    imr_reportid INTEGER  NOT NULL,
    imr_roleoid    VARCHAR2(64) NOT NULL,
    imr_roleruleorder  INTEGER DEFAULT (0) NOT NULL,
    imr_scopeobjecttype   VARCHAR2(64) NOT NULL,
    imr_constraintvalue VARCHAR2(3800) NULL,
    CONSTRAINT PK_01_IMRROLESCOPERULE6 PRIMARY KEY  (imr_reportid,imr_roleoid,imr_roleruleorder,imr_scopeobjecttype)
);

CREATE INDEX imrrolescoperule6_index_1 ON imrrolescoperule6 (imr_reportid,imr_roleoid,imr_roleruleorder);


--
-- IMR ROLE TASK 6
--
CREATE TABLE imrroletask6 (
	imr_reportid INTEGER NOT NULL,
	imr_roleoid    VARCHAR2(64) NOT NULL,
  	imr_taskoid   VARCHAR2(64) NOT NULL,
  	CONSTRAINT PK_01_IMRROLETASK6 PRIMARY KEY  (imr_reportid,imr_roleoid,imr_taskoid)
);

CREATE INDEX imrroletask6_index_1 ON imrroletask6 (imr_reportid,imr_roleoid);


--
-- IMR TASK 6
--
CREATE TABLE imrtask6 (
    imr_reportid INTEGER NOT NULL,
    imr_oid    VARCHAR2(64) NOT NULL,
    imr_name   VARCHAR2(255) NULL,
    imr_description   VARCHAR2(255) NULL,
    imr_type   VARCHAR2(64) NOT NULL,
    imr_category VARCHAR2(255)  NULL,
    imr_workflowprocess VARCHAR2(255) NULL,
    imr_application   VARCHAR2(255) NULL,
    imr_externalurl VARCHAR2(255) NULL,
    imr_tasktag VARCHAR2(255) NULL,
    imr_reserved1 VARCHAR2(255) NULL,
    imr_reserved2 VARCHAR2(255) NULL,
    imr_reserved3 VARCHAR2(255) NULL,
    imr_reserved4 VARCHAR2(255) NULL,
    imr_reserved5 VARCHAR2(255) NULL,
    imr_reserved6 VARCHAR2(255) NULL,
    imr_reserved7 VARCHAR2(255) NULL,
    imr_reserved8 VARCHAR2(255) NULL,
    imr_reserved9 VARCHAR2(255) NULL,
    imr_reserved10 VARCHAR2(255) NULL,
    CONSTRAINT PK_01_IMRTASK6 PRIMARY KEY (imr_reportid,imr_oid)
);


--
-- IMR ROLE MEMBER 6
--
CREATE TABLE imrrolemember6 (
    imr_reportid INTEGER NOT NULL,
    imr_roleoid VARCHAR2(64) NOT NULL,
    imr_userdn VARCHAR2(628) NOT NULL
);

CREATE INDEX imrrolemember6_index_1 ON imrrolemember6 (imr_reportid,imr_roleoid);

CREATE INDEX imrrolemember6_index_2 ON imrrolemember6 (imr_reportid,imr_userdn);


--
-- IMR ROLEADMIN6
--
CREATE TABLE imrroleadmin6 (
    imr_reportid integer NOT NULL,
    imr_roleoid   VARCHAR2(64) NOT NULL,
    imr_userdn   VARCHAR2(628) NOT NULL
 );

CREATE INDEX imrroleadmin6_index_1 ON imrroleadmin6 (imr_reportid,imr_roleoid);

CREATE INDEX imrroleadmin6_index_2 ON imrroleadmin6 (imr_reportid,imr_userdn);

--
-- IMR ROLEOWNER6
--
CREATE TABLE imrroleowner6 (
    imr_reportid INTEGER NOT NULL,
    imr_roleoid   VARCHAR2(64) NOT NULL,
    imr_userdn   VARCHAR2(628) NOT NULL
);

CREATE INDEX imrroleowner6_index_1 ON imrroleowner6 (imr_reportid,imr_roleoid);

CREATE INDEX imrroleowner6_index_2 ON imrroleowner6 (imr_reportid,imr_userdn);

--
-- SP to insert members into role tables
--

CREATE OR REPLACE PROCEDURE insertusermembertable  (
    tablename VARCHAR,
    reportid int ,
    roleoid  VARCHAR2 ,
    userdn   VARCHAR2  )
IS
    rowcount INTEGER;
BEGIN
    rowcount :=0;
    BEGIN
        IF (tablename = 'imrrolemember6') THEN
            Select count(*) into rowcount from imrrolemember6
            WHERE
                imr_reportid= reportid
                AND imr_roleoid= roleoid
                AND imr_userdn = userdn;

        ELSIF (tablename = 'imrroleadmin6') THEN

            Select count(*) into rowcount from imrroleadmin6
                  WHERE
                    imr_reportid= reportid
                    AND imr_roleoid= roleoid
                    AND imr_userdn = userdn;

        ELSIF (tablename = 'imrroleowner6') then
            Select count(*) into rowcount from imrroleowner6
                WHERE
                    imr_reportid= reportid
                    AND imr_roleoid= roleoid
                    AND imr_userdn = userdn;
        END IF;
        --- handle the exceptions.
        EXCEPTION
           WHEN NO_DATA_FOUND THEN
           rowcount :=0;
      END;
    IF rowcount = 0  THEN
        IF (tablename = 'imrrolemember6') THEN
            insert into imrrolemember6 (
                imr_reportid ,
                imr_roleoid ,
                imr_userdn  )
                values (
                reportid ,
                roleoid ,
                userdn );
        ELSIF (tablename = 'imrroleadmin6') THEN

                 insert into imrroleadmin6 (
                    imr_reportid ,
                    imr_roleoid ,
                    imr_userdn  )
                    values (
                    reportid ,
                    roleoid ,
                    userdn );
        ELSIF (tablename = 'imrroleowner6') then
                 insert into imrroleowner6 (
                    imr_reportid ,
                    imr_roleoid ,
                    imr_userdn  )
                    values (
                    reportid ,
                    roleoid ,
                    userdn );
        END IF;
    END IF;
END;
/

--
-- IMR ORG6
--
CREATE TABLE imrorg6 (
	imr_reportid INTEGER NOT NULL,
	imr_orgdn    VARCHAR2(628) NOT NULL,
  	imr_orgname  VARCHAR2(255) NULL,
  	imr_parentorgdn VARCHAR2(628) NULL
);


CREATE INDEX imrorg6_index_1 ON imrorg6 (imr_reportid, imr_parentorgdn);

-- Insert into imrorg6
CREATE OR REPLACE PROCEDURE insertintoorgtable (
    reportid INTEGER ,
    orgdn    VARCHAR2,
    orgname    VARCHAR2,
    parentorgdn  VARCHAR2
    )
IS
    rowcount   INTEGER;
BEGIN
    rowcount :=0;
    BEGIN
        Select count(*) into rowcount from imrorg6
            WHERE
                imr_reportid= reportid
                AND imr_orgdn = orgdn;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            rowcount :=0;
    END;
    IF rowcount = 0  THEN
        insert into imrorg6 (
                imr_reportid ,
                imr_orgdn,
                imr_orgname    ,
                imr_parentorgdn
              )
                    values (
                reportid ,
                orgdn,
                orgname    ,
                parentorgdn  );
    END IF;

END;
/
--
-- IMR OrgAttr6
--

CREATE CLUSTER imrorgattr6_cluster (
	imr_reportid INTEGER,
	imr_orgdn    VARCHAR2(628)
);


CREATE TABLE imrorgattr6 (
	imr_reportid INTEGER NOT NULL,
	imr_orgdn    VARCHAR2(628) NOT NULL,
  	imr_attrname   VARCHAR2(255) NOT NULL,
  	imr_attrvalue   VARCHAR2(628) NULL
)
CLUSTER imrorgattr6_cluster (imr_reportid, imr_orgdn) ;

CREATE INDEX imrorgattr6_index_1 ON  CLUSTER imrorgattr6_cluster;

CREATE INDEX imrorgattr6_index_2 ON imrorgattr6 (imr_reportid, imr_attrname, imr_attrvalue);


--
-- IMR GROUP6
--
CREATE TABLE imrgroup6 (
	imr_reportid INTEGER NOT NULL,
	imr_groupdn    VARCHAR2(628) NOT NULL,
  	imr_name   VARCHAR2(255) NULL,
  	imr_description VARCHAR2(255) NULL,
  	imr_parentorgdn      VARCHAR2(628) NOT NULL,
  	imr_selfsubscribing  VARCHAR2(64) NULL,
  	imr_dynamicgroupmembership VARCHAR2(255) NULL
);
CREATE INDEX imrgroup6_index_1 ON imrgroup6 (imr_reportid, imr_parentorgdn);


--
-- Insert into imrgroup6
--

CREATE OR REPLACE PROCEDURE insertintogrouptable (
    reportid int ,
    groupdn    VARCHAR2 ,
    name   VARCHAR2 ,
    description VARCHAR2 ,
    parentorgdn   VARCHAR2 ,
    selfsubscribing  VARCHAR2,
    dynamicgroupmembership VARCHAR2)
IS
        rowcount   INTEGER;
BEGIN
    BEGIN
      rowcount :=0;
        Select count(*) into rowcount  from imrgroup6
            WHERE
                imr_reportid= reportid
                AND imr_groupdn = groupdn;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            rowcount :=0;
    END;
    IF rowcount = 0  THEN
        insert into imrgroup6 (
            imr_reportid ,
            imr_groupdn  ,
            imr_name   ,
            imr_description ,
            imr_parentorgdn ,
            imr_selfsubscribing  ,
            imr_dynamicgroupmembership

          )
            values (
            reportid ,
            groupdn  ,
            name   ,
            description ,
            parentorgdn ,
            selfsubscribing  ,
            dynamicgroupmembership
            );
      END IF;
END;
/

--
-- IMR GroupAttr6
--
CREATE CLUSTER imrgroupattr6_cluster (
	imr_reportid INTEGER,
	imr_groupdn    VARCHAR2(628)
);


CREATE TABLE imrgroupattr6 (
	imr_reportid INTEGER NOT NULL,
	imr_groupdn    VARCHAR2(628) NOT NULL,
  	imr_attrname   VARCHAR2(255) NOT NULL,
  	imr_attrvalue  VARCHAR2(628) NULL
)
CLUSTER imrgroupattr6_cluster (imr_reportid, imr_groupdn);

CREATE INDEX imrgroupattr6_index_1 ON CLUSTER  imrgroupattr6_cluster;

CREATE INDEX imrgroupattr6_index_2 ON imrgroupattr6 (imr_reportid, imr_attrname, imr_attrvalue);

--
-- IMR GroupMember6
--
CREATE CLUSTER imrgroupmember6_cluster (
	imr_reportid INTEGER,
	imr_groupdn    VARCHAR2(628)
);


CREATE TABLE imrgroupmember6 (
	imr_reportid INTEGER NOT NULL,
	imr_groupdn    VARCHAR2(628) NOT NULL,
  	imr_memberdn   VARCHAR2(628) NOT NULL
)
CLUSTER imrgroupmember6_cluster (imr_reportid,imr_groupdn) ;

CREATE INDEX imrgroupmember6_index_1 ON  CLUSTER imrgroupmember6_cluster;

CREATE INDEX imrgroupmember6_index_2 ON imrgroupmember6 (imr_reportid, imr_memberdn);
--
-- IMR GroupAdmin6
--
CREATE CLUSTER imrgroupadmin6_cluster (
	imr_reportid INTEGER,
	imr_groupdn    VARCHAR2(628)
);


CREATE TABLE imrgroupadmin6 (
	imr_reportid INTEGER NOT NULL,
	imr_groupdn    VARCHAR2(628) NOT NULL,
  	imr_admindn   VARCHAR2(628) NOT NULL
)
CLUSTER imrgroupadmin6_cluster (imr_reportid,imr_groupdn);

CREATE INDEX imrgroupadmin6_index_1 ON CLUSTER imrgroupadmin6_cluster ;

CREATE INDEX imrgroupadmin6_index_2 ON imrgroupadmin6 (imr_reportid, imr_admindn);
--
-- IMR User 6
--

CREATE TABLE imruser6 (
    imr_reportid    INTEGER NOT NULL,
    imr_userdn      VARCHAR2(628) NOT NULL,
    imr_isenabled   INTEGER DEFAULT(1) NULL,
    imr_userid      VARCHAR2(255) NULL,
    imr_fullname    VARCHAR2(255) NULL,
    imr_lastname    VARCHAR2(255) NULL,
    imr_firstname   VARCHAR2(255) NULL,
    imr_parentorgdn VARCHAR2(628) NULL,
    imr_email       VARCHAR2(255) NULL,
	imr_creationmode integer,
	imr_createdby varchar2(255),
	imr_creationdate timestamp,
	imr_modificationdate timestamp,
	imr_lastlogindate timestamp,
	imr_usertype varchar2(255)
);

CREATE INDEX imruser6_index_1 ON imruser6 (imr_reportid, imr_parentorgdn);

CREATE INDEX imruser6_index_2 ON imruser6 (imr_reportid, imr_userid);

CREATE INDEX imruser6_index_3 ON imruser6 (imr_reportid, imr_userdn);

--
-- Insert into user table
--
CREATE OR REPLACE PROCEDURE insertintousertable  (
    reportid INTEGER ,
    userdn    VARCHAR2,
    isenabled INTEGER ,
    userid    VARCHAR2,
    fullname  VARCHAR2,
    lastname  VARCHAR2 ,
    firstname VARCHAR2,
    parentorgdn VARCHAR2 ,
    email       VARCHAR2,
	creationmode INTEGER,
	createdby VARCHAR2,
	creationdate DATE,
	modificationdate DATE,
	lastlogindate DATE,
	usertype VARCHAR2)
IS
    rowcount   INTEGER;
BEGIN
  BEGIN
      rowcount :=0;
        Select count(*) into rowcount from imruser6
            WHERE
                imruser6.imr_reportid= reportid
                AND imruser6.imr_userdn = userdn;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        rowcount :=0;
  END;
  IF rowcount = 0  THEN
     insert into imruser6 (
          imr_reportid ,
          imr_userdn   ,
          imr_isenabled,
          imr_userid   ,
          imr_fullname ,
          imr_lastname ,
          imr_firstname,
          imr_parentorgdn ,
          imr_email,
		  imr_creationmode,
		  imr_createdby,
		  imr_creationdate,
		  imr_modificationdate,
		  imr_lastlogindate,
		  imr_usertype)
                values (
           reportid ,
           userdn   ,
           isenabled,
           userid   ,
           fullname ,
           lastname ,
           firstname,
           parentorgdn ,
           email,
		   creationmode,
		   createdby,
		   creationdate,
		   modificationdate,
		   lastlogindate,
		   usertype);
  END IF;
  END;
/

--
-- IMR userattr6
--
CREATE CLUSTER imruserattr6_cluster (
	imr_reportid INTEGER,
	imr_userdn    VARCHAR2(628)
);


CREATE TABLE imruserattr6 (
    imr_reportid INTEGER NOT NULL,
    imr_userdn    VARCHAR2(628) NOT NULL,
    imr_attrname   VARCHAR2(255) NOT NULL,
    imr_attrvalue  VARCHAR2(628) NULL
)
CLUSTER imruserattr6_cluster (imr_reportid,imr_userdn) ;

CREATE INDEX imruserattr6_index_1 ON CLUSTER imruserattr6_cluster ;

CREATE  INDEX imruserattr6_index_2 ON imruserattr6 (imr_reportid, imr_attrname, imr_userdn);



-- IM imridentitypolicy6
--
CREATE TABLE imridentitypolicy6 (
	imr_reportid INTEGER  NOT NULL,
  	imr_policyid  VARCHAR2(64) NOT NULL,
	imr_policyname VARCHAR2(255) NOT NULL,
	imr_policyindex INTEGER  NOT NULL,
  	imr_apptype  VARCHAR2(64) NOT NULL,
  	imr_compliance      VARCHAR2(64) NOT NULL,
  	imr_ruleevaluationtype  VARCHAR2(64) NOT NULL,
        imr_rule VARCHAR2(3000) NOT NULL,
 	imr_policysetid  VARCHAR2(64) NOT NULL,
  	CONSTRAINT PK_01_imridentitypolicy6 PRIMARY KEY  (imr_reportid, imr_policyid, imr_policysetid)
);

CREATE INDEX imridentitypolicy6_index_1 ON imridentitypolicy6 (imr_reportid, imr_policyid);

CREATE INDEX imridentitypolicy6_index_2 ON imridentitypolicy6 (imr_reportid, imr_policysetid);

--
-- IMR imridentitypolicyset6
--
CREATE TABLE imridentitypolicyset6 (
	imr_reportid INTEGER  NOT NULL,
  	imr_policysetid  VARCHAR2(64) NOT NULL,
	imr_policysetname VARCHAR2(255) NOT NULL,
	imr_category VARCHAR2(255) NOT NULL,
  	CONSTRAINT PK_01_imridentitypolicyset6 PRIMARY KEY  (imr_reportid, imr_policysetid)
);

--
-- IM imridentitypolicyactions6
--
CREATE TABLE imridentitypolicyactions6 (
	imr_reportid INTEGER  NOT NULL,
  	imr_policyid  VARCHAR2(64) NOT NULL,
  	imr_actiontype  VARCHAR2(64) NOT NULL,
	imr_actionindex INTEGER  NOT NULL,
        imr_actionevaluationtype  VARCHAR2(64) NOT NULL,
        imr_action VARCHAR2(3000) NOT NULL,
  	CONSTRAINT PK_01_imrpolicyactions6 PRIMARY KEY  (imr_reportid, imr_policyid, imr_actiontype, imr_actionindex)
);

CREATE INDEX imrpolicyactions6_index_1 ON imridentitypolicyactions6 (imr_reportid, imr_policyid);


--
-- IM imridentitypolicyusersync6
--
CREATE TABLE imridentitypolicyusersync6 (
	imr_reportid INTEGER  NOT NULL,
	imr_policysetname VARCHAR2(255) NOT NULL,
  	imr_policyid  VARCHAR2(64) NOT NULL,
	imr_userdn VARCHAR2(255) NOT NULL,
	imr_syncstatus VARCHAR2(255) NOT NULL,
  	CONSTRAINT PK_01_imrpolicyusersync6 PRIMARY KEY  (imr_reportid, imr_policyid, imr_userdn)
);

CREATE INDEX imrpolicyusersync6_index_1 ON imridentitypolicyusersync6 (imr_reportid, imr_userdn);

CREATE INDEX imrpolicyusersync6_index_2 ON imridentitypolicyusersync6 (imr_reportid, imr_policyid);


CREATE VIEW imrroleinfo6
AS
SELECT                imrrole6.imr_reportid, imrrole6.imr_oid, imrrole6.imr_name, imrrole6.imr_type,
                      imruser6.imr_userdn, imrrolemember6.imr_userdn AS member, imrroleowner6.imr_userdn AS owner,
                      imrroleadmin6.imr_userdn AS admin
FROM         imrrole6 INNER JOIN
                      imruser6 ON imrrole6.imr_reportid = imruser6.imr_reportid AND
                      imrrole6.imr_reportid = imruser6.imr_reportid LEFT OUTER JOIN
                      imrroleowner6 ON imrrole6.imr_reportid = imrroleowner6.imr_reportid AND
                      imrroleowner6.imr_userdn = imruser6.imr_userdn AND imrroleowner6.imr_reportid = imruser6.imr_reportid AND
                      imrroleowner6.imr_roleoid = imrrole6.imr_oid LEFT OUTER JOIN
                      imrrolemember6 ON imrrole6.imr_reportid = imrrolemember6.imr_reportid AND
                      imrrolemember6.imr_userdn = imruser6.imr_userdn AND
                      imrrolemember6.imr_reportid = imruser6.imr_reportid AND
                      imrrolemember6.imr_roleoid = imrrole6.imr_oid LEFT OUTER JOIN
                      imrroleadmin6 ON imrrole6.imr_reportid = imrroleowner6.imr_reportid AND
                      imrroleadmin6.imr_userdn = imruser6.imr_userdn AND imrroleadmin6.imr_reportid = imruser6.imr_reportid AND
                      imrroleadmin6.imr_roleoid = imrrole6.imr_oid
WHERE     (NOT (imrrolemember6.imr_userdn IS NULL AND imrroleowner6.imr_userdn IS NULL AND
                      imrroleadmin6.imr_userdn IS NULL));
/

--
-- IM imrptsnapshots12
--
CREATE TABLE imrptsnapshots12 (
	imr_rptname 			VARCHAR2(255) NOT NULL,
	imr_snapshotname    		VARCHAR2(255) NOT NULL,
	imr_envname   		        VARCHAR2(255) NOT NULL,
	IMR_RPT_TASK_TAG_NAME VARCHAR2(255) NOT NULL
);

CREATE INDEX imrptsnapshots12_index_1 ON imrptsnapshots12 (imr_rptname,imr_snapshotname,imr_envname);


------------------------------------------------------
-- New r12.5 support.
------------------------------------------------------

CREATE TABLE imrAttribute (
	imr_reportid INTEGER NOT NULL ,
	imr_objectid INTEGER NOT NULL ,
	imr_attrinfoid INTEGER NOT NULL ,
	imr_value varchar2(4000)  ,
	imr_objectrefid INTEGER
);

CREATE INDEX imrattribute_index_1 ON imrattribute (imr_reportid, imr_attrinfoid, imr_objectid);

CREATE INDEX imrattribute_index_2 ON imrattribute (imr_reportid,imr_objectid);

CREATE INDEX imrattribute_index_3 ON imrattribute (imr_reportid,imr_value);

CREATE TABLE imrAttributeInfo (
	imr_reportid INTEGER NOT NULL ,
	imr_attrinfoid INTEGER  NOT NULL ,
	imr_syntaxid INTEGER NOT NULL ,
	imr_name varchar2(64) NOT NULL ,
	imr_objectinfoid INTEGER NOT NULL ,
	imr_isbasetype varchar2(256) NULL
);

CREATE INDEX imrattributeinfo_index_1 ON imrattributeinfo (imr_reportid, imr_name);

CREATE TABLE imrObject (
	imr_reportid INTEGER NOT NULL ,
	imr_friendlyname VARCHAR2 (255) NOT NULL ,
	imr_objectdn VARCHAR2 (628) NOT NULL ,
	imr_objectinfoname VARCHAR2 (255) NOT NULL ,
	imr_objectid INTEGER  NOT NULL
);

CREATE INDEX imrobject_index_1 ON imrobject (imr_reportid, imr_friendlyname);

CREATE INDEX imrobject_index_2 ON imrobject (imr_reportid, imr_objectdn);

CREATE TABLE imrObjectCategory (
	imr_reportid INTEGER NOT NULL ,
	imr_name VARCHAR2 (255) NOT NULL
);

CREATE INDEX imrobjectcategory_index_1 ON imrobjectcategory (imr_reportid, imr_name);

CREATE TABLE imrObjectInfo (
	imr_reportid INTEGER NOT NULL ,
	imr_objectid INTEGER  NOT NULL ,
	imr_name VARCHAR2 (255) NOT NULL ,
	imr_objectcatname VARCHAR2 (255) NOT NULL
);

CREATE INDEX imrobjectinfo_index_1 ON imrobjectinfo (imr_reportid, imr_name);

CREATE TABLE imrProvRolesRelationship (
	imr_reportid INTEGER NOT NULL ,
	imr_parentoid VARCHAR2 (64) NOT NULL ,
	imr_childoid VARCHAR2 (64) NOT NULL
);

CREATE INDEX imrprovrolesrel_index_1 ON imrProvRolesRelationship (imr_reportid, imr_parentoid, imr_childoid);

CREATE TABLE imrProvRoleTemplRelationship (
	imr_reportid INTEGER NOT NULL ,
	imr_roleoid VARCHAR2(64) NOT NULL,
	imr_roledn VARCHAR2 (628) NOT NULL ,
	imr_templatedn VARCHAR2 (628) NOT NULL
);

CREATE INDEX imrprovroletemplrel_index_1 ON imrprovroletemplrelationship (imr_reportid, imr_roleoid, imr_templatedn, imr_roledn);


CREATE TABLE imrUserAccountRelationship (
	imr_reportid INTEGER NOT NULL ,
	imr_useraccountdn VARCHAR2 (1024) NOT NULL ,
	imr_provuserdn VARCHAR2 (1024) NOT NULL ,
	imr_userid VARCHAR2 (1024) NOT NULL
);

CREATE INDEX imruseraccountrel_index_1 ON imruseraccountrelationship (imr_reportid, imr_userid);

CREATE INDEX imruseraccountrel_index_2 ON imruseraccountrelationship (imr_reportid, imr_provuserdn);

CREATE INDEX imruseraccountrel_index_3 ON imruseraccountrelationship (imr_reportid, imr_useraccountdn);

--
-- IM imrTemplEndpointRelationship
--

CREATE TABLE imrTemplEndpointRelationship (
   	imr_reportid INTEGER  NOT NULL,
	imr_templatedn VARCHAR2(628) ,
	imr_endpointname VARCHAR2(628) ,
	imr_endpointdn VARCHAR2(628)
);

CREATE INDEX imrtemplendpointrel_index_1 ON imrtemplendpointrelationship (imr_reportid, imr_templatedn, imr_endpointname);

--
-- IM imrAccount
--

CREATE TABLE imrAccountTemplateRel (
    imr_reportid INTEGER  NOT NULL,
	imr_name VARCHAR2(628) NOT NULL,
	imr_templatedn VARCHAR2(628),
	imr_endpointtype VARCHAR2(628)
);

CREATE INDEX imrAccountTemplateRel_index_1 ON imrAccountTemplateRel (imr_reportid, imr_name,imr_templatedn,imr_endpointtype);

--
-- IM imrCapabilityAttrRelationship
--

CREATE TABLE imrCapabilityAttrRel (
    imr_reportid integer  NOT NULL,
	imr_endpointtype VARCHAR2(628) not null,
	imr_objecttypename VARCHAR2(628) NOT NULL,
	imr_jiamobjecttypename VARCHAR2(628) NOT NULL,
	imr_attrid integer,
	imr_capabilityobjectname VARCHAR2(628) NOT NULL,
	imr_capabilityobjectdn VARCHAR2(628) NOT NULL,
	imr_endpointobjid integer,
	imr_name VARCHAR2(64),
	imr_value varchar2(4000)
);

CREATE INDEX imrcapabilityattrrel_index_1 ON imrcapabilityattrrel (imr_reportid, imr_endpointtype, imr_endpointobjid, imr_objecttypename, imr_capabilityobjectdn, imr_name);

--
-- IM imrroleattr12
--
CREATE TABLE imrroleattr12 (
	imr_reportid INTEGER NOT NULL ,
	imr_roleoid varchar(64) NOT NULL ,
	imr_name VARCHAR2 (628) NOT NULL ,
	imr_value varchar2(4000) NULL
);

CREATE INDEX imrroleattr12_index_1 ON imrroleattr12 (imr_reportid, imr_roleoid, imr_name);

CREATE OR REPLACE PROCEDURE insertCapabilityAttrRel(
	reportid INTEGER,
	endpointtype VARCHAR2,
	objecttypename VARCHAR2,
	jiamobjecttypename VARCHAR2,
	attrid INTEGER,
	capabilityobjectname VARCHAR2,
	capabilityobjectdn VARCHAR2,
	endpointobjid integer,
	attrname VARCHAR2,
	attrvalue VARCHAR2
	)
IS
	rowcount INTEGER;
BEGIN
	rowcount:=0;
	BEGIN
		select count(*) into rowcount from imrCapabilityAttrRel
			where
				imr_reportid=reportid
				and imr_endpointtype=endpointtype
				and imr_objecttypename=objecttypename
				and imr_endpointtype=endpointtype
				and imr_jiamobjecttypename=jiamobjecttypename
				and imr_attrid=attrid
				and imr_capabilityobjectname=capabilityobjectname
				and imr_capabilityobjectdn=capabilityobjectdn
				and imr_endpointobjid=endpointobjid
				and imr_name=attrname;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
			rowcount:=0;
	END;
	IF rowcount = 0 THEN
		insert into imrCapabilityAttrRel(
			imr_reportid,
			imr_endpointtype,
			imr_objecttypename,
			imr_jiamobjecttypename,
			imr_attrid,
			imr_capabilityobjectname,
			imr_capabilityobjectdn,
			imr_endpointobjid,
			imr_name,
			imr_value
		)
		values(
			reportid,
			endpointtype,
			objecttypename,
			jiamobjecttypename,
			attrid,
			capabilityobjectname,
			capabilityobjectdn,
			endpointobjid,
			attrname,
			attrvalue
		);
	END IF;
END;
/

CREATE OR REPLACE PROCEDURE insertTemplEndpointRel(
	reportid INTEGER,
	templatedn VARCHAR2,
	endpointname VARCHAR2,
	endpointdn VARCHAR2
	)
IS
	rowcount INTEGER;
BEGIN
	rowcount:=0;
	BEGIN
		select count(*) into rowcount from imrTemplEndpointRelationship
			where
				imr_reportid=reportid
				and imr_templatedn=templatedn
				and imr_endpointname=endpointname
				and imr_endpointdn=endpointdn;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
			rowcount:=0;
	END;
	IF rowcount = 0 THEN
		insert into imrTemplEndpointRelationship(
			imr_reportid,
			imr_templatedn,
			imr_endpointname,
			imr_endpointdn
		)
		values(
			reportid,
			templatedn,
			endpointname,
			endpointdn
		);
	END IF;
END;
/

--
-- IM insertAccountTemplateRel
--

CREATE OR REPLACE PROCEDURE insertAccountTemplateRel(
	reportid INTEGER,
	name VARCHAR2,
	templatedn VARCHAR2,
	endpointtype VARCHAR2
	)
IS
	rowcount INTEGER;
BEGIN
	rowcount :=0;
	BEGIN
		select count(*) into rowcount from imrAccountTemplateRel
		where
			imr_reportid=reportid
			and imr_name=name
			and imr_templatedn=templatedn
			and imr_endpointtype=endpointtype;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				rowcount:=0;
	END;
	IF(rowcount = 0) THEN
		insert into imrAccountTemplateRel(
			imr_reportid,
			imr_name,
			imr_templatedn,
			imr_endpointtype
		)
		values(
			reportid,
			name,
			templatedn,
			endpointtype
		);
	END IF;
END;
/

------------------------
-- Views
------------------------


CREATE OR REPLACE VIEW imruseraccounts6
AS
SELECT 	accountObject.imr_reportid as imr_reportid,
	imruser.imr_userid as imr_userid,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn,
	SUBSTR(accountObject.imr_objectdn,
              INSTR(accountObject.imr_objectdn,',EndPoint=') + LENGTH(',EndPoint='),
			  CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0
			  THEN
                  INSTR(accountObject.imr_objectdn,',Namespace=') - INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObject.imr_objectinfoname as imr_accounttype,
	accountAttr1.imr_value as imr_createtime
FROM imrObject accountObject,
	imrUserAccountRelationship userAccountRelationship,
	imrAttributeInfo accountAttrInfo1,
	imrAttribute accountAttr1,
	imruser6 imruser
WHERE imruser.imr_userid = userAccountRelationship.imr_userid and
	accountObject.imr_objectdn = userAccountRelationship.imr_useraccountdn and
	accountAttr1.imr_attrinfoid = accountAttrInfo1.imr_attrinfoid and
	accountAttrInfo1.imr_name='createDate' and
	accountObject.imr_objectid = accountAttr1.imr_objectid and
	accountObject.imr_reportid=imruser.imr_reportid and
	userAccountRelationship.imr_reportid = imruser.imr_reportid and
	accountAttrInfo1.imr_reportid = imruser.imr_reportid and
	accountAttr1.imr_reportid = imruser.imr_reportid;
/

CREATE OR REPLACE VIEW imraccountattrs6
AS
SELECT accountObject.imr_reportid as imr_reportid,
	accountObject.imr_objectid as accountObjid,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn,
    SUBSTR(accountObject.imr_objectdn,
              INSTR(accountObject.imr_objectdn,',EndPoint=') + LENGTH(',EndPoint='),
			  CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0
			  THEN
                  INSTR(accountObject.imr_objectdn,',Namespace=') - INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	resourceAttrInfo.imr_name as imr_accountattr,
	resourceAttr.imr_value as imr_accountattrvalue,
	resourceAttr.imr_objectrefid as imr_objectrefid,
	resourceAttrInfo.imr_isbasetype as imr_isbasetype
FROM
	imrObjectCategory accountCategory,
	imrObjectInfo accountObjectInfo,
	imrObject accountObject,
	imrAttributeInfo resourceAttrInfo,
	imrAttribute resourceAttr
WHERE
	(resourceAttr.imr_value IS NOT NULL or
	resourceAttr.imr_objectrefid IS NOT NULL) and
	resourceAttr.imr_objectid = accountObject.imr_objectid and
	resourceAttrInfo.imr_attrinfoid = resourceAttr.imr_attrinfoid and
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = accountCategory.imr_name and
	accountCategory.imr_name = 'Account' and
	accountCategory.imr_reportid=accountObject.imr_reportid and
	accountObjectInfo.imr_reportid=accountObject.imr_reportid and
	resourceAttrInfo.imr_reportid=accountObject.imr_reportid and
	resourceAttr.imr_reportid=accountObject.imr_reportid;
/


CREATE OR REPLACE VIEW endpointaccountattrs6
AS
SELECT accountObject.imr_reportid as imr_reportid,
	accountObject.imr_objectid as imr_accountid,
	accountObject.imr_friendlyname as imr_accountname,
    accountObject.imr_objectdn as imr_accountdn,
    SUBSTR(accountObject.imr_objectdn,
              INSTR(accountObject.imr_objectdn,',EndPoint=') + LENGTH(',EndPoint='),
			  CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0
			  THEN
                  INSTR(accountObject.imr_objectdn,',Namespace=') - INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	attr1.imr_value as imr_createtime,
	attr3.imr_value as imr_accountlastmodified
FROM
	imrObjectInfo accountObjectInfo,
	imrObject accountObject,
	imrAttributeInfo attrInfo1,
	imrAttribute attr1,
    imrAttributeInfo attrInfo3,
	imrAttribute attr3
WHERE
	((attr1.imr_value IS NOT NULL ) or
	 (attr3.imr_value IS NOT NULL )) and
	attr1.imr_objectid = accountObject.imr_objectid and
	attr3.imr_objectid = accountObject.imr_objectid and
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = 'Account' and
	(attr1.imr_attrinfoid = attrInfo1.imr_attrinfoid and
	attrInfo1.imr_name = 'createDate') and
	(attr3.imr_attrinfoid = attrInfo3.imr_attrinfoid and
	attrInfo3.imr_name = 'updateDate') and
	accountObjectInfo.imr_reportid=accountObject.imr_reportid and
	attrInfo1.imr_reportid=accountObject.imr_reportid and
	attr1.imr_reportid=accountObject.imr_reportid and
    attrInfo3.imr_reportid=accountObject.imr_reportid and
	attr3.imr_reportid=accountObject.imr_reportid;
/

CREATE OR REPLACE VIEW endpointuseraccounts6
AS
select
	a.imr_reportid imr_reportid,
	b.imr_userid imr_userid,
	a.imr_accountname imr_accountname,
	a.imr_accountdn imr_accountdn,
	a.imr_endpoint imr_endpoint,
	a.imr_endpointtype imr_accounttype,
	a.imr_createtime imr_createtime,
	a.imr_accountlastmodified imr_accountlastmodified,
	c.imr_firstname imr_firstname,
	c.imr_lastname imr_lastname,
	c.imr_fullname imr_fullname
from
	endpointaccountattrs6 a,
	imrUserAccountRelationship b,
	imruser6 c
where
	b.imr_useraccountdn = a.imr_accountdn
	and b.imr_reportid = a.imr_reportid
	and c.imr_reportid = a.imr_reportid
	and c.imr_userid=b.imr_userid;
/

---
--- Procedure for counting the number of exception accounts
---

-- Procedures to insert in to the provisioning tables r12.1 additions

create sequence SEQ_ATTRINFO increment by 1  start with 1 ;
/
create or replace PROCEDURE insertimrAttrInfo(
	reportid INTEGER,
	syntaxid INTEGER,
	name varchar2,
	objectinfoid INTEGER,
	isbasetype char,
	infoid OUT INTEGER)
AS
BEGIN
    BEGIN
        infoid := 0;
        Select imr_attrinfoid into infoid from imrAttributeInfo
        WHERE
        ( imr_reportid= reportid
        AND imr_name  = name
        AND imr_objectinfoid  = objectinfoid );
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            infoid := 0;
    END;
    IF NOT (infoid > 0) THEN
  		SELECT SEQ_ATTRINFO.NEXTVAL INTO infoid
                    FROM Dual;
    INSERT into imrAttributeInfo(
                        imr_reportid,
                        IMR_ATTRINFOID,
                        imr_syntaxid,
                        imr_name,
                        imr_objectinfoid,
			imr_isbasetype
                        )
                    values (
			reportid,
			infoid,
                        syntaxid,
                    name,
                    objectinfoid,
                    isbasetype
		);
    END IF;
END insertimrAttrInfo;
/
-----------------------
create sequence SEQ_IMROBJ increment by 1  start with 1 ;
/
create or replace PROCEDURE insertimrobject(
    reportid integer ,
    friendlyname   varchar2,
    objectdn   varchar2,
    objectinfoname   varchar2,
    objectid OUT INTEGER )

AS
BEGIN
	BEGIN
            objectid := 0;
            Select imr_objectid into objectid from imrObject
			WHERE
				imr_reportid= reportid
				AND imr_objectdn = objectdn;
            EXCEPTION
		        WHEN NO_DATA_FOUND THEN
                objectid := 0;
    END;
    IF not  (objectid >0) THEN
    	SELECT SEQ_IMROBJ.NEXTVAL INTO objectid
        FROM Dual;
    insert into imrObject (
            imr_reportid ,
            imr_friendlyname  ,
            imr_objectdn,
            imr_objectinfoname,
            IMR_OBJECTID
          	)
            values (
            reportid ,
            friendlyname ,
            objectdn,
            objectinfoname,
            objectid
            );
    END IF;
END insertimrobject;
/

---------------
create or replace PROCEDURE insertimrobjectcategory(
    reportid integer ,
    name   varchar2)
AS
    rowcount INTEGER;
BEGIN
    BEGIN
	    rowcount :=0;
Select count(*) into rowcount from imrObjectCategory
		WHERE
	    (imr_reportid= reportid
	    AND imr_name = name);
EXCEPTION
	       WHEN NO_DATA_FOUND THEN
	       rowcount :=0;
END;
IF rowcount = 0 THEN
        insert into imrObjectCategory (
            imr_reportid ,
            imr_name
          )
            values (
            reportid ,
            name
           );
END IF;
END insertimrobjectcategory;
/

-------------
create sequence SEQ_OBJINFO increment by 1  start with 1 ;
/
CREATE OR REPLACE PROCEDURE insertimrobjectinfo(
    reportid integer ,
    name   varchar2,
    objectcatname   varchar2,
    infoid OUT INTEGER )
AS
BEGIN
     BEGIN
		infoid := 0;
		Select imr_objectid into infoid from imrObjectInfo
        WHERE
            imr_reportid= reportid
            AND imr_name = name
			AND imr_objectcatname = objectcatname;
        EXCEPTION
		WHEN NO_DATA_FOUND THEN
           	infoid := 0;
    END;
	IF not  (infoid >0) THEN
		SELECT SEQ_OBJINFO.NEXTVAL INTO infoid
                    FROM Dual;
	    insert into imrObjectInfo (
            imr_reportid ,
            imr_objectid,
            imr_name  ,
	    	imr_objectcatname
          )
            values (
            reportid ,
            infoid,
            name ,
		objectcatname
            );
   END IF;
END insertimrobjectinfo;
/

-----------
create or replace PROCEDURE insertimrroletemplate(
    reportid integer ,
	roleoid varchar2,
    roledn   varchar2,
    templatedn   varchar2)
AS
    rowcount INTEGER;
BEGIN
	BEGIN
		rowcount :=0;
        Select count (*) into rowcount from imrProvRoleTemplRelationship
		WHERE
		    (imr_reportid= reportid
			and imr_roleoid=roleoid
		    AND imr_roledn = roledn
			AND imr_templatedn = templatedn);
			EXCEPTION
				WHEN NO_DATA_FOUND THEN
			rowcount :=0;

        END;
        IF rowcount = 0 THEN
            insert into imrProvRoleTemplRelationship(
            imr_reportid ,
			imr_roleoid,
            imr_roledn  ,
		    imr_templatedn
            )
            values (
            reportid ,
			roleoid,
            roledn ,
	    	templatedn
            );
        END IF;
END insertimrroletemplate;
/

CREATE OR REPLACE PROCEDURE insertProvRolesRel(
	reportid integer,
	parentoid varchar2,
	childoid varchar2)
AS
  rowcount integer;
BEGIN
	BEGIN
		rowcount:=0;
		select count(imr_reportid) into rowcount from imrProvRolesRelationship
		where
			imr_reportid=reportid
			and imr_parentoid=parentoid
			and imr_childoid=childoid;
		  EXCEPTION
				 WHEN NO_DATA_FOUND THEN
		  rowcount :=0;

	END;
	IF not  (rowcount >0) THEN
		insert into imrProvRolesRelationship(
			imr_reportid,
			imr_parentoid,
			imr_childoid
		)
		values(
			reportid,
			parentoid,
			childoid
		);
	END IF;
END insertProvRolesRel;
/

CREATE VIEW endPointAttributes
AS
SELECT Attribute.imr_reportid as imr_reportid,
	Attribute.imr_objectid as imr_endpointid,
	Attribute.imr_attrInfoID as AttributeInfoID, AttributeInfo.imr_name as Name,
	Attribute.imr_value as AttributeValue
FROM
	imrattribute Attribute,
	imrattributeinfo AttributeInfo
WHERE imr_objectid IN (SELECT endpointObject.imr_objectid
		FROM
			imrObject endpointObject,
			imrObjectCategory endpointCategory,
			imrObjectInfo endpointObjectInfo
		WHERE
			endpointObject.imr_objectinfoname = endpointObjectInfo.imr_name and
			endpointObjectInfo.imr_objectcatname = endpointCategory.imr_name and
			endpointCategory.imr_name = 'EndPoint' and
			endpointCategory.imr_reportid = endpointObject.imr_reportid and
			endpointObjectInfo.imr_reportid = endpointCategory.imr_reportid and
			endpointObject.imr_reportid = endpointCategory.imr_reportid)
AND AttributeInfo.imr_attrinfoid = Attribute.imr_attrinfoid;
/

CREATE VIEW accttmplattribute
AS
SELECT
	acctTmplObject.imr_reportid as imr_reportid,
	acctTmplObject.imr_friendlyname as imr_accttmplname,
	acctTmplObject.imr_objectdn as imr_accttmpldn,
	acctTmplObjectInfo.imr_name as imr_endpointtype,
	resourceAttrInfo.imr_name as imr_attrname,
	resourceAttr.imr_value as imr_attrvalue,
	resourceAttr.imr_objectrefid as imr_objectrefid,
	resourceAttrInfo.imr_isbasetype as imr_isbasetype
FROM
	imrObjectCategory acctTmplCategory,
	imrObjectInfo acctTmplObjectInfo,
	imrObject acctTmplObject,
	imrAttributeInfo resourceAttrInfo,
	imrAttribute resourceAttr
WHERE
  -- NULL attr values means attr is a subattr
	(resourceAttr.imr_value IS NOT NULL or
	resourceAttr.imr_objectrefid IS NOT NULL) and
	resourceAttr.imr_objectid = acctTmplObject.imr_objectid and
	resourceAttrInfo.imr_attrinfoid = resourceAttr.imr_attrinfoid and
	acctTmplObject.imr_objectinfoname = acctTmplObjectInfo.imr_name and
	acctTmplObjectInfo.imr_objectcatname = acctTmplCategory.imr_name and
	acctTmplCategory.imr_name = 'Policy' and
	acctTmplCategory.imr_reportid=acctTmplObject.imr_reportid and
	acctTmplObjectInfo.imr_reportid = acctTmplObject.imr_reportid and
	resourceAttrInfo.imr_reportid = acctTmplObject.imr_reportid and
	resourceAttr.imr_reportid = acctTmplObject.imr_reportid;
/

CREATE OR REPLACE VIEW endpointaccount
AS
SELECT accountObject.imr_reportid as imr_reportid,
    SUBSTR(accountObject.imr_objectdn,
              INSTR(accountObject.imr_objectdn, ',EndPoint=') + LENGTH(',EndPoint='),
			  CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0
			  THEN
                  INSTR(accountObject.imr_objectdn,',Namespace=') - INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn
FROM
	imrObject accountObject,
	imrObjectInfo accountObjectInfo
WHERE
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = 'Account' and
	accountObject.imr_objectdn like 'Account=%' and 
	accountObject.imr_reportid=accountObjectInfo.imr_reportid;
/

CREATE VIEW endpointview
AS
SELECT
	endpointObject.imr_reportid as imr_reportid,
	endpointObject.imr_friendlyname as imr_endpoint,
	endpointObject.imr_objectdn as imr_endpointdn,
	endpointObject.imr_objectid as imr_endpointid,
	endpointObjectInfo.imr_name as imr_endpointtype
FROM
	imrObject endpointObject,
	imrObjectCategory endpointCategory,
	imrObjectInfo endpointObjectInfo
WHERE
	endpointObject.imr_objectinfoname = endpointObjectInfo.imr_name and
	endpointObjectInfo.imr_objectcatname = endpointCategory.imr_name and
	endpointCategory.imr_name = 'EndPoint' and
	endpointObject.imr_objectdn like 'EndPoint=%' and 
	endpointCategory.imr_reportid = endpointObject.imr_reportid and
	endpointObjectInfo.imr_reportid = endpointObject.imr_reportid;
/

CREATE VIEW provrolesrelationshipview
AS
SELECT
	provRolesRelationship.imr_reportid as imr_reportid,
	provRolesRelationship.imr_parentoid as imr_parentoid,
	provRolesRelationship.imr_childoid as imr_childoid
FROM
	imrProvRolesRelationship provRolesRelationship;
/

CREATE VIEW accttemplEndpointRelView
AS
SELECT
	imrObjectInfo.imr_name AS TemplateType, imrObject.imr_objectdn AS TemplateDN, 		     imrObject.imr_friendlyname AS TemplateName,
    imrTemplEndpointRelationship.imr_endpointname AS EndpointName,  	imrTemplEndpointRelationship.imr_endpointdn AS EndpointDN,
    imrObject.imr_reportid
FROM
	imrTemplEndpointRelationship RIGHT OUTER JOIN
    imrObject ON imrTemplEndpointRelationship.imr_reportid = imrObject.imr_reportid AND
    imrTemplEndpointRelationship.imr_templatedn = imrObject.imr_objectdn LEFT OUTER JOIN
    imrObjectCategory INNER JOIN
    imrObjectInfo ON imrObjectCategory.imr_name = imrObjectInfo.imr_objectcatname ON 	imrObject.imr_reportid = imrObjectCategory.imr_reportid AND
    imrObject.imr_reportid = imrObjectInfo.imr_reportid AND imrObject.imr_objectinfoname = 	imrObjectInfo.imr_name
WHERE
	(imrObjectCategory.imr_name = 'Policy');
/

CREATE VIEW capabilityattrsview
AS
SELECT
imrCapabilityAttrRel.imr_reportid, imrCapabilityAttrRel.imr_endpointtype AS EndpointType, imrObject.imr_friendlyname AS EndpointName,
                      imrObject.imr_objectdn AS EndpointDN, imrCapabilityAttrRel.imr_objecttypename, imrCapabilityAttrRel.imr_jiamobjecttypename,
                      imrCapabilityAttrRel.imr_capabilityobjectname, imrCapabilityAttrRel.imr_capabilityobjectdn, imrCapabilityAttrRel.imr_endpointobjid AS endpointid, imrCapabilityAttrRel.imr_attrid AS Attributeid,imrCapabilityAttrRel.imr_name, imrCapabilityAttrRel.imr_value
FROM imrCapabilityAttrRel, imrObject
where
	imrCapabilityAttrRel.imr_reportid = imrObject.imr_reportid
	and imrCapabilityAttrRel.imr_endpointobjid = imrObject.imr_objectid;
/

CREATE VIEW provroleaccttmplendptrelview
AS
SELECT
	imrProvRoleTemplRelationship.imr_reportid,
	imrProvRoleTemplRelationship.imr_roleoid,
	imrProvRoleTemplRelationship.imr_roledn,
	accttemplEndpointRelView.TemplateType,
	accttemplEndpointRelView.TemplateDN,
	accttemplEndpointRelView.TemplateName,
	accttemplEndpointRelView.EndpointName,
	accttemplEndpointRelView.EndpointDN
FROM
	imrProvRoleTemplRelationship LEFT OUTER JOIN
    accttemplEndpointRelView ON imrProvRoleTemplRelationship.imr_reportid = accttemplEndpointRelView.imr_reportid
	AND imrProvRoleTemplRelationship.imr_templatedn = accttemplEndpointRelView.TemplateDN;
/


------------------------------------------------------
-- New r12.5 SP1 support.
------------------------------------------------------

--
-- IMR imrpolicyxpress125
--

CREATE TABLE imrpolicyxpress125 (
    imr_reportid INTEGER  NOT NULL,
    imr_policyid     VARCHAR2(64) NOT NULL,
    imr_policyname    VARCHAR2(255) NULL,
    imr_enabled    VARCHAR2(5) NULL,
    imr_category    VARCHAR2(255) NULL,
    imr_description    VARCHAR2(1023) NULL,
    imr_runonce    VARCHAR2(5) NULL,
    imr_priority    INTEGER NULL,
    imr_type    VARCHAR2(255) NULL,
    imr_policyevents    NCLOB NULL,
    imr_policydata    NCLOB NULL,
    imr_entryrules    NCLOB NULL,
    imr_actionrules    NCLOB NULL,
    CONSTRAINT PK_01_imrpolicyxpress125 PRIMARY KEY  (imr_reportid, imr_policyid)
    );


--
-- IMR imrreversesyncmodifyaccount125
--
CREATE TABLE imrreversesyncmodifyaccount125 (
    imr_reportid INTEGER  NOT NULL,
    imr_policyid     VARCHAR2(64) NOT NULL,
    imr_policyname    VARCHAR2(255) NULL,
    imr_enabled    VARCHAR2(5) NULL,
    imr_description    VARCHAR2(1023) NULL,
    imr_priority    INTEGER NULL,
    imr_endpointtype    VARCHAR2(255) NULL,
    imr_endpointname    VARCHAR2(1023) NULL,
    imr_container    VARCHAR2(255) NULL,
    imr_attributename    VARCHAR2(255) NULL,
    imr_attributevalue    VARCHAR2(1023) NULL,
    imr_action    VARCHAR2(255) NULL,
    CONSTRAINT PK_01_imrreversesyncmodify125 PRIMARY KEY  (imr_reportid, imr_policyid)
    );

--
-- IMR imrreversesyncnewaccount125
--

CREATE TABLE imrreversesyncnewaccount125 (
    imr_reportid INTEGER  NOT NULL,
    imr_policyid     VARCHAR2(64) NOT NULL,
    imr_policyname    VARCHAR2(255) NULL,
    imr_enabled    VARCHAR2(5) NULL,
    imr_description    VARCHAR2(1023) NULL,
    imr_priority    INTEGER NULL,
    imr_endpointtype    VARCHAR2(255) NULL,
    imr_endpointname    VARCHAR2(1023) NULL,
    imr_container    VARCHAR2(255) NULL,
    imr_targetuserstatus    VARCHAR2(255) NULL,
    imr_action    VARCHAR2(255) NULL,
    CONSTRAINT PK_01_imrreversesyncnew125 PRIMARY KEY  (imr_reportid, imr_policyid)
    );

--
-- IMR imremailmanagement125
--

CREATE TABLE imremailmanagement125 (
    imr_reportid INTEGER  NOT NULL,
    imr_emailid     VARCHAR2(64) NOT NULL,
    imr_emailname    VARCHAR2(255) NULL,
    imr_enabled    VARCHAR2(5) NULL,
    imr_category    VARCHAR2(255) NULL,
    imr_description    VARCHAR2(1023) NULL,
    imr_subject    VARCHAR2(1023) NULL,
    CONSTRAINT PK_01_imremailmanagement125 PRIMARY KEY  (imr_reportid, imr_emailid)
    );

--
-- IMR imrbulktasks125
--

CREATE TABLE imrbulktasks125 (
    imr_reportid INTEGER  NOT NULL,
    imr_bulktaskid     VARCHAR2(64) NOT NULL,
    imr_bulktaskname    VARCHAR2(255) NULL,
    imr_description    VARCHAR2(1023) NULL,
    imr_tasktag    VARCHAR2(255) NULL,
    imr_objecttype    VARCHAR2(255) NULL,
    imr_initiator    VARCHAR2(255) NULL,
    imr_dateformat    VARCHAR2(255) NULL,
    imr_populationrule    NCLOB NULL,
    imr_datefilter    NCLOB NULL,
    CONSTRAINT PK_01_imrbulktasks125 PRIMARY KEY  (imr_reportid, imr_bulktaskid)
    );
/
------------------------------------------------------
-- End of r12.5 SP1 support.
------------------------------------------------------

------------------------------------------------------
-- New r12.5 SP7 support.
------------------------------------------------------

CREATE TABLE imrNonStandardAccts(
	imr_reportid int NOT NULL,
	imr_endpointtype varchar2(64) NULL,
	imr_endpoint varchar2(64) NULL,
	imr_accounttype varchar2(628) NULL,
	imr_totalcount int NULL
);
/

CREATE INDEX imrNonStandardAccts_index_1 ON imrNonStandardAccts (imr_reportid, imr_endpointtype,imr_endpoint);
/

CREATE OR REPLACE VIEW exceptionaccountsview
AS
SELECT accountObject.imr_reportid as imr_reportid,
	accountObject.imr_objectid as imr_accountid,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn,
	(SELECT imr_value
	FROM
		imrAttribute endPointAttr,
		imrAttributeInfo endPointAttrInfo
	WHERE
		accountObject.imr_objectid = endPointAttr.imr_objectid and
		endPointAttr.imr_attrinfoid = endPointAttrInfo.imr_attrinfoid and
		endPointAttrInfo.imr_name = 'endPointName' 
	) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	attr1.imr_value as imr_createtime,
	attr2.imr_value as imr_accountlastmodified,
  attr3.imr_value as exceptionaccount
FROM
	imrObjectCategory accountCategory,
	imrObjectInfo accountObjectInfo,
	imrObject accountObject,
	imrAttributeInfo attrInfo1,
	imrAttribute attr1,
	imrAttributeInfo attrInfo2,
	imrAttribute attr2,
 	imrAttributeInfo attrInfo3,
	imrAttribute attr3

WHERE
	(attr1.imr_value IS NOT NULL or
	attr1.imr_objectrefid IS NOT NULL or
	attr2.imr_value IS NOT NULL or
	attr2.imr_objectrefid IS NOT NULL or 
  attr3.imr_value IS NOT NULL or
	attr3.imr_objectrefid IS NOT NULL) and
	attr1.imr_objectid = accountObject.imr_objectid and
	attr2.imr_objectid = accountObject.imr_objectid and
  attr3.imr_objectid = accountObject.imr_objectid and
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = accountCategory.imr_name and
	accountCategory.imr_name = 'Account' and
	(attr1.imr_attrinfoid = attrInfo1.imr_attrinfoid and 
	attrInfo1.imr_name = 'createDate') and 
	(attr2.imr_attrinfoid = attrInfo2.imr_attrinfoid and 
	attrInfo2.imr_name = 'updateDate') and 
	(attr3.imr_attrinfoid = attrInfo3.imr_attrinfoid and 
	attrInfo3.imr_name = 'exceptionAccount' and attr3.imr_value = 'true') and 
	accountCategory.imr_reportid=accountObject.imr_reportid and
	accountObjectInfo.imr_reportid=accountObject.imr_reportid and
	attrInfo1.imr_reportid=accountObject.imr_reportid and
	attr1.imr_reportid=accountObject.imr_reportid and 
	attrInfo2.imr_reportid=accountObject.imr_reportid and
	attr2.imr_reportid=accountObject.imr_reportid and 
    attrInfo3.imr_reportid=accountObject.imr_reportid and
	attr3.imr_reportid=accountObject.imr_reportid;
/

CREATE OR REPLACE VIEW endpointexceptionuseraccounts6
AS
SELECT a.imr_reportid imr_reportid, 
	b.imr_userid imr_userid, 
	a.imr_accountname imr_accountname, 
	a.imr_accountdn imr_accountdn, 
	a.imr_endpoint as imr_endpoint,
	a.imr_endpointtype imr_accounttype, 
	a.imr_createtime imr_createtime,
	a.imr_accountlastmodified imr_accountlastmodified,
	c.imr_firstname imr_firstname,
	c.imr_lastname imr_lastname,
	c.imr_fullname imr_fullname
FROM  
	exceptionaccountsview a, 
	imrUserAccountRelationship b,
	imruser6 c 
WHERE 
	b.imr_useraccountdn = a.imr_accountdn
	and b.imr_reportid = a.imr_reportid
	and c.imr_reportid = a.imr_reportid
	and c.imr_userid=b.imr_userid;
/

CREATE OR REPLACE VIEW totalExceptionAccounts
AS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'Exception' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME, imr_createtime as CREATION_DATE, imr_accountlastmodified as LAST_MODIFIED 
FROM         endpointexceptionuseraccounts6
WHERE imr_userid NOT IN ( '[default user]','[system user]')
UNION
-- ADD ORPHAN ACCOUNTS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'Orphan' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME, imr_createtime as CREATION_DATE, imr_accountlastmodified as LAST_MODIFIED 
FROM         endpointuseraccounts6
WHERE      imr_userid  IN('[default user]')
UNION
-- ADD SYSTEM ACCOUNTS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'System' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME, imr_createtime as CREATION_DATE, imr_accountlastmodified as LAST_MODIFIED 
FROM         endpointuseraccounts6
WHERE      imr_userid  IN('[system user]');
/

------------------------------------------------------
-- End of r12.5 SP7 support.
------------------------------------------------------
----------
-- Cleanup Stuff
----------


--
-- PROCEDURE IMRCleanupProc6 For cleaning reports older than a month
-- *********DEPRICATED DO NOT USE
--
CREATE OR REPLACE PROCEDURE IMRCleanupProc6 AS
BEGIN
    DELETE FROM imrrole6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrroletask6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrtask6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrrolerule6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrrolescoperule6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrrolemember6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrroleadmin6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrroleowner6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrorg6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrorgattr6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imruser6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imruserattr6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrgroup6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrgroupattr6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrgroupmember6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrgroupadmin6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    -- delete identitypolicy related items
    DELETE FROM imridentitypolicy6 where imr_reportid IN
                 (
                 SELECT imreport6.imr_reportid
                 FROM imreport6
                 WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imridentitypolicyset6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imridentitypolicyactions6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imridentitypolicyusersync6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   --New 12.5 SP1 suport
   DELETE FROM imrpolicyxpress125 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imrreversesyncnewaccount125 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imrreversesyncmodifyaccount125 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imremailmanagement125 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imrbulktasks125 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   --End of 12.5 SP1 suport
    DELETE FROM imreport6 WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) );
END IMRCleanupProc6 ;
/


--
-- Temporary table for keeping snapshots id we should remove
--
CREATE GLOBAL TEMPORARY TABLE retired_snapshots ON COMMIT DELETE ROWS AS SELECT * FROM imreport6 WHERE 1=0;

--
-- Procedure for removing a snapshot from all the tables it exist
--
CREATE OR REPLACE PROCEDURE IMRRemoveSnapshot (
	id INTEGER)
AS
BEGIN
	DELETE FROM imrrole6 WHERE imr_reportid = id;
	DELETE FROM imrrolemember6 WHERE imr_reportid = id;
	DELETE FROM imrroleadmin6 WHERE imr_reportid = id;
	DELETE FROM imrroleowner6 WHERE imr_reportid = id;
	DELETE FROM imrrolerule6 WHERE imr_reportid = id;
	DELETE FROM imrrolescoperule6 WHERE imr_reportid = id;

	DELETE FROM imrroletask6 WHERE imr_reportid = id;
	DELETE FROM imrtask6 WHERE imr_reportid = id;

	DELETE FROM imrorg6 WHERE imr_reportid = id;
	DELETE FROM imrorgattr6 WHERE imr_reportid = id;

	DELETE FROM imruser6 WHERE imr_reportid = id;
	DELETE FROM imruserattr6 WHERE imr_reportid = id;

	DELETE FROM imrgroup6 WHERE imr_reportid = id;
	DELETE FROM imrgroupattr6 WHERE imr_reportid = id;
	DELETE FROM imrgroupmember6 WHERE imr_reportid = id;
	DELETE FROM imrgroupadmin6 WHERE imr_reportid = id;

	DELETE FROM imridentitypolicy6 WHERE imr_reportid = id;
	DELETE FROM imridentitypolicyset6 WHERE imr_reportid = id;
	DELETE FROM imridentitypolicyusersync6 WHERE imr_reportid = id;
	DELETE FROM imridentitypolicyactions6 WHERE imr_reportid = id;

	DELETE FROM imrAttribute WHERE imr_reportid = id;
	DELETE FROM imrAttributeInfo WHERE imr_reportid = id;
	DELETE FROM imrObject WHERE imr_reportid = id;
	DELETE FROM imrObjectCategory WHERE imr_reportid = id;
	DELETE FROM imrObjectInfo WHERE imr_reportid = id;
	DELETE FROM imrProvRolesRelationship WHERE imr_reportid = id;
	DELETE FROM imrProvRoleTemplRelationship WHERE imr_reportid = id;
	DELETE FROM imrUserAccountRelationship WHERE imr_reportid = id;
	DELETE FROM imrTemplEndpointRelationship WHERE imr_reportid = id;
	DELETE FROM imrAccountTemplateRel WHERE imr_reportid = id;
	DELETE FROM imrCapabilityAttrRel WHERE imr_reportid = id;
	DELETE FROM imrroleattr12 WHERE imr_reportid = id;

	--New 12.5 SP1 suport
	DELETE FROM imrpolicyxpress125 WHERE imr_reportid = id;
	DELETE FROM imrreversesyncnewaccount125 WHERE imr_reportid = id;
	DELETE FROM imrreversesyncmodifyaccount125 WHERE imr_reportid = id;
	DELETE FROM imremailmanagement125 WHERE imr_reportid = id;
	DELETE FROM imrbulktasks125 WHERE imr_reportid = id;
	--End of 12.5 SP1 suport

	DELETE FROM imreport6 WHERE imr_reportid = id;
END;
/

--
-- Procedure for keeping only top X snapshots that we want, and remove the rest.
--
CREATE OR REPLACE PROCEDURE IMRCleanOldSnapshotsProc (
	   skeep INTEGER,
	   name VARCHAR2,
	   source VARCHAR2)

IS
  snap_cur    SYS_REFCURSOR;
  at_rec      imreport6%ROWTYPE;

BEGIN
	 INSERT INTO retired_snapshots (SELECT * FROM imreport6 WHERE imr_reportname=name
	  AND imr_snapshotsource=source AND imr_snapshotstatus = 'Completed'
	   AND imr_reportid NOT IN
	    (SELECT imr_reportid FROM
		 (SELECT imr_reportid FROM imreport6 WHERE  imr_reportname=name
		  AND imr_snapshotsource=source AND imr_snapshotstatus = 'Completed' ORDER BY imr_reportid DESC)
		   WHERE ROWNUM <= skeep));

	 OPEN snap_cur FOR SELECT * FROM retired_snapshots;

	 LOOP
	  	  FETCH snap_cur INTO at_rec;
		  EXIT WHEN snap_cur%NOTFOUND;
		  IMRRemoveSnapshot (at_rec.imr_reportid);
	 END LOOP;

END;
/

create or replace FUNCTION IM_Reports (getid int, gettaskname VARCHAR2, getenvoid VARCHAR2)

RETURN int is
tempid1 int;
tempid2 int;

BEGIN
SELECT max(imr_reportid) into tempid2
FROM 
 imreport6
WHERE imr_reportname IN
  (SELECT imr_snapshotname
  FROM imrptsnapshots12
  WHERE imr_rpt_task_tag_name = gettaskname
  AND imr_envoid = getenvoid
  );

if getid = -1
then tempid1 := tempid2;

else
tempid1 := getid;

end if;
return tempid1;
END;
/