-- Copyright (C) 2010, CA, Inc. All rights reserved.
--
-- This file contains the upgrade schema for all of the 12.5 SP7
-- tables for snapshot
--
--
-------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE insertimrobjectinfo(
    reportid integer ,
    name   varchar2,
    objectcatname   varchar2,
    infoid OUT INTEGER )
AS
BEGIN
     BEGIN
		infoid := 0;
		Select imr_objectid into infoid from imrObjectInfo
        WHERE
            imr_reportid= reportid
            AND imr_name = name
			AND imr_objectcatname = objectcatname;
        EXCEPTION
		WHEN NO_DATA_FOUND THEN
           	infoid := 0;
    END;
	IF not  (infoid >0) THEN
		SELECT SEQ_OBJINFO.NEXTVAL INTO infoid
                    FROM Dual;
	    insert into imrObjectInfo (
            imr_reportid ,
            imr_objectid,
            imr_name  ,
	    	imr_objectcatname
          )
            values (
            reportid ,
            infoid,
            name ,
		objectcatname
            );
   END IF;
END insertimrobjectinfo;
/

CREATE OR REPLACE VIEW imruseraccounts6
AS
SELECT 	accountObject.imr_reportid as imr_reportid,
	imruser.imr_userid as imr_userid,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn,
	SUBSTR(accountObject.imr_objectdn,
              INSTR(accountObject.imr_objectdn,',EndPoint=') + LENGTH(',EndPoint='),
			  CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0
			  THEN
                  INSTR(accountObject.imr_objectdn,',Namespace=') - INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObject.imr_objectinfoname as imr_accounttype,
	accountAttr1.imr_value as imr_createtime
FROM imrObject accountObject,
	imrUserAccountRelationship userAccountRelationship,
	imrAttributeInfo accountAttrInfo1,
	imrAttribute accountAttr1,
	imruser6 imruser
WHERE imruser.imr_userid = userAccountRelationship.imr_userid and
	accountObject.imr_objectdn = userAccountRelationship.imr_useraccountdn and
	accountAttr1.imr_attrinfoid = accountAttrInfo1.imr_attrinfoid and
	accountAttrInfo1.imr_name='createDate' and
	accountObject.imr_objectid = accountAttr1.imr_objectid and
	accountObject.imr_reportid=imruser.imr_reportid and
	userAccountRelationship.imr_reportid = imruser.imr_reportid and
	accountAttrInfo1.imr_reportid = imruser.imr_reportid and
	accountAttr1.imr_reportid = imruser.imr_reportid;
/

CREATE OR REPLACE VIEW imraccountattrs6
AS
SELECT accountObject.imr_reportid as imr_reportid,
	accountObject.imr_objectid as accountObjid,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn,
    SUBSTR(accountObject.imr_objectdn,
              INSTR(accountObject.imr_objectdn,',EndPoint=') + LENGTH(',EndPoint='),
			  CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0
			  THEN
                  INSTR(accountObject.imr_objectdn,',Namespace=') - INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	resourceAttrInfo.imr_name as imr_accountattr,
	resourceAttr.imr_value as imr_accountattrvalue,
	resourceAttr.imr_objectrefid as imr_objectrefid,
	resourceAttrInfo.imr_isbasetype as imr_isbasetype
FROM
	imrObjectCategory accountCategory,
	imrObjectInfo accountObjectInfo,
	imrObject accountObject,
	imrAttributeInfo resourceAttrInfo,
	imrAttribute resourceAttr
WHERE
	(resourceAttr.imr_value IS NOT NULL or
	resourceAttr.imr_objectrefid IS NOT NULL) and
	resourceAttr.imr_objectid = accountObject.imr_objectid and
	resourceAttrInfo.imr_attrinfoid = resourceAttr.imr_attrinfoid and
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = accountCategory.imr_name and
	accountCategory.imr_name = 'Account' and
	accountCategory.imr_reportid=accountObject.imr_reportid and
	accountObjectInfo.imr_reportid=accountObject.imr_reportid and
	resourceAttrInfo.imr_reportid=accountObject.imr_reportid and
	resourceAttr.imr_reportid=accountObject.imr_reportid;
/

CREATE OR REPLACE VIEW endpointaccountattrs6
AS
SELECT accountObject.imr_reportid as imr_reportid,
	accountObject.imr_objectid as imr_accountid,
	accountObject.imr_friendlyname as imr_accountname,
    accountObject.imr_objectdn as imr_accountdn,
    SUBSTR(accountObject.imr_objectdn,
              INSTR(accountObject.imr_objectdn,',EndPoint=') + LENGTH(',EndPoint='),
			  CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0
			  THEN
                  INSTR(accountObject.imr_objectdn,',Namespace=') - INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	attr1.imr_value as imr_createtime,
	attr3.imr_value as imr_accountlastmodified
FROM
	imrObjectInfo accountObjectInfo,
	imrObject accountObject,
	imrAttributeInfo attrInfo1,
	imrAttribute attr1,
    imrAttributeInfo attrInfo3,
	imrAttribute attr3
WHERE
	((attr1.imr_value IS NOT NULL ) or
	 (attr3.imr_value IS NOT NULL )) and
	attr1.imr_objectid = accountObject.imr_objectid and
	attr3.imr_objectid = accountObject.imr_objectid and
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = 'Account' and
	(attr1.imr_attrinfoid = attrInfo1.imr_attrinfoid and
	attrInfo1.imr_name = 'createDate') and
	(attr3.imr_attrinfoid = attrInfo3.imr_attrinfoid and
	attrInfo3.imr_name = 'updateDate') and
	accountObjectInfo.imr_reportid=accountObject.imr_reportid and
	attrInfo1.imr_reportid=accountObject.imr_reportid and
	attr1.imr_reportid=accountObject.imr_reportid and
    attrInfo3.imr_reportid=accountObject.imr_reportid and
	attr3.imr_reportid=accountObject.imr_reportid;
/

CREATE OR REPLACE VIEW endpointaccount
AS
SELECT accountObject.imr_reportid as imr_reportid,
    SUBSTR(accountObject.imr_objectdn,
              INSTR(accountObject.imr_objectdn, ',EndPoint=') + LENGTH(',EndPoint='),
			  CASE WHEN INSTR(accountObject.imr_objectdn,',Namespace=') > 0
			  THEN
                  INSTR(accountObject.imr_objectdn,',Namespace=') - INSTR(accountObject.imr_objectdn,',EndPoint=') - LENGTH(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn
FROM
	imrObject accountObject,
	imrObjectInfo accountObjectInfo
WHERE
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = 'Account' and
	accountObject.imr_objectdn like 'Account=%' and 
	accountObject.imr_reportid=accountObjectInfo.imr_reportid;
/

CREATE OR REPLACE VIEW exceptionaccountsview
AS
SELECT accountObject.imr_reportid as imr_reportid,
	accountObject.imr_objectid as imr_accountid,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn,
	(SELECT imr_value
	FROM
		imrAttribute endPointAttr,
		imrAttributeInfo endPointAttrInfo
	WHERE
		accountObject.imr_objectid = endPointAttr.imr_objectid and
		endPointAttr.imr_attrinfoid = endPointAttrInfo.imr_attrinfoid and
		endPointAttrInfo.imr_name = 'endPointName' 
	) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	attr1.imr_value as imr_createtime,
	attr2.imr_value as imr_accountlastmodified,
  attr3.imr_value as exceptionaccount
FROM
	imrObjectCategory accountCategory,
	imrObjectInfo accountObjectInfo,
	imrObject accountObject,
	imrAttributeInfo attrInfo1,
	imrAttribute attr1,
	imrAttributeInfo attrInfo2,
	imrAttribute attr2,
 	imrAttributeInfo attrInfo3,
	imrAttribute attr3

WHERE
	(attr1.imr_value IS NOT NULL or
	attr1.imr_objectrefid IS NOT NULL or
	attr2.imr_value IS NOT NULL or
	attr2.imr_objectrefid IS NOT NULL or 
  attr3.imr_value IS NOT NULL or
	attr3.imr_objectrefid IS NOT NULL) and
	attr1.imr_objectid = accountObject.imr_objectid and
	attr2.imr_objectid = accountObject.imr_objectid and
  attr3.imr_objectid = accountObject.imr_objectid and
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = accountCategory.imr_name and
	accountCategory.imr_name = 'Account' and
	(attr1.imr_attrinfoid = attrInfo1.imr_attrinfoid and 
	attrInfo1.imr_name = 'createDate') and 
	(attr2.imr_attrinfoid = attrInfo2.imr_attrinfoid and 
	attrInfo2.imr_name = 'updateDate') and 
	(attr3.imr_attrinfoid = attrInfo3.imr_attrinfoid and 
	attrInfo3.imr_name = 'exceptionAccount' and attr3.imr_value = 'true') and 
	accountCategory.imr_reportid=accountObject.imr_reportid and
	accountObjectInfo.imr_reportid=accountObject.imr_reportid and
	attrInfo1.imr_reportid=accountObject.imr_reportid and
	attr1.imr_reportid=accountObject.imr_reportid and 
	attrInfo2.imr_reportid=accountObject.imr_reportid and
	attr2.imr_reportid=accountObject.imr_reportid and 
    attrInfo3.imr_reportid=accountObject.imr_reportid and
	attr3.imr_reportid=accountObject.imr_reportid;
/

CREATE OR REPLACE VIEW endpointexceptionuseraccounts6
AS
SELECT a.imr_reportid imr_reportid, 
	b.imr_userid imr_userid, 
	a.imr_accountname imr_accountname, 
	a.imr_accountdn imr_accountdn, 
	a.imr_endpoint as imr_endpoint,
	a.imr_endpointtype imr_accounttype, 
	a.imr_createtime imr_createtime,
	a.imr_accountlastmodified imr_accountlastmodified,
	c.imr_firstname imr_firstname,
	c.imr_lastname imr_lastname,
	c.imr_fullname imr_fullname
FROM  
	exceptionaccountsview a, 
	imrUserAccountRelationship b,
	imruser6 c 
WHERE 
	b.imr_useraccountdn = a.imr_accountdn
	and b.imr_reportid = a.imr_reportid
	and c.imr_reportid = a.imr_reportid
	and c.imr_userid=b.imr_userid;
/


CREATE OR REPLACE VIEW totalExceptionAccounts
AS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'Exception' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME, imr_createtime as CREATION_DATE, imr_accountlastmodified as LAST_MODIFIED 
FROM         endpointexceptionuseraccounts6
WHERE imr_userid NOT IN ( '[default user]','[system user]')
UNION
-- ADD ORPHAN ACCOUNTS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'Orphan' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME, imr_createtime as CREATION_DATE, imr_accountlastmodified as LAST_MODIFIED 
FROM         endpointuseraccounts6
WHERE      imr_userid  IN('[default user]')
UNION
-- ADD SYSTEM ACCOUNTS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'System' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME, imr_createtime as CREATION_DATE, imr_accountlastmodified as LAST_MODIFIED 
FROM         endpointuseraccounts6
WHERE      imr_userid  IN('[system user]');
/

CREATE OR REPLACE VIEW endpointview
AS
SELECT
	endpointObject.imr_reportid as imr_reportid,
	endpointObject.imr_friendlyname as imr_endpoint,
	endpointObject.imr_objectdn as imr_endpointdn,
	endpointObject.imr_objectid as imr_endpointid,
	endpointObjectInfo.imr_name as imr_endpointtype
FROM
	imrObject endpointObject,
	imrObjectCategory endpointCategory,
	imrObjectInfo endpointObjectInfo
WHERE
	endpointObject.imr_objectinfoname = endpointObjectInfo.imr_name and
	endpointObjectInfo.imr_objectcatname = endpointCategory.imr_name and
	endpointCategory.imr_name = 'EndPoint' and
	endpointObject.imr_objectdn like 'EndPoint=%' and 
	endpointCategory.imr_reportid = endpointObject.imr_reportid and
	endpointObjectInfo.imr_reportid = endpointObject.imr_reportid;
/

UPDATE imreport_version SET imr_value='12.5.7.0.0';
/
