-- Copyright (C) 2005, Computer Associates, Inc. All rights reserved.
--
-- This file contains the schema for upgrading a IM 8 schema to a IM R12 schema
--
-------------------------------------------------------------------

-- Add imr_reportdescription field to imreport6 table


ALTER TABLE imreport6 ADD (imr_reportdescription VARCHAR2(628),
				imr_keepsnapshot INTEGER ,
				imr_snapshotsource VARCHAR2 (255) ,
				imr_snapshotstatus VARCHAR2 (64) DEFAULT 'Completed' NOT NULL);
/
Alter table imreport6 drop constraint PK_01_IMREPORT6;
/
Alter table imreport6 add constraint PK_01_IMREPORT6 PRIMARY KEY (imr_reportid);
/

DROP PROCEDURE insertintoimreport;
/

CREATE OR REPLACE PROCEDURE insertintoimreport (
    reportname  VARCHAR2 ,
    reportdescription  VARCHAR2 ,
    reportuser  VARCHAR2 ,
    envoid      VARCHAR2 ,
    envname     VARCHAR2 ,
	keepsnapshot INTEGER ,
	snapshotsource VARCHAR2 ,
	snapshotstatus VARCHAR2 ,
    current_id      OUT INTEGER)
AS
BEGIN
SELECT SEQ_IDREPORT.NEXTVAL
    INTO   current_id
    FROM   Dual;

 INSERT INTO imreport6 (
	 	imr_reportid,
        imr_reportname ,
        imr_reportdescription,
        imr_reportuser ,
        imr_envoid ,
        imr_envname ,
		imr_keepsnapshot ,
		imr_snapshotsource ,
		imr_snapshotstatus)
            VALUES (
        current_id ,
        reportname ,
        reportdescription ,
        reportuser ,
        envoid ,
        envname ,
		keepsnapshot ,
		snapshotsource ,
		snapshotstatus);
RETURN;
END;
/


DROP PROCEDURE insertintousertable;
/

CREATE OR REPLACE PROCEDURE insertintousertable  (
    reportid INTEGER ,
    userdn    VARCHAR2,
    isenabled INTEGER ,
    userid    VARCHAR2,
    fullname  VARCHAR2,
    lastname  VARCHAR2 ,
    firstname VARCHAR2,
    parentorgdn VARCHAR2 ,
    email       VARCHAR2)
IS
    rowcount   INTEGER;
BEGIN
  BEGIN
      rowcount :=0;
        Select count(*) into rowcount from imruser6
            WHERE
                imruser6.imr_reportid= reportid
                AND imruser6.imr_userdn = userdn;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        rowcount :=0;
  END;
  IF rowcount = 0  THEN
     insert into imruser6 (
          imr_reportid ,
          imr_userdn   ,
          imr_isenabled,
          imr_userid   ,
          imr_fullname ,
          imr_lastname ,
          imr_firstname,
          imr_parentorgdn ,
          imr_email)
                values (
           reportid ,
           userdn   ,
           isenabled,
           userid   ,
           fullname ,
           lastname ,
           firstname,
           parentorgdn ,
           email );
  END IF;
  END;
/



-- New tables
-- Add userAccounts tables

DROP TABLE imruseraccounts6 CASCADE CONSTRAINTS;
/

CREATE TABLE imruseraccounts6 (
	imr_reportid 				INTEGER NOT NULL,
	imr_userdn    				VARCHAR2(628) NOT NULL,
	imr_username   		        VARCHAR2(64)NULL,
	imr_accountdn   			VARCHAR2(628) NOT NULL,
	imr_accountname  			VARCHAR2(64) NULL,
    imr_endpoint  				VARCHAR2(64) NULL,
	imr_accountcontainer 		VARCHAR2(64) NULL,
	imr_accounttype  			VARCHAR2(64) NULL,
	imr_accountdomain  			VARCHAR2(64) NULL,
	imr_accountreason			VARCHAR2(256) NULL,
	imr_accountcreator			VARCHAR2(256) NULL,
	imr_accountcreationdate  	DATE NULL,
	imr_accountlastmodified 	DATE NULL,
    imr_accountapprover			VARCHAR2(256) NULL,
	imr_accountpolicy			VARCHAR2(256) NULL,
	imr_accountsyncwithroles    INTEGER NULL,
	IMR_ISENABLED    INTEGER NULL
);
/

CREATE INDEX imruseraccounts6_index_1 ON imruseraccounts6 (imr_reportid,imr_userdn);
/

CREATE INDEX imruseraccounts6_index_2 ON imruseraccounts6 (imr_reportid,imr_userdn,imr_accountdn);
/

CREATE INDEX imruseraccounts6_index_3 ON imruseraccounts6 (imr_reportid,imr_endpoint);
/

CREATE INDEX imruseraccounts6_index_4 ON imruseraccounts6 (imr_reportid,imr_accounttype);
/

CREATE INDEX imruseraccounts6_index_5 ON imruseraccounts6 (imr_reportid,imr_endpoint,imr_accounttype);
/

-- SP to do the insert into the imruseraccounts6 table
CREATE OR REPLACE PROCEDURE insertintoimruseraccounts  (
	reportid 			INTEGER,
	userdn    			VARCHAR2,
	username   			VARCHAR2,
	accountdn   			VARCHAR2,
	accountname  			VARCHAR2,
        endpoint  			VARCHAR2,
	accountcontainer 		VARCHAR2,
	accounttype  			VARCHAR2,
	accountdomain  			VARCHAR2,
	accountreason			VARCHAR2,
	accountcreator			VARCHAR2,
	accountcreationdate 		DATE,
	accountlastmodified 		DATE,
	accountapprover			VARCHAR2,
	accountpolicy			VARCHAR2,
	accountsyncwithroles    INTEGER)
IS
	rowcount	INTEGER;
BEGIN
    BEGIN
    	rowcount := 0;
    	Select count(*) into rowcount from imruseraccounts6
    	    WHERE
    	 	imr_reportid=reportid
    	 	AND imr_userdn = userdn
    	 	AND imr_accountdn = accountdn;
    	 EXCEPTION
    	 WHEN NO_DATA_FOUND THEN
    	 	rowcount := 0;
    END;
    IF rowcount = 0 THEN
    	insert into imruseraccounts6 (
    		imr_reportid,
			imr_userdn,
			imr_username,
			imr_accountdn,
			imr_accountname,
	        imr_endpoint,
			imr_accountcontainer,
			imr_accounttype,
			imr_accountdomain,
			imr_accountreason,
			imr_accountcreator,
			imr_accountcreationdate,
			imr_accountlastmodified,
	        imr_accountapprover,
			imr_accountpolicy,
			imr_accountsyncwithroles
		)
		values (
	    	reportid,
			userdn,
			username,
			accountdn,
			accountname,
	        endpoint,
			accountcontainer,
			accounttype,
			accountdomain,
			accountreason,
			accountcreator,
			accountcreationdate,
			accountlastmodified,
	        accountapprover,
			accountpolicy,
			accountsyncwithroles
		);
    END IF;
END;
/



--
-- IM imraccountattrs6
--

DROP TABLE imraccountattrs6 CASCADE CONSTRAINTS;
/

CREATE TABLE imraccountattrs6 (
    imr_reportid 		INTEGER  NOT NULL,
    imr_accountdn          	VARCHAR2(628) NOT NULL,
    imr_accountattr		VARCHAR2(256) NULL,
    imr_accountattrvalue   	VARCHAR2(628) NULL
);
/

CREATE INDEX imraccountattrs6_index_1 ON imraccountattrs6 (imr_reportid, imr_accountdn);
/

--
-- IM imrptsnapshots12
--
DROP TABLE imrptsnapshots12 CASCADE CONSTRAINTS;
/

CREATE TABLE imrptsnapshots12 (
	imr_rptname 			VARCHAR2(255) NOT NULL,
	imr_snapshotname    		VARCHAR2(255) NOT NULL,
	imr_envname   		        VARCHAR2(255) NOT NULL,
	IMR_RPT_TASK_TAG_NAME VARCHAR2(255) NOT NULL
);
/

CREATE INDEX imrptsnapshots12_index_1 ON imrptsnapshots12 (imr_rptname,imr_snapshotname,imr_envname);
/

--
-- IMR TASK 6
--
ALTER TABLE imrtask6 ADD (
	imr_reserved6 VARCHAR2(255) NULL,
	imr_reserved7 VARCHAR2(255) NULL,
    	imr_reserved8 VARCHAR2(255) NULL,
    	imr_reserved9 VARCHAR2(255) NULL,
    	imr_reserved10 VARCHAR2(255) NULL
);
/

--
-- PROCEDURE IMRCleanupProc6 For cleaning reports older than a month
-- modify existing cleanup procedure to include useraccounts support
-- *********DEPRICATED DO NOT USE
--


DROP PROCEDURE IMRCleanupProc6;
CREATE OR REPLACE PROCEDURE IMRCleanupProc6 AS
BEGIN
    DELETE FROM imrrole6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrroletask6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrtask6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrrolerule6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrrolescoperule6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrrolemember6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrroleadmin6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrroleowner6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrorg6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrorgattr6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imruser6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imruserattr6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrgroup6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrgroupattr6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrgroupmember6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imrgroupadmin6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imruseraccounts6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    DELETE FROM imraccountattrs6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
    -- delete identitypolicy related items
    DELETE FROM imridentitypolicy6 where imr_reportid IN
                 (
                 SELECT imreport6.imr_reportid
                 FROM imreport6
                 WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imridentitypolicyset6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imridentitypolicyactions6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
   DELETE FROM imridentitypolicyusersync6 where imr_reportid IN
                (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) )
                );
 DELETE FROM imreport6 WHERE ( imreport6.imr_reporttime < ADD_MONTHS(SYSDATE,-1) );
END IMRCleanupProc6 ;
/


--
-- Temporary table for keeping snapshots id we should remove
--
DROP TABLE retired_snapshots CASCADE CONSTRAINTS;
CREATE GLOBAL TEMPORARY TABLE retired_snapshots ON COMMIT DELETE ROWS AS SELECT * FROM imreport6 WHERE 1=0;

--
-- Procedure for removing a snapshot from all the tables it exist
--
CREATE OR REPLACE PROCEDURE IMRRemoveSnapshot (
	id INTEGER)
AS
BEGIN
	DELETE FROM imrrole6 WHERE imr_reportid = id;
	DELETE FROM imrrolemember6 WHERE imr_reportid = id;
	DELETE FROM imrroleadmin6 WHERE imr_reportid = id;
	DELETE FROM imrroleowner6 WHERE imr_reportid = id;
	DELETE FROM imrrolerule6 WHERE imr_reportid = id;
	DELETE FROM imrrolescoperule6 WHERE imr_reportid = id;

	DELETE FROM imrroletask6 WHERE imr_reportid = id;
	DELETE FROM imrtask6 WHERE imr_reportid = id;

	DELETE FROM imrorg6 WHERE imr_reportid = id;
	DELETE FROM imrorgattr6 WHERE imr_reportid = id;

	DELETE FROM imruser6 WHERE imr_reportid = id;
	DELETE FROM imruserattr6 WHERE imr_reportid = id;

	DELETE FROM imrgroup6 WHERE imr_reportid = id;
	DELETE FROM imrgroupattr6 WHERE imr_reportid = id;
	DELETE FROM imrgroupmember6 WHERE imr_reportid = id;
	DELETE FROM imrgroupadmin6 WHERE imr_reportid = id;

	DELETE FROM imridentitypolicy6 WHERE imr_reportid = id;
	DELETE FROM imridentitypolicyset6 WHERE imr_reportid = id;
	DELETE FROM imridentitypolicyusersync6 WHERE imr_reportid = id;
	DELETE FROM imridentitypolicyactions6 WHERE imr_reportid = id;
	
	DELETE FROM imrAttribute WHERE imr_reportid = id;
	DELETE FROM imrAttributeInfo WHERE imr_reportid = id;
	DELETE FROM imrObject WHERE imr_reportid = id;
	DELETE FROM imrObjectCategory WHERE imr_reportid = id;
	DELETE FROM imrObjectInfo WHERE imr_reportid = id;
	DELETE FROM imrProvRolesRelationship WHERE imr_reportid = id;
	DELETE FROM imrProvRoleTemplRelationship WHERE imr_reportid = id;
	DELETE FROM imrUserAccountRelationship WHERE imr_reportid = id;
	DELETE FROM imrTemplEndpointRelationship WHERE imr_reportid = id;
	DELETE FROM imrAccountTemplateRel WHERE imr_reportid = id;
	DELETE FROM imrCapabilityAttrRel WHERE imr_reportid = id;
	DELETE FROM imrroleattr12 WHERE imr_reportid = id;
	

	DELETE FROM imreport6 WHERE imr_reportid = id;
END;
/

--
-- Procedure for keeping only top X snapshots that we want, and remove the rest.
--
CREATE OR REPLACE PROCEDURE IMRCleanOldSnapshotsProc (
	   skeep INTEGER,
	   name VARCHAR2,
	   source VARCHAR2)

IS
  snap_cur    SYS_REFCURSOR;
  at_rec      imreport6%ROWTYPE;

BEGIN
	 INSERT INTO retired_snapshots (SELECT * FROM imreport6 WHERE imr_reportname=name
	  AND imr_snapshotsource=source AND imr_snapshotstatus = 'Completed'
	   AND imr_reportid NOT IN
	    (SELECT imr_reportid FROM
		 (SELECT imr_reportid FROM imreport6 WHERE  imr_reportname=name
		  AND imr_snapshotsource=source AND imr_snapshotstatus = 'Completed' ORDER BY imr_reportid DESC)
		   WHERE ROWNUM <= skeep));

	 OPEN snap_cur FOR SELECT * FROM retired_snapshots;

	 LOOP
	  	  FETCH snap_cur INTO at_rec;
		  EXIT WHEN snap_cur%NOTFOUND;
		  IMRRemoveSnapshot (at_rec.imr_reportid);
	 END LOOP;

END;
/
---
--- Procedure for counting the number of exception accounts
---
CREATE OR REPLACE VIEW
    "TOTALEXCEPTIONACCOUNTS" ("REPORT_ID",
    "END_POINT_TYPE","END_POINT","EXCEPTION_TYPE","ACCOUNT_NAME",
    "CREATION_DATE","IS_ENABLED","LAST_MODIFIED") AS
    SELECT imr_reportid AS REPORT_ID, imr_accounttype AS
END_POINT_TYPE, imr_endpoint AS END_POINT, 'Exception' AS
EXCEPTION_TYPE,imr_accountname AS ACCOUNT_NAME,imr_accountcreationdate as CREATION_DATE, imr_isenabled as IS_ENABLED,imr_accountlastmodified as LAST_MODIFIED
FROM imruseraccounts6
WHERE imr_accountsyncwithroles = 1 AND imr_username NOT IN ( '[default user]','[system user]')
UNION
SELECT imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'Orphan' AS EXCEPTION_TYPE,
imr_accountname AS ACCOUNT_NAME,imr_accountcreationdate as CREATION_DATE, imr_isenabled as IS_ENABLED,imr_accountlastmodified as LAST_MODIFIED
FROM         imruseraccounts6
WHERE  imr_username  IN('[default user]')
UNION
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'System' AS EXCEPTION_TYPE,
imr_accountname AS ACCOUNT_NAME,imr_accountcreationdate as CREATION_DATE, imr_isenabled as IS_ENABLED,imr_accountlastmodified as LAST_MODIFIED
FROM         imruseraccounts6
WHERE      imr_username  IN('[system user]');
/

CREATE OR REPLACE PROCEDURE insertintoaccountAttributes (

	imr_reportid 			 INTEGER,
	imr_accountdn    	 VARCHAR2,
	imr_accountattr    	 VARCHAR2,
	imr_accountattrvalue VARCHAR2)
AS
BEGIN

    	insert into imraccountattrs6 (
			imr_reportid,
    		imr_accountdn,
			imr_accountattr,
			imr_accountattrvalue)
		values (
	    	imr_reportid,
			imr_accountdn ,
			imr_accountattr,
			imr_accountattrvalue);

END;
/