-- Copyright (C) 2009, CA, Inc. All rights reserved.
--
-- This file contains the schema for all of the 12.5 SP1
-- tables for snapshot
--
--
-------------------------------------------------------------------

-- Add the Version table
CREATE TABLE imreport_version (
    imr_key nvarchar(255) NOT NULL,
    imr_value nvarchar(255) NOT NULL
    )
go

insert into imreport_version(imr_key,imr_value) values ('version','12.6.2.0.0')
go

--
-- IM Report6
--

CREATE TABLE imreport6 (
    imr_reportid integer IDENTITY(1,1) NOT NULL,
    imr_reportname nvarchar(255) NOT NULL,
    imr_reportdescription nvarchar(628) NULL,
    imr_reporttime datetime NOT NULL DEFAULT GETDATE(),
    imr_reportuser  nvarchar(628) NOT NULL,
    imr_envoid      nvarchar(64) NOT NULL,
    imr_envname     nvarchar(255) NULL,
    imr_keepsnapshot int,
	imr_snapshotsource nvarchar (255),
	imr_snapshotstatus nvarchar (64) NOT NULL,
    CONSTRAINT PK_01_IMREPORT6 PRIMARY KEY  (imr_reportid)
)
go

CREATE INDEX imreport6_index_1 ON imreport6 (imr_reporttime)
go

CREATE INDEX imreport6_index_2 ON imreport6 (imr_reportid, imr_envoid)
go

CREATE INDEX imreport6_index_3 ON imreport6 (imr_reportid, imr_envname)
go

--SP to check the byte length of index and create appropriately, can be re-run by the client later if their is a problem with index

CREATE PROCEDURE sp_imruser6_index_3_exists
AS
BEGIN
DECLARE @MAX_LEN integer
DECLARE @sql_cmd nvarchar(255)
DECLARE @stmt nvarchar(255)
    
    SET @MAX_LEN = (SELECT SUM(max_length)AS TotalIndexKeySize FROM sys.columns WHERE name IN (N'imr_userdn', N'imr_reportid') AND object_id = OBJECT_ID(N'imruser6'))
         IF EXISTS (SELECT name FROM sysindexes WHERE name = 'imruser6_index_3') DROP INDEX imruser6_index_3 on imruser6
    IF (@MAX_LEN > 900)
        CREATE INDEX imruser6_index_3 ON imruser6 (imr_reportid) INCLUDE(imr_userdn)
    ELSE
        CREATE INDEX imruser6_index_3 ON imruser6 (imr_reportid, imr_userdn)
END
GO

CREATE PROCEDURE GetReportIDProc6
    @id INTEGER OUTPUT
AS
BEGIN
    SELECT @id=IDENT_CURRENT('imreport6')
    RETURN
END
GO

-- SP to do the insert into the imreport6 table

CREATE PROCEDURE insertintoimreport
    @imr_reportname nvarchar(255) ,
    @imr_reportdescription nvarchar (628),
    @imr_reportuser  nvarchar(628) ,
    @imr_envoid      nvarchar(64) ,
    @imr_envname     nvarchar(255) ,
    @imr_keepsnapshot int ,
    @imr_snapshotsource nvarchar(255) ,
    @imr_snapshotstatus nvarchar(64) ,
    @current_id INTEGER OUTPUT
AS
BEGIN
 insert into imreport6 (
        imr_reportname ,
        imr_reportdescription,
        imr_reportuser ,
        imr_envoid ,
        imr_envname ,
		imr_keepsnapshot ,
		imr_snapshotsource ,
		imr_snapshotstatus)
            values (
        @imr_reportname ,
        @imr_reportdescription,
        @imr_reportuser ,
        @imr_envoid ,
        @imr_envname ,
		@imr_keepsnapshot ,
		@imr_snapshotsource ,
		@imr_snapshotstatus);

    SELECT @current_id=IDENT_CURRENT('imreport6');
END
GO

--
-- IMR ROLE6
--

CREATE TABLE imrrole6 (
    imr_reportid integer  NOT NULL,
    imr_oid    nvarchar(64) NOT NULL,
    imr_name  nvarchar(255) NULL,
    imr_description   nvarchar(255) NULL,
    imr_type  nvarchar(64) NOT NULL,
    imr_isenabled  int  NULL,
    imr_isassignable int NULL,
    imr_isadminassignable int NULL,
    CONSTRAINT PK_01_IMRROLE6 PRIMARY KEY  (imr_reportid,imr_oid)
)
go

CREATE INDEX imrrole6_index_1 ON imrrole6 (imr_reportid,imr_name)
go


--
-- IMR ROLE RULES 6
-- member, administrator and owner rules
-- add and remove actions
--

CREATE TABLE imrrolerule6 (
    imr_reportid integer  NOT NULL,
    imr_roleoid    nvarchar(64) NOT NULL,
    imr_ruleorder  int DEFAULT (0) NOT  NULL,
    imr_ruletype   nvarchar(64) NULL,
    imr_constraintvalue nvarchar(3800) NULL,

    CONSTRAINT PK_01_IMRROLERULE6 PRIMARY KEY  (imr_reportid,imr_roleoid,imr_ruleorder)
)
go

CREATE INDEX imrrolerule6_index_1 ON imrrolerule6 (imr_reportid,imr_roleoid)
go

--
-- IMR ROLE SCOPE RULES 6
-- scope rules associated with role rules (admin, member and owner rules)
--

CREATE TABLE imrrolescoperule6 (
    imr_reportid integer  NOT NULL,
    imr_roleoid    nvarchar(64) NOT NULL,
    imr_roleruleorder  int DEFAULT (0) NOT NULL,
    imr_scopeobjecttype   nvarchar(64) NOT NULL,
    imr_constraintvalue nvarchar(3800) NULL,

    CONSTRAINT PK_01_IMRROLESCOPERULE6 PRIMARY KEY  (imr_reportid,imr_roleoid,imr_roleruleorder,imr_scopeobjecttype)
)
go

CREATE INDEX imrrolescoperule6_index_1 ON imrrolescoperule6 (imr_reportid,imr_roleoid,imr_roleruleorder)
go



--
-- IMR ROLE TASK 6
-- tasks by role
--

CREATE TABLE imrroletask6 (
      imr_reportid integer NOT NULL,
      imr_roleoid nvarchar(64) NOT NULL,
      imr_taskoid nvarchar(64) NOT NULL,
      CONSTRAINT PK_01_IMRROLETASK6 PRIMARY KEY  (imr_reportid,imr_roleoid,imr_taskoid)
)
go

CREATE INDEX imrroletask6_index_1 ON imrroletask6 (imr_reportid,imr_roleoid)
go


--
-- IMR TASK 6
--

CREATE TABLE imrtask6 (
      imr_reportid integer  NOT NULL,
      imr_oid nvarchar(64) NOT NULL,
      imr_name nvarchar(255) NOT NULL,
      imr_description nvarchar(255) NULL,
      imr_type  nvarchar(64) NOT NULL,
      imr_category nvarchar(255) NULL,
      imr_workflowprocess nvarchar(255) NULL,
      imr_application nvarchar(255) NULL,
      imr_externalurl nvarchar(255) NULL,
      imr_tasktag nvarchar(255) NULL,
      imr_reserved1 nvarchar(255) NULL,
      imr_reserved2 nvarchar(255) NULL,
      imr_reserved3 nvarchar(255) NULL,
      imr_reserved4 nvarchar(255) NULL,
      imr_reserved5 nvarchar(255) NULL,
      imr_reserved6 nvarchar(255) NULL,
      imr_reserved7 nvarchar(255) NULL,
      imr_reserved8 nvarchar(255) NULL,
      imr_reserved9 nvarchar(255) NULL,
      imr_reserved10 nvarchar(255) NULL,
      CONSTRAINT PK_01_IMRTASK6 PRIMARY KEY (imr_reportid,imr_oid)
)
go


--
-- IMR ROLE MEMBER 6
--

CREATE TABLE imrrolemember6 (
    imr_reportid integer NOT NULL,
    imr_roleoid nvarchar(64) NOT NULL,
    imr_userdn nvarchar(628) NOT NULL
)
go

CREATE INDEX imrrolemember6_index_1 ON imrrolemember6 (imr_reportid,imr_roleoid)
go

CREATE INDEX imrrolemember6_index_2 ON imrrolemember6 (imr_reportid)
	INCLUDE (imr_userdn)
go


--
-- IMR ROLEADMIN6
--

CREATE TABLE imrroleadmin6 (
    imr_reportid integer NOT NULL,
    imr_roleoid   nvarchar(64) NOT NULL,
    imr_userdn   nvarchar(628) NOT NULL
)
go

CREATE INDEX imrroleadmin6_index_1 ON imrroleadmin6 (imr_reportid,imr_roleoid)
go

CREATE INDEX imrroleadmin6_index_2 ON imrroleadmin6 (imr_reportid)
	INCLUDE (imr_userdn)
go

--
-- IMR ROLEOWNER6
--

CREATE TABLE imrroleowner6 (
    imr_reportid integer NOT NULL,
    imr_roleoid   nvarchar(64) NOT NULL,
    imr_userdn   nvarchar(628) NOT NULL
)
go

CREATE INDEX imrroleowner6_index_1 ON imrroleowner6 (imr_reportid,imr_roleoid)
go

CREATE INDEX imrroleowner6_index_2 ON imrroleowner6 (imr_reportid)
	INCLUDE (imr_userdn)
go

-- SP to insert members into role tables

CREATE PROCEDURE insertusermembertable
    @tablename varchar(128),
    @imr_reportid integer ,
    @imr_roleoid  nvarchar(64) ,
    @imr_userdn   nvarchar(628)
AS
BEGIN
IF @tablename = 'imrrolemember6'
    IF NOT EXISTS (Select imr_userdn from imrrolemember6
        WHERE
            imr_reportid= @imr_reportid
            AND imr_roleoid= @imr_roleoid
            AND imr_userdn = @imr_userdn)
     insert into imrrolemember6 (
            imr_reportid ,
            imr_roleoid ,
            imr_userdn  )
                values (
            @imr_reportid ,
            @imr_roleoid ,
            @imr_userdn );
IF @tablename = 'imrroleadmin6'
    IF NOT EXISTS (Select imr_userdn from imrroleadmin6
        WHERE
            imr_reportid= @imr_reportid
            AND imr_roleoid= @imr_roleoid
            AND imr_userdn = @imr_userdn)
     insert into imrroleadmin6 (
            imr_reportid ,
            imr_roleoid ,
            imr_userdn  )
                values (
            @imr_reportid ,
            @imr_roleoid ,
            @imr_userdn );
IF @tablename = 'imrroleowner6'
    IF NOT EXISTS (Select imr_userdn from imrroleowner6
        WHERE
            imr_reportid= @imr_reportid
            AND imr_roleoid= @imr_roleoid
            AND imr_userdn = @imr_userdn)
     insert into imrroleowner6 (
            imr_reportid ,
            imr_roleoid ,
            imr_userdn  )
                values (
            @imr_reportid ,
            @imr_roleoid ,
            @imr_userdn );

END
go

--
-- IMR ORG6
--

CREATE TABLE imrorg6 (
    imr_reportid integer  NOT NULL,
    imr_orgdn    nvarchar(628) NOT NULL,
    imr_orgname  nvarchar(255) NULL,
    imr_parentorgdn nvarchar(628) NULL
)
go

CREATE INDEX imrorg6_index_1 ON imrorg6 (imr_reportid)
	INCLUDE (imr_parentorgdn)
go

CREATE PROCEDURE insertintoorgtable
    @imr_reportid integer ,
    @imr_orgdn    nvarchar(628),
    @imr_orgname    nvarchar(255) ,
    @imr_parentorgdn  nvarchar(628)
AS
BEGIN
    IF NOT EXISTS (Select imr_orgdn from imrorg6
        WHERE
            imr_reportid= @imr_reportid
            AND imr_orgdn = @imr_orgdn)
     insert into imrorg6 (
            imr_reportid ,
            imr_orgdn,
            imr_orgname    ,
            imr_parentorgdn
          )
                values (
            @imr_reportid ,
            @imr_orgdn,
            @imr_orgname    ,
            @imr_parentorgdn  );
END
go


--
-- IMR OrgAttr6
--

CREATE TABLE imrorgattr6 (
    imr_reportid integer NOT NULL,
    imr_orgdn    nvarchar(628) NOT NULL,
    imr_attrname   nvarchar(255) NOT NULL,
    imr_attrvalue   nvarchar(628) NULL
)
go

CREATE INDEX imrorgattr6_index_2 ON imrorgattr6 (imr_reportid, imr_attrname)
	INCLUDE (imr_attrvalue)
go

--
-- IMR GROUP6
--

CREATE TABLE imrgroup6 (
    imr_reportid integer  NOT NULL,
    imr_groupdn    nvarchar(628) NOT NULL,
    imr_name   nvarchar(255) NULL,
    imr_description nvarchar(255) NULL,
    imr_parentorgdn   nvarchar(628) NOT NULL,
    imr_selfsubscribing  nvarchar(64) NULL,
    imr_dynamicgroupmembership nvarchar(255) NULL
)
go

CREATE INDEX imrgroup6_index_1 ON imrgroup6 (imr_reportid)
	INCLUDE (imr_parentorgdn)
go

CREATE PROCEDURE insertintogrouptable
    @imr_reportid integer ,
    @imr_groupdn    nvarchar(628) ,
    @imr_name   nvarchar(255) ,
    @imr_description nvarchar(255) ,
    @imr_parentorgdn   nvarchar(628) ,
    @imr_selfsubscribing  nvarchar(64),
    @imr_dynamicgroupmembership nvarchar(255)
AS
BEGIN
    IF NOT EXISTS (Select imr_groupdn from imrgroup6
        WHERE
            imr_reportid= @imr_reportid
            AND imr_groupdn = @imr_groupdn)
        insert into imrgroup6 (
            imr_reportid ,
            imr_groupdn  ,
            imr_name   ,
            imr_description ,
            imr_parentorgdn ,
            imr_selfsubscribing  ,
            imr_dynamicgroupmembership

          )
            values (
            @imr_reportid ,
            @imr_groupdn  ,
            @imr_name   ,
            @imr_description ,
            @imr_parentorgdn ,
            @imr_selfsubscribing  ,
            @imr_dynamicgroupmembership

            );
END
go


--
-- IMR GroupAttr 6
--

CREATE TABLE imrgroupattr6 (
    imr_reportid integer  NOT NULL,
    imr_groupdn    nvarchar(628) NOT NULL,
    imr_attrname   nvarchar(255) NOT NULL,
    imr_attrvalue  nvarchar(628) NULL
)
go

CREATE INDEX imrgroupattr6_index_2 ON imrgroupattr6 (imr_reportid, imr_attrname)
	INCLUDE (imr_attrvalue)
go

--
-- IMR groupmember6
--

CREATE TABLE imrgroupmember6 (
    imr_reportid  integer NOT NULL,
    imr_groupdn    nvarchar(628) NOT NULL,
    imr_memberdn   nvarchar(628) NOT NULL
)
go

CREATE INDEX imrgroupmember6_index_2 ON imrgroupmember6 (imr_reportid)
	INCLUDE (imr_memberdn)
go

--
-- IMR groupadmin6
--

CREATE TABLE imrgroupadmin6 (
    imr_reportid integer NOT NULL,
    imr_groupdn    nvarchar(628) NOT NULL,
    imr_admindn   nvarchar(628) NOT NULL
)
go

CREATE INDEX imrgroupadmin6_index_2 ON imrgroupadmin6 (imr_reportid)
	INCLUDE (imr_admindn)
go

--
-- IMR user6
--

CREATE TABLE imruser6 (
    imr_reportid integer NOT NULL,
    imr_userdn    nvarchar(628) NOT NULL,
    imr_isenabled INTEGER DEFAULT(1) NULL,
    imr_userid        nvarchar(255) NULL,
    imr_fullname  nvarchar(255) NULL,
    imr_lastname  nvarchar(255) NULL,
    imr_firstname nvarchar(255) NULL,
    imr_parentorgdn nvarchar(628) NULL,
    imr_email       nvarchar(255) NULL,
	imr_creationmode integer,
	imr_createdby nvarchar(255),
	imr_creationdate datetime,
	imr_modificationdate datetime,
	imr_lastlogindate datetime,
	imr_usertype nvarchar(255)
)
go

CREATE INDEX imruser6_index_1 ON imruser6 (imr_reportid)
	INCLUDE (imr_parentorgdn)
go

CREATE INDEX imruser6_index_2 ON imruser6 (imr_reportid, imr_userid)
go

CREATE INDEX imruser6_index_3 ON imruser6 (imr_reportid, imr_userdn)
go

-- SP to insert in to the user table

CREATE PROCEDURE insertintousertable
    @imr_reportid integer ,
    @imr_userdn    nvarchar(628),
    @imr_isenabled INTEGER ,
    @imr_userid    nvarchar(255) ,
    @imr_fullname  nvarchar(255) ,
    @imr_lastname  nvarchar(255) ,
    @imr_firstname nvarchar(255) ,
    @imr_parentorgdn nvarchar(628) ,
    @imr_email       nvarchar(255),
	@creationmode INTEGER,
	@createdby nvarchar(255),
	@creationdate DATETIME,
	@modificationdate DATETIME,
	@lastlogindate DATETIME,
	@usertype nvarchar(255)
AS
BEGIN
    IF NOT EXISTS (Select imr_userdn from imruser6
        WHERE
            imr_reportid= @imr_reportid
            AND imr_userdn = @imr_userdn)
     insert into imruser6 (
          imr_reportid ,
          imr_userdn   ,
          imr_isenabled,
          imr_userid   ,
          imr_fullname ,
          imr_lastname ,
          imr_firstname,
          imr_parentorgdn ,
          imr_email,
		  imr_creationmode,
		  imr_createdby,
		  imr_creationdate,
		  imr_modificationdate,
		  imr_lastlogindate,
		  imr_usertype)
                values (
           @imr_reportid ,
           @imr_userdn   ,
           @imr_isenabled,
           @imr_userid   ,
           @imr_fullname ,
           @imr_lastname ,
           @imr_firstname,
           @imr_parentorgdn ,
           @imr_email,
		   @creationmode,
		   @createdby,
		   @creationdate,
		   @modificationdate,
		   @lastlogindate,
		   @usertype);
END
go

--
-- IMR userattr6
--

CREATE TABLE imruserattr6 (
    imr_reportid integer NOT NULL,
    imr_userdn    nvarchar(628) NOT NULL,
    imr_attrname   nvarchar(255) NOT NULL,
    imr_attrvalue  nvarchar(628) NULL
)
go

CREATE  INDEX imruserattr6_index_2 ON imruserattr6 (imr_reportid, imr_attrname)
	INCLUDE (imr_attrvalue)
go

-- add identity policy support

CREATE TABLE imridentitypolicy6 (
    imr_reportid integer  NOT NULL,
    imr_policyid      nvarchar(64) NOT NULL,
    imr_policyname     nvarchar(255) NULL,
    imr_policyindex    int DEFAULT (0) NOT  NULL,
    imr_apptype          nvarchar(64) NOT NULL,
    imr_compliance      nvarchar(64) NOT NULL,
    imr_ruleevaluationtype nvarchar(64) NULL,
    imr_rule   nvarchar(3000) NULL,
    imr_policysetid   nvarchar(64) NOT NULL,
    CONSTRAINT PK_01_imridentitypolicy6 PRIMARY KEY  (imr_reportid, imr_policyid, imr_policysetid)
    )
go

CREATE INDEX imridentitypolicy6_index_1 ON imridentitypolicy6 (imr_reportid, imr_policyid)
go

CREATE INDEX imridentitypolicy6_index_2 ON imridentitypolicy6 (imr_reportid, imr_policysetid)
go

CREATE INDEX imridentitypolicy6_index_3 ON imridentitypolicy6 (imr_reportid, imr_policyid,imr_policysetid)
go

--
-- IMR imridentitypolicyset6
--

CREATE TABLE imridentitypolicyset6 (
    imr_reportid integer  NOT NULL,
    imr_policysetid     nvarchar(64) NOT NULL,
    imr_policysetname    nvarchar(255) NOT NULL,
    imr_category    nvarchar(255) NOT NULL,
    CONSTRAINT PK_01_imridentitypolicyset6 PRIMARY KEY  (imr_reportid, imr_policysetid)
    )
go


CREATE INDEX imridentitypolicyset6_index_1 ON imridentitypolicyset6 (imr_reportid, imr_policysetid)
go

--
-- IM imridentitypolicyactions6
--

CREATE TABLE imridentitypolicyactions6 (
    imr_reportid integer  NOT NULL,
    imr_policyid      nvarchar(64) NOT NULL,
    imr_actiontype    nvarchar(65) NOT NULL,
    imr_actionindex    int DEFAULT (0) NOT  NULL,
    imr_actionevaluationtype    nvarchar(65) NOT NULL,
    imr_action   nvarchar(3000) NOT NULL,

    CONSTRAINT PK_01_imridentitypolicyactions6 PRIMARY KEY  (imr_reportid, imr_policyid, imr_actiontype, imr_actionindex)
    )
go

CREATE INDEX imridentitypolicyactions6_index_1 ON imridentitypolicyactions6 (imr_reportid, imr_policyid, imr_actiontype)
go

--
-- IM imridentitypolicyusersync6
--

CREATE TABLE imridentitypolicyusersync6 (
    imr_reportid        integer  NOT NULL,
    imr_policysetname    nvarchar(255) NOT NULL,
    imr_policyid       nvarchar(64) NOT NULL,
    imr_userdn          nvarchar(255) NOT NULL,
    imr_syncstatus      nvarchar(255) NOT NULL,
    CONSTRAINT PK_01_imrpolicyusersync6 PRIMARY KEY  (imr_reportid, imr_policyid, imr_userdn)
    )
go

CREATE INDEX imridentitypolicyusersync6_index_1 ON imridentitypolicyusersync6 (imr_reportid)
	INCLUDE (imr_userdn)
go

CREATE INDEX imridentitypolicyusersync6_index_2 ON imridentitypolicyusersync6 (imr_reportid, imr_policyid)
go

CREATE VIEW imrroleinfo6
AS
SELECT     TOP 100 PERCENT imrrole6.imr_reportid, imrrole6.imr_oid, imrrole6.imr_name, imrrole6.imr_type,
                      imruser6.imr_userdn, imrrolemember6.imr_userdn AS member, imrroleowner6.imr_userdn AS owner,
                      imrroleadmin6.imr_userdn AS admin
FROM         imrrole6 INNER JOIN
                      imruser6 ON imrrole6.imr_reportid = imruser6.imr_reportid AND
                      imrrole6.imr_reportid = imruser6.imr_reportid LEFT OUTER JOIN
                      imrroleowner6 ON imrrole6.imr_reportid = imrroleowner6.imr_reportid AND
                      imrroleowner6.imr_userdn = imruser6.imr_userdn AND imrroleowner6.imr_reportid = imruser6.imr_reportid AND
                      imrroleowner6.imr_roleoid = imrrole6.imr_oid LEFT OUTER JOIN
                      imrrolemember6 ON imrrole6.imr_reportid = imrrolemember6.imr_reportid AND
                      imrrolemember6.imr_userdn = imruser6.imr_userdn AND
                      imrrolemember6.imr_reportid = imruser6.imr_reportid AND
                      imrrolemember6.imr_roleoid = imrrole6.imr_oid LEFT OUTER JOIN
                      imrroleadmin6 ON imrrole6.imr_reportid = imrroleowner6.imr_reportid AND
                      imrroleadmin6.imr_userdn = imruser6.imr_userdn AND imrroleadmin6.imr_reportid = imruser6.imr_reportid AND
                      imrroleadmin6.imr_roleoid = imrrole6.imr_oid
WHERE     (NOT (imrrolemember6.imr_userdn IS NULL AND imrroleowner6.imr_userdn IS NULL AND
                      imrroleadmin6.imr_userdn IS NULL))
ORDER BY imrrole6.imr_reportid

go

--
-- IM imrptsnapshots12
--

CREATE TABLE imrptsnapshots12 (
    imr_rptname 		nvarchar(255) NOT NULL,
    imr_snapshotname          	nvarchar(255) NOT NULL,
    imr_envname			nvarchar(255) NOT NULL,
    IMR_RPT_TASK_TAG_NAME nvarchar(255) NOT NULL
    )
go

CREATE INDEX imrptsnapshots12_index_1 ON imrptsnapshots12 (imr_rptname)
	INCLUDE (imr_snapshotname, imr_envname)
go



------------------------------------------------------ 
-- New r12.5 support.
------------------------------------------------------ 
CREATE TABLE [imrAttribute] (
	[imr_reportid] [int] NOT NULL ,
	[imr_objectid] [int] NOT NULL ,
	[imr_attrinfoid] [int] NOT NULL ,
	[imr_value] nvarchar(max) NULL ,
	[imr_objectrefid] [int] NULL 
) ON [PRIMARY]
GO

CREATE INDEX imrattribute_index_1 ON imrattribute (imr_reportid, imr_attrinfoid, imr_objectid)
go

CREATE INDEX imrattribute_index_2 ON imrattribute (imr_reportid, imr_objectid)
go

CREATE INDEX imrattribute_index_3 ON imrattribute (imr_reportid)
INCLUDE(imr_value)
go

CREATE TABLE [imrAttributeInfo] (
	[imr_reportid] [int] NOT NULL ,
	[imr_attrinfoid] [int] IDENTITY (1, 1) NOT NULL ,
	[imr_syntaxid] [int] NOT NULL ,
	[imr_name] [nvarchar] (64) NOT NULL ,
	[imr_objectinfoid] [int] NOT NULL ,
	[imr_isbasetype] [nvarchar] (256) NULL 
) ON [PRIMARY]
GO

CREATE INDEX imrattributeinfo_index_1 ON imrattributeinfo (imr_reportid, imr_name)
go

CREATE TABLE [imrObject] (
	[imr_reportid] [int] NOT NULL ,
	[imr_friendlyname] [nvarchar] (255) NOT NULL ,
	[imr_objectdn] [nvarchar] (628) NOT NULL ,
	[imr_objectinfoname] [nvarchar] (255) NOT NULL ,
	[imr_objectid] [int] IDENTITY (1, 1) NOT NULL 
) ON [PRIMARY]
GO

CREATE INDEX imrobject_index_1 ON imrobject (imr_reportid)
	INCLUDE(imr_friendlyname)
go

CREATE INDEX imrobject_index_2 ON imrobject (imr_reportid)
	INCLUDE(imr_objectdn)
go

CREATE TABLE [imrObjectCategory] (
	[imr_reportid] [int] NOT NULL ,
	[imr_name] [nvarchar] (255) NOT NULL 
) ON [PRIMARY]
GO

CREATE INDEX imrobjectcategory_index_1 ON imrobjectcategory (imr_reportid, imr_name)
go

CREATE TABLE [imrObjectInfo] (
	[imr_reportid] [int] NOT NULL ,
	[imr_objectid] [int] IDENTITY (1, 1) NOT NULL ,
	[imr_name] [nvarchar] (255) NOT NULL ,
	[imr_objectcatname] [nvarchar] (255) NOT NULL 
) ON [PRIMARY]
GO

CREATE INDEX imrobjectinfo_index_1 ON imrobjectinfo (imr_reportid, imr_name)
go

CREATE TABLE [imrProvRolesRelationship] (
	[imr_reportid] [int] NOT NULL ,
	[imr_parentoid] [nvarchar] (64) NOT NULL ,
	[imr_childoid] [nvarchar] (64) NOT NULL 
) ON [PRIMARY]
GO

CREATE INDEX imrprovrolesrel_index_1 ON imrProvRolesRelationship (imr_reportid, imr_parentoid, imr_childoid)
go

CREATE TABLE imrProvRoleTemplRelationship (
	imr_reportid int NOT NULL ,
    imr_roleoid    nvarchar(64) NOT NULL,
	imr_roledn nvarchar(628) NOT NULL ,
	imr_templatedn nvarchar(628) NOT NULL 
)
GO

CREATE INDEX imrprovroletemplrel_index_1 ON imrprovroletemplrelationship (imr_reportid, imr_roleoid)
	INCLUDE(imr_templatedn, imr_roledn)
go

CREATE TABLE [imrUserAccountRelationship] (
	[imr_reportid] [int] NOT NULL ,
	[imr_useraccountdn] [nvarchar] (1024) NOT NULL ,
	[imr_provuserdn] [nvarchar] (1024) NOT NULL ,
	[imr_userid] [nvarchar] (1024) NOT NULL ,
) ON [PRIMARY]
GO

CREATE INDEX imruseraccountrel_index_1 ON imruseraccountrelationship (imr_reportid)
	INCLUDE (imr_useraccountdn)
go

CREATE INDEX imruseraccountrel_index_2 ON imruseraccountrelationship (imr_reportid)
	INCLUDE (imr_provuserdn)
go

CREATE INDEX imruseraccountrel_index_3 ON imruseraccountrelationship (imr_reportid)
	INCLUDE (imr_userid)
go

-- Procedures to insert in to the provisioning tables


CREATE PROCEDURE insertimrAttrInfo 
	@imr_reportid integer,
	@imr_syntaxid integer,
	@imr_name nvarchar(64),
	@imr_objectinfoid integer,
	@imr_isbasetype char(1),    
	@infoid INTEGER OUTPUT
AS
BEGIN
SET @infoid = 0;
Select @infoid =imr_attrinfoid  from imrAttributeInfo
        WHERE
           ( imr_reportid= @imr_reportid
            AND imr_name  = @imr_name
            AND imr_objectinfoid  = @imr_objectinfoid ) ;

  IF not  @infoid >0
       BEGIN
 	INSERT into imrAttributeInfo(
            imr_reportid, 
            imr_syntaxid,
            imr_name,
            imr_objectinfoid,
	imr_isbasetype		
		) 
            values (
	@imr_reportid, 
            	@imr_syntaxid,
            	@imr_name,
            	@imr_objectinfoid,
	@imr_isbasetype
		) 
      SELECT @infoid=IDENT_CURRENT('imrAttributeInfo');
    END
END
GO

CREATE PROCEDURE insertimrobject
    @imr_reportid integer ,
    @imr_friendlyname   nvarchar(255),
    @imr_objectdn   nvarchar(628), 
    @imr_objectinfoname   nvarchar(255),
    @objectid INTEGER OUTPUT

AS
BEGIN
SET @objectid = 0;
Select @objectid = imr_objectid  from imrObject
        WHERE
            imr_reportid= @imr_reportid
            AND imr_objectdn = @imr_objectdn;

    IF not  @objectid >0
     BEGIN
        insert into imrObject (
            imr_reportid ,
            imr_friendlyname  ,
            imr_objectdn,
	imr_objectinfoname

          )
            values (
            @imr_reportid ,
            @imr_friendlyname ,
	@imr_objectdn,
	@imr_objectinfoname

            )
	SELECT @objectid=IDENT_CURRENT('imrObject');
      END
END
GO

CREATE PROCEDURE insertimrobjectcategory
    @imr_reportid integer ,
    @imr_name   nvarchar(255) 

AS
BEGIN
    IF NOT EXISTS (Select imr_name  from imrObjectCategory
        WHERE
            imr_reportid= @imr_reportid
            AND imr_name = @imr_name)
        insert into imrObjectCategory (
            imr_reportid ,
            imr_name  

          )
            values (
            @imr_reportid ,
            @imr_name 

            );
END
GO

CREATE PROCEDURE insertimrobjectinfo
    @imr_reportid integer ,
    @imr_name   nvarchar(255),
    @imr_objectcatname   nvarchar(255) ,
    @infoid INTEGER OUTPUT

AS
BEGIN
SET @infoid = 0;
Select @infoid=imr_objectid  from imrObjectInfo
        WHERE
            imr_reportid= @imr_reportid
            AND imr_name = @imr_name
			AND imr_objectcatname = @imr_objectcatname;

IF not  @infoid >0
    BEGIN
        insert into imrObjectInfo (
            imr_reportid ,
            imr_name  ,
	    imr_objectcatname

          )
            values (
            @imr_reportid ,
            @imr_name ,
	@imr_objectcatname

            )
         SELECT @infoid=IDENT_CURRENT('imrObjectInfo');
   END

END
GO

CREATE PROCEDURE insertimrroletemplate
    @imr_reportid integer ,
	@imr_roleoid nvarchar(64),
    @imr_roledn   nvarchar(628),
    @imr_templatedn   nvarchar(628) 

AS
BEGIN
    IF NOT EXISTS (Select imr_roledn from imrProvRoleTemplRelationship
        WHERE
            imr_reportid= @imr_reportid
			and imr_roleoid = @imr_roleoid
            AND imr_roledn = @imr_roledn
   	    AND imr_templatedn = @imr_templatedn)
        insert into imrProvRoleTemplRelationship(
            imr_reportid ,
			imr_roleoid,
            imr_roledn  ,
	    imr_templatedn

          )
            values (
            @imr_reportid ,
			@imr_roleoid,
            @imr_roledn ,
	    @imr_templatedn

            );
END
GO

CREATE PROCEDURE insertTemplEndpointRel
	@imr_reportid integer,
	@imr_templatedn nvarchar(628),
	@imr_endpointname nvarchar(628),
	@imr_endpointdn nvarchar(628)
AS
BEGIN
	IF NOT EXISTS (select imr_reportid from imrTemplEndpointRelationship
		where
			imr_reportid=@imr_reportid
			and imr_templatedn=@imr_templatedn
			and imr_endpointname=@imr_endpointname
			and imr_endpointdn=@imr_endpointdn)
			insert into imrTemplEndpointRelationship(
				imr_reportid,
				imr_templatedn,
				imr_endpointname,
				imr_endpointdn
			)
			values(
				@imr_reportid,
				@imr_templatedn,
				@imr_endpointname,
				@imr_endpointdn
			);
END
GO

CREATE PROCEDURE insertAccountTemplateRel
	@imr_reportid integer,
	@imr_name nvarchar(628),
	@imr_templatedn nvarchar(628),
	@imr_endpointtype nvarchar(628)
AS
BEGIN
	IF NOT EXISTS (select imr_reportid from imrAccountTemplateRel
		where
			imr_reportid=@imr_reportid
			and imr_name=@imr_name
			and imr_templatedn=@imr_templatedn
			and imr_endpointtype=@imr_endpointtype)
			insert into imrAccountTemplateRel(
				imr_reportid,
				imr_name,
				imr_templatedn,
				imr_endpointtype
			)
			values(
				@imr_reportid,
				@imr_name,
				@imr_templatedn,
				@imr_endpointtype
			);
END
GO

CREATE PROCEDURE insertProvRolesRel
	@imr_reportid integer,
	@imr_parentoid nvarchar(64),
	@imr_childoid nvarchar(64)
AS
BEGIN
	IF NOT EXISTS(select imr_reportid from imrProvRolesRelationship
		where 
			imr_reportid=@imr_reportid
			and imr_parentoid=@imr_parentoid
			and imr_childoid=@imr_childoid)
			insert into imrProvRolesRelationship(
				imr_reportid,
				imr_parentoid,
				imr_childoid
			)
			values(
				@imr_reportid,
				@imr_parentoid,
				@imr_childoid
			);
			
END
GO

--
-- IM imrTemplEndpointRelationship
--
CREATE TABLE imrTemplEndpointRelationship (
    imr_reportid integer  NOT NULL,
	imr_templatedn nvarchar(628) ,
	imr_endpointname nvarchar(628) ,
	imr_endpointdn nvarchar(628) 
    )
go

CREATE INDEX imrtemplendpointrel_index_1 ON imrtemplendpointrelationship (imr_reportid)
	INCLUDE(imr_templatedn, imr_endpointname)
go

--
-- IM imrAccount
--

CREATE TABLE imrAccountTemplateRel (
    imr_reportid integer  NOT NULL,
	imr_name nvarchar(628) NOT NULL,
	imr_templatedn nvarchar(628),
	imr_endpointtype nvarchar(628) 
    )
go

CREATE INDEX imrAccountTemplateRel ON imrAccountTemplateRel (imr_reportid)
	INCLUDE(imr_name,imr_templatedn,imr_endpointtype)
go

CREATE TABLE imrCapabilityAttrRel (
    imr_reportid integer  NOT NULL,
	imr_endpointtype nvarchar(628) not null,
	imr_objecttypename nvarchar(628) NOT NULL,
	imr_jiamobjecttypename nvarchar(628) NOT NULL,
	imr_attrid integer,
	imr_capabilityobjectname nvarchar(628) NOT NULL,
	imr_capabilityobjectdn nvarchar(628) NOT NULL,
	imr_endpointobjid integer,
	imr_name nvarchar(64),
	imr_value nvarchar(max)
    )
go

CREATE INDEX imrcapabilityattrrel_index_1 ON imrcapabilityattrrel (imr_reportid)
	INCLUDE(imr_endpointtype, imr_endpointobjid, imr_objecttypename, imr_capabilityobjectdn, imr_name)
go

CREATE TABLE imrroleattr12(
	imr_reportid int NOT NULL,
	imr_roleoid nvarchar(64) NOT NULL,
	imr_name nvarchar(628) NOT NULL,
	imr_value nvarchar(max)  NULL
) 
GO

CREATE INDEX imrroleattr12_index_1 ON imrroleattr12 (imr_reportid, imr_roleoid)
	INCLUDE(imr_name)
go

--
-- IM imrCapabilityAttrRel
--

CREATE PROCEDURE insertCapabilityAttrRel
	@imr_reportid integer,
	@imr_endpointtype nvarchar(628),
	@imr_objecttypename nvarchar(628),
	@imr_jiamobjecttypename nvarchar(628) ,
	@imr_attrid integer,
	@imr_capabilityobjectname nvarchar(628),
	@imr_capabilityobjectdn nvarchar(628),
	@imr_endpointobjid integer,
	@imr_name nvarchar(64),
	@imr_value nvarchar(max)
	
AS
BEGIN
	IF NOT EXISTS (select imr_reportid from imrCapabilityAttrRel
		where
			imr_reportid=@imr_reportid
			and imr_endpointtype=@imr_endpointtype
			and imr_objecttypename=@imr_objecttypename
			and imr_endpointtype=@imr_endpointtype
			and imr_jiamobjecttypename=@imr_jiamobjecttypename
			and imr_attrid=@imr_attrid
			and imr_capabilityobjectname=@imr_capabilityobjectname
			and imr_capabilityobjectdn=@imr_capabilityobjectdn
			and imr_endpointobjid=@imr_endpointobjid 
			and imr_name=@imr_name)
			insert into imrCapabilityAttrRel(
				imr_reportid,
				imr_endpointtype,
				imr_objecttypename,
				imr_jiamobjecttypename,
				imr_attrid,
				imr_capabilityobjectname,
				imr_capabilityobjectdn,
				imr_endpointobjid,
				imr_name,
				imr_value
			)
			values(
				@imr_reportid,
				@imr_endpointtype,
				@imr_objecttypename,
				@imr_jiamobjecttypename,
				@imr_attrid,
				@imr_capabilityobjectname,
				@imr_capabilityobjectdn,
				@imr_endpointobjid,
				@imr_name,
				@imr_value
			);
END
GO

CREATE PROCEDURE insertintoimrroleattr12
	@imr_reportid integer,
	@imr_roleoid nvarchar(64),
	@imr_name nvarchar(256),
	@imr_value nvarchar(max)
AS
BEGIN
	IF NOT EXISTS (select imr_name from imrroleattr12
		where
			imr_reportid = @imr_reportid
			and imr_roleoid = @imr_roleoid
			and imr_name = @imr_name)
			insert into imrroleattr12(
				imr_reportid,
				imr_roleoid,
				imr_name,
				imr_value
			)
			values(
				@imr_reportid,
				@imr_roleoid,
				@imr_name,
				@imr_value
			);
END
GO

-- ---- Views

CREATE VIEW imruseraccounts6
AS
SELECT 	accountObject.imr_reportid as imr_reportid,
	imruser.imr_userid as imr_userid,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn,
	SUBSTRING(accountObject.imr_objectdn,
              CHARINDEX(',EndPoint=',accountObject.imr_objectdn) + LEN(',EndPoint='),
			  CASE WHEN CHARINDEX(',Namespace=',accountObject.imr_objectdn) > 0
			  THEN
                  CHARINDEX(',Namespace=',accountObject.imr_objectdn) - CHARINDEX(',EndPoint=',accountObject.imr_objectdn) - LEN(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObject.imr_objectinfoname as imr_accounttype,
	accountAttr1.imr_value + ' ' + REPLACE(accountAttr2.imr_value,'.000','') as imr_createtime
FROM imrObject accountObject,
	imrUserAccountRelationship userAccountRelationship,
	imrAttributeInfo accountAttrInfo1,
    imrAttributeInfo accountAttrInfo2,
	imrAttribute accountAttr1,
    imrAttribute accountAttr2,
	imruser6 imruser
WHERE imruser.imr_userid = userAccountRelationship.imr_userid and
	accountObject.imr_objectdn = userAccountRelationship.imr_useraccountdn and
	accountAttr1.imr_attrinfoid = accountAttrInfo1.imr_attrinfoid and
	accountAttrInfo1.imr_name='createDate' and
    accountAttr2.imr_attrinfoid = accountAttrInfo2.imr_attrinfoid and
	accountAttrInfo2.imr_name='createTime' and
	accountObject.imr_objectid = accountAttr1.imr_objectid and
    accountObject.imr_objectid = accountAttr2.imr_objectid and
	accountObject.imr_reportid=imruser.imr_reportid and
	userAccountRelationship.imr_reportid = imruser.imr_reportid and
	accountAttrInfo1.imr_reportid = imruser.imr_reportid and
    accountAttrInfo2.imr_reportid = imruser.imr_reportid and
	accountAttr1.imr_reportid = imruser.imr_reportid and
    accountAttr2.imr_reportid = imruser.imr_reportid;
GO

CREATE VIEW imraccountattrs6
AS
SELECT
	accountObject.imr_reportid as imr_reportid,
	accountObject.imr_objectid as accountObjid,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn,
    SUBSTRING(accountObject.imr_objectdn,
              CHARINDEX(',EndPoint=',accountObject.imr_objectdn) + LEN(',EndPoint='),
			  CASE WHEN CHARINDEX(',Namespace=',accountObject.imr_objectdn) > 0
			  THEN
                  CHARINDEX(',Namespace=',accountObject.imr_objectdn) - CHARINDEX(',EndPoint=',accountObject.imr_objectdn) - LEN(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	resourceAttrInfo.imr_name as imr_accountattr,
	resourceAttr.imr_value as imr_accountattrvalue,
	resourceAttr.imr_objectrefid as imr_objectrefid,
	resourceAttrInfo.imr_isbasetype as imr_isbasetype
FROM
	imrObjectCategory accountCategory,
	imrObjectInfo accountObjectInfo,
	imrObject accountObject,
	imrAttributeInfo resourceAttrInfo,
	imrAttribute resourceAttr
WHERE
	(resourceAttr.imr_value IS NOT NULL or
	resourceAttr.imr_objectrefid IS NOT NULL) and
	resourceAttr.imr_objectid = accountObject.imr_objectid and
	resourceAttrInfo.imr_attrinfoid = resourceAttr.imr_attrinfoid and
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = accountCategory.imr_name and
	accountCategory.imr_name = 'Account' and
	accountCategory.imr_reportid=accountObject.imr_reportid and
	accountObjectInfo.imr_reportid=accountObject.imr_reportid and
	resourceAttrInfo.imr_reportid=accountObject.imr_reportid and
	resourceAttr.imr_reportid=accountObject.imr_reportid;
GO

CREATE VIEW endpointaccountattrs6
AS
SELECT
	accountObject.imr_reportid as imr_reportid,
	accountObject.imr_objectid as imr_accountid,
	accountObject.imr_friendlyname as imr_accountname,
    accountObject.imr_objectdn as imr_accountdn,
    SUBSTRING(accountObject.imr_objectdn,
              CHARINDEX(',EndPoint=',accountObject.imr_objectdn) + LEN(',EndPoint='),
			  CASE WHEN CHARINDEX(',Namespace=',accountObject.imr_objectdn) > 0
			  THEN
                  CHARINDEX(',Namespace=',accountObject.imr_objectdn) - CHARINDEX(',EndPoint=',accountObject.imr_objectdn) - LEN(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	attr1.imr_value + ' ' + REPLACE(attr2.imr_value,'.000','') as imr_createtime,
	attr3.imr_value + ' ' + REPLACE(attr4.imr_value,'.000','') as imr_accountlastmodified
FROM
	imrObjectInfo accountObjectInfo,
	imrObject accountObject,
	imrAttributeInfo attrInfo1,
	imrAttribute attr1,
	imrAttributeInfo attrInfo2,
	imrAttribute attr2,
    imrAttributeInfo attrInfo3,
	imrAttribute attr3,
	imrAttributeInfo attrInfo4,
	imrAttribute attr4
WHERE
	((attr1.imr_value IS NOT NULL or attr2.imr_value IS NOT NULL) or
	 (attr3.imr_value IS NOT NULL or attr4.imr_value IS NOT NULL)) and
	attr1.imr_objectid = accountObject.imr_objectid and
	attr2.imr_objectid = accountObject.imr_objectid and
	attr3.imr_objectid = accountObject.imr_objectid and
	attr4.imr_objectid = accountObject.imr_objectid and
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = 'Account' and
	(attr1.imr_attrinfoid = attrInfo1.imr_attrinfoid and
	attrInfo1.imr_name = 'createDate') and
    (attr2.imr_attrinfoid = attrInfo2.imr_attrinfoid and
	attrInfo2.imr_name = 'createTime') and
	(attr3.imr_attrinfoid = attrInfo3.imr_attrinfoid and
	attrInfo3.imr_name = 'updateDate') and
    (attr4.imr_attrinfoid = attrInfo4.imr_attrinfoid and
	attrInfo4.imr_name = 'updateTime') and
	accountObjectInfo.imr_reportid=accountObject.imr_reportid and
	attrInfo1.imr_reportid=accountObject.imr_reportid and
	attr1.imr_reportid=accountObject.imr_reportid and
	attrInfo2.imr_reportid=accountObject.imr_reportid and
	attr2.imr_reportid=accountObject.imr_reportid and
    attrInfo3.imr_reportid=accountObject.imr_reportid and
	attr3.imr_reportid=accountObject.imr_reportid and
	attrInfo4.imr_reportid=accountObject.imr_reportid and
	attr4.imr_reportid=accountObject.imr_reportid;
GO


CREATE VIEW endpointuseraccounts6
AS
SELECT 
	a.imr_reportid imr_reportid, 
	b.imr_userid imr_userid, 
	a.imr_accountname imr_accountname, 
	a.imr_accountdn imr_accountdn, 
	a.imr_endpoint as imr_endpoint,
	a.imr_endpointtype imr_accounttype, 
	a.imr_createtime imr_createtime,
	a.imr_accountlastmodified imr_accountlastmodified,
	c.imr_firstname imr_firstname,
	c.imr_lastname imr_lastname,
	c.imr_fullname imr_fullname
FROM  
	endpointaccountattrs6 a, 
	imrUserAccountRelationship b,
	imruser6 c 
WHERE 
	b.imr_useraccountdn = a.imr_accountdn
	and b.imr_reportid = a.imr_reportid
	and c.imr_reportid = a.imr_reportid
	and c.imr_userid=b.imr_userid;
GO

CREATE VIEW endpointattributes
AS
SELECT Attribute.imr_reportid as imr_reportid,  
	Attribute.imr_objectid as imr_endpointid, 
	Attribute.imr_attrInfoID as AttributeInfoID, AttributeInfo.imr_name as Name, 
	Attribute.imr_value as AttributeValue 
FROM 
	imrattribute Attribute, 
	imrattributeinfo AttributeInfo  
WHERE imr_objectid IN (SELECT endpointObject.imr_objectid
		FROM
			imrObject endpointObject,
			imrObjectCategory endpointCategory,
			imrObjectInfo endpointObjectInfo
		WHERE
			endpointObject.imr_objectinfoname = endpointObjectInfo.imr_name and
			endpointObjectInfo.imr_objectcatname = endpointCategory.imr_name and
			endpointCategory.imr_name = 'EndPoint' and 
			endpointCategory.imr_reportid = endpointObject.imr_reportid and 
			endpointObjectInfo.imr_reportid = endpointCategory.imr_reportid and
			endpointObject.imr_reportid = endpointCategory.imr_reportid)
AND AttributeInfo.imr_attrinfoid = Attribute.imr_attrinfoid;

GO

CREATE VIEW accttmplattribute 
AS
SELECT
	acctTmplObject.imr_reportid as imr_reportid,
	acctTmplObject.imr_objectid as acctTmplObjectid,
	acctTmplObject.imr_friendlyname as imr_accttmplname,
	acctTmplObject.imr_objectdn as imr_accttmpldn,
	acctTmplObjectInfo.imr_name as imr_endpointtype,
	resourceAttrInfo.imr_name as imr_attrname,
	resourceAttr.imr_value as imr_attrvalue,
	resourceAttr.imr_objectrefid as imr_objectrefid,
	resourceAttrInfo.imr_isbasetype as imr_isbasetype
FROM
	imrObjectCategory acctTmplCategory,
	imrObjectInfo acctTmplObjectInfo,
	imrObject acctTmplObject,
	imrAttributeInfo resourceAttrInfo,
	imrAttribute resourceAttr
WHERE
	(resourceAttr.imr_value IS NOT NULL or
	resourceAttr.imr_objectrefid IS NOT NULL) and
	resourceAttr.imr_objectid = acctTmplObject.imr_objectid and
	resourceAttrInfo.imr_attrinfoid = resourceAttr.imr_attrinfoid and
	acctTmplObject.imr_objectinfoname = acctTmplObjectInfo.imr_name and
	acctTmplObjectInfo.imr_objectcatname = acctTmplCategory.imr_name and
	acctTmplCategory.imr_name = 'Policy' and 
	acctTmplCategory.imr_reportid=acctTmplObject.imr_reportid and 
	acctTmplObjectInfo.imr_reportid = acctTmplObject.imr_reportid and 
	resourceAttrInfo.imr_reportid = acctTmplObject.imr_reportid and 
	resourceAttr.imr_reportid = acctTmplObject.imr_reportid;
GO

CREATE VIEW endpointaccount
AS
SELECT
	accountObject.imr_reportid as imr_reportid,
    SUBSTRING(accountObject.imr_objectdn,
              CHARINDEX(',EndPoint=',accountObject.imr_objectdn) + LEN(',EndPoint='),
			  CASE WHEN CHARINDEX(',Namespace=',accountObject.imr_objectdn) > 0
			  THEN
                  CHARINDEX(',Namespace=',accountObject.imr_objectdn) - CHARINDEX(',EndPoint=',accountObject.imr_objectdn) - LEN(',EndPoint=')
			  ELSE
                 0
              END
             ) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn
FROM
	imrObject accountObject,
	imrObjectInfo accountObjectInfo
WHERE
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = 'Account' and
	accountObject.imr_objectdn like 'Account=%' and 
	accountObject.imr_reportid=accountObjectInfo.imr_reportid;
GO

CREATE VIEW endpointview 
AS
SELECT
	endpointObject.imr_reportid as imr_reportid,
	endpointObject.imr_friendlyname as imr_endpoint,
	endpointObject.imr_objectdn as imr_endpointdn,
	endpointObject.imr_objectid as imr_endpointid,
	endpointObjectInfo.imr_name as imr_endpointtype
FROM
	imrObject endpointObject,
	imrObjectCategory endpointCategory,
	imrObjectInfo endpointObjectInfo
WHERE
	endpointObject.imr_objectinfoname = endpointObjectInfo.imr_name and
	endpointObjectInfo.imr_objectcatname = endpointCategory.imr_name and
	endpointCategory.imr_name = 'EndPoint' and 
	endpointObject.imr_objectdn like 'EndPoint=%' and 
	endpointCategory.imr_reportid = endpointObject.imr_reportid and 
	endpointObjectInfo.imr_reportid = endpointObject.imr_reportid;
GO

CREATE VIEW provrolesrelationshipview 
AS
SELECT
	provRolesRelationship.imr_reportid as imr_reportid,
	provRolesRelationship.imr_parentoid as imr_parentoid,
	provRolesRelationship.imr_childoid as imr_childoid
FROM
	imrProvRolesRelationship provRolesRelationship
GO

CREATE VIEW accttemplEndpointRelView
AS
SELECT     
	imrObjectInfo.imr_name AS TemplateType, imrObject.imr_objectdn AS TemplateDN, 		     imrObject.imr_friendlyname AS TemplateName, 
    imrTemplEndpointRelationship.imr_endpointname AS EndpointName,  	imrTemplEndpointRelationship.imr_endpointdn AS EndpointDN, 
    imrObject.imr_reportid
FROM         
	imrTemplEndpointRelationship RIGHT OUTER JOIN
    imrObject ON imrTemplEndpointRelationship.imr_reportid = imrObject.imr_reportid AND 
    imrTemplEndpointRelationship.imr_templatedn = imrObject.imr_objectdn LEFT OUTER JOIN
    imrObjectCategory INNER JOIN
    imrObjectInfo ON imrObjectCategory.imr_name = imrObjectInfo.imr_objectcatname ON 	imrObject.imr_reportid = imrObjectCategory.imr_reportid AND 
    imrObject.imr_reportid = imrObjectInfo.imr_reportid AND imrObject.imr_objectinfoname = 	imrObjectInfo.imr_name
WHERE     
	(imrObjectCategory.imr_name = 'Policy')
GO

CREATE VIEW capabilityattrsview
AS
SELECT     
imrCapabilityAttrRel.imr_reportid, imrCapabilityAttrRel.imr_endpointtype AS EndpointType, imrObject.imr_friendlyname AS EndpointName, 
                      imrObject.imr_objectdn AS EndpointDN, imrCapabilityAttrRel.imr_objecttypename, imrCapabilityAttrRel.imr_jiamobjecttypename, 
                      imrCapabilityAttrRel.imr_capabilityobjectname, imrCapabilityAttrRel.imr_capabilityobjectdn, imrCapabilityAttrRel.imr_endpointobjid AS endpointid, imrCapabilityAttrRel.imr_attrid AS Attributeid, imrCapabilityAttrRel.imr_name, imrCapabilityAttrRel.imr_value
FROM imrCapabilityAttrRel, imrObject where 
	imrCapabilityAttrRel.imr_reportid = imrObject.imr_reportid
	and imrCapabilityAttrRel.imr_endpointobjid = imrObject.imr_objectid
GO

CREATE VIEW provroleaccttmplendptrelview
AS
SELECT     
	imrProvRoleTemplRelationship.imr_reportid, 
	imrProvRoleTemplRelationship.imr_roleoid,
	imrProvRoleTemplRelationship.imr_roledn,
	accttemplEndpointRelView.TemplateType,
	accttemplEndpointRelView.TemplateDN,
	accttemplEndpointRelView.TemplateName,
	accttemplEndpointRelView.EndpointName,
	accttemplEndpointRelView.EndpointDN
		
FROM         
	imrProvRoleTemplRelationship LEFT OUTER JOIN
    accttemplEndpointRelView ON imrProvRoleTemplRelationship.imr_reportid = accttemplEndpointRelView.imr_reportid 
	AND imrProvRoleTemplRelationship.imr_templatedn = accttemplEndpointRelView.TemplateDN
go


------------------------------------------------------
-- New r12.5 SP1 support.
------------------------------------------------------

--
-- IMR imrpolicyxpress125
--

CREATE TABLE imrpolicyxpress125 (
    imr_reportid integer  NOT NULL,
    imr_policyid     nvarchar(64) NOT NULL,
    imr_policyname    nvarchar(255) NOT NULL,
    imr_enabled    nvarchar(5) NOT NULL,
    imr_category    nvarchar(255) NOT NULL,
    imr_description    nvarchar(1023) NOT NULL,
    imr_runonce    nvarchar(5) NOT NULL,
    imr_priority    integer NOT NULL,
    imr_type    nvarchar(255) NOT NULL,
    imr_policyevents    ntext NOT NULL,
    imr_policydata    ntext NOT NULL,
    imr_entryrules    ntext NOT NULL,
    imr_actionrules    ntext NOT NULL,
    CONSTRAINT PK_01_imrpolicyxpress125 PRIMARY KEY  (imr_reportid, imr_policyid)
    )
go

CREATE INDEX imrpolicyxpress125_index_1 ON imrpolicyxpress125 (imr_reportid, imr_policyid)
go

--
-- IMR imrreversesyncmodifyaccount125
--

CREATE TABLE imrreversesyncmodifyaccount125 (
    imr_reportid integer  NOT NULL,
    imr_policyid     nvarchar(64) NOT NULL,
    imr_policyname    nvarchar(255) NOT NULL,
    imr_enabled    nvarchar(5) NOT NULL,
    imr_description    nvarchar(1023) NOT NULL,
    imr_priority    integer NOT NULL,
    imr_endpointtype    nvarchar(255) NOT NULL,
    imr_endpointname    nvarchar(1023) NOT NULL,
    imr_container    nvarchar(255) NOT NULL,
    imr_attributename    nvarchar(255) NOT NULL,
    imr_attributevalue    nvarchar(1023) NOT NULL,
    imr_action    nvarchar(255) NOT NULL,
    CONSTRAINT PK_01_imrreversesyncmodifyaccount125 PRIMARY KEY  (imr_reportid, imr_policyid)
    )
go

CREATE INDEX imrreversesyncmodifyaccount125_index_1 ON imrreversesyncmodifyaccount125 (imr_reportid, imr_policyid)
go

--
-- IMR imrreversesyncnewaccount125
--

CREATE TABLE imrreversesyncnewaccount125 (
    imr_reportid integer  NOT NULL,
    imr_policyid     nvarchar(64) NOT NULL,
    imr_policyname    nvarchar(255) NOT NULL,
    imr_enabled    nvarchar(5) NOT NULL,
    imr_description    nvarchar(1023) NOT NULL,
    imr_priority    integer NOT NULL,
    imr_endpointtype    nvarchar(255) NOT NULL,
    imr_endpointname    nvarchar(1023) NOT NULL,
    imr_container    nvarchar(255) NOT NULL,
    imr_targetuserstatus    nvarchar(255) NOT NULL,
    imr_action    nvarchar(255) NOT NULL,
    CONSTRAINT PK_01_imrreversesyncnewaccount125 PRIMARY KEY  (imr_reportid, imr_policyid)
    )
go

CREATE INDEX imrreversesyncnewaccount125_index_1 ON imrreversesyncnewaccount125 (imr_reportid, imr_policyid)
go
--
-- IMR imremailmanagement125
--

CREATE TABLE imremailmanagement125 (
    imr_reportid integer  NOT NULL,
    imr_emailid     nvarchar(64) NOT NULL,
    imr_emailname    nvarchar(255) NOT NULL,
    imr_enabled    nvarchar(5) NOT NULL,
    imr_category    nvarchar(255) NOT NULL,
    imr_description    nvarchar(1023) NOT NULL,
    imr_subject    nvarchar(1023) NOT NULL,
    CONSTRAINT PK_01_imremailmanagement125 PRIMARY KEY  (imr_reportid, imr_emailid)
    )
go

CREATE INDEX imremailmanagement125_index_1 ON imremailmanagement125 (imr_reportid, imr_emailid)
go

--
-- IMR imrbulktasks125
--

CREATE TABLE imrbulktasks125 (
    imr_reportid integer  NOT NULL,
    imr_bulktaskid     nvarchar(64) NOT NULL,
    imr_bulktaskname    nvarchar(255) NOT NULL,
    imr_description    nvarchar(1023) NOT NULL,
    imr_tasktag    nvarchar(255) NOT NULL,
    imr_objecttype    nvarchar(255) NOT NULL,
    imr_initiator    nvarchar(255) NOT NULL,
    imr_dateformat    nvarchar(255) NOT NULL,
    imr_populationrule    ntext NULL,
    imr_datefilter    ntext NULL,
    CONSTRAINT PK_01_imrbulktasks125 PRIMARY KEY  (imr_reportid, imr_bulktaskid)
    )
go

CREATE INDEX imrbulktasks125_index_1 ON imrbulktasks125 (imr_reportid, imr_bulktaskid)
go

------------------------------------------------------
-- End of r12.5 SP1 support.
------------------------------------------------------

------------------------------------------------------
-- New r12.5 SP7 support.
------------------------------------------------------
CREATE TABLE imrNonStandardAccts(
	imr_reportid int NOT NULL,
	imr_endpointtype nvarchar(64) NULL,
	imr_objecttype nvarchar(64) NULL,
	imr_endpoint nvarchar(64) NULL,
	imr_accounttype nvarchar(628) NULL,
	imr_totalcount int NULL
)
go

CREATE INDEX imrNonStandardAccts_index_1 ON imrNonStandardAccts (imr_reportid, imr_endpointtype,imr_endpoint)
go

CREATE VIEW exceptionaccountsview
AS
SELECT
	accountObject.imr_reportid as imr_reportid,
	accountObject.imr_objectid as imr_accountid,
	accountObject.imr_friendlyname as imr_accountname,
	accountObject.imr_objectdn as imr_accountdn,
	(SELECT imr_value
	FROM
		imrAttribute endPointAttr,
		imrAttributeInfo endPointAttrInfo
	WHERE
		accountObject.imr_objectid = endPointAttr.imr_objectid and
		endPointAttr.imr_attrinfoid = endPointAttrInfo.imr_attrinfoid and
		endPointAttrInfo.imr_name = 'endPointName' 
	) as imr_endpoint,
	accountObjectInfo.imr_name as imr_endpointtype,
	attr1.imr_value as imr_createtime,
	attr2.imr_value as imr_accountlastmodified,
  attr3.imr_value as exceptionaccount
FROM
	imrObjectCategory accountCategory,
	imrObjectInfo accountObjectInfo,
	imrObject accountObject,
	imrAttributeInfo attrInfo1,
	imrAttribute attr1,
	imrAttributeInfo attrInfo2,
	imrAttribute attr2,
 	imrAttributeInfo attrInfo3,
	imrAttribute attr3

WHERE
	(attr1.imr_value IS NOT NULL or
	attr1.imr_objectrefid IS NOT NULL or
	attr2.imr_value IS NOT NULL or
	attr2.imr_objectrefid IS NOT NULL or 
  attr3.imr_value IS NOT NULL or
	attr3.imr_objectrefid IS NOT NULL) and
	attr1.imr_objectid = accountObject.imr_objectid and
	attr2.imr_objectid = accountObject.imr_objectid and
  attr3.imr_objectid = accountObject.imr_objectid and
	accountObject.imr_objectinfoname = accountObjectInfo.imr_name and
	accountObjectInfo.imr_objectcatname = accountCategory.imr_name and
	accountCategory.imr_name = 'Account' and
	(attr1.imr_attrinfoid = attrInfo1.imr_attrinfoid and 
	attrInfo1.imr_name = 'createDate') and 
	(attr2.imr_attrinfoid = attrInfo2.imr_attrinfoid and 
	attrInfo2.imr_name = 'updateDate') and 
	(attr3.imr_attrinfoid = attrInfo3.imr_attrinfoid and 
	attrInfo3.imr_name = 'exceptionAccount' and attr3.imr_value = 'true') and 
	accountCategory.imr_reportid=accountObject.imr_reportid and
	accountObjectInfo.imr_reportid=accountObject.imr_reportid and
	attrInfo1.imr_reportid=accountObject.imr_reportid and
	attr1.imr_reportid=accountObject.imr_reportid and 
	attrInfo2.imr_reportid=accountObject.imr_reportid and
	attr2.imr_reportid=accountObject.imr_reportid and 
    attrInfo3.imr_reportid=accountObject.imr_reportid and
	attr3.imr_reportid=accountObject.imr_reportid;
GO

CREATE VIEW endpointexceptionuseraccounts6
AS
SELECT 
	a.imr_reportid imr_reportid, 
	b.imr_userid imr_userid, 
	a.imr_accountname imr_accountname, 
	a.imr_accountdn imr_accountdn, 
	a.imr_endpoint as imr_endpoint,
	a.imr_endpointtype imr_accounttype, 
	a.imr_createtime imr_createtime,
	a.imr_accountlastmodified imr_accountlastmodified,
	c.imr_firstname imr_firstname,
	c.imr_lastname imr_lastname,
	c.imr_fullname imr_fullname
FROM  
	exceptionaccountsview a, 
	imrUserAccountRelationship b,
	imruser6 c 
WHERE 
	b.imr_useraccountdn = a.imr_accountdn
	and b.imr_reportid = a.imr_reportid
	and c.imr_reportid = a.imr_reportid
	and c.imr_userid=b.imr_userid;
GO

CREATE VIEW totalExceptionAccounts
AS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'Exception' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME, imr_createtime as CREATION_DATE, imr_accountlastmodified as LAST_MODIFIED 
FROM         endpointexceptionuseraccounts6
WHERE imr_userid NOT IN ( '[default user]','[system user]')
UNION
-- ADD ORPHAN ACCOUNTS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'Orphan' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME, imr_createtime as CREATION_DATE, imr_accountlastmodified as LAST_MODIFIED 
FROM         endpointuseraccounts6
WHERE      imr_userid  IN('[default user]')
UNION
-- ADD SYSTEM ACCOUNTS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'System' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME, imr_createtime as CREATION_DATE, imr_accountlastmodified as LAST_MODIFIED 
FROM         endpointuseraccounts6
WHERE      imr_userid  IN('[system user]')
GO

------------------------------------------------------
-- End of r12.5 SP7 support.
------------------------------------------------------

----------
-- Cleanup Stuff
----------


--
-- PROCEDURE IMRCleanupProc6 For cleaning reports older than a month
-- *********DEPRICATED DO NOT USE
--

CREATE PROCEDURE IMRCleanupProc6 AS
BEGIN


    DELETE FROM imrrole6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrroletask6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrtask6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

    DELETE FROM imrrolerule6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

    DELETE FROM imrrolescoperule6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

    DELETE FROM imrrolemember6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

    DELETE FROM imrroleadmin6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrroleowner6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrorg6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrorgattr6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imruser6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imruserattr6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrgroup6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrgroupattr6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrgroupmember6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrgroupadmin6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
-- add identitypolicy support
     DELETE FROM imridentitypolicy6 where imr_reportid IN (
                 SELECT imreport6.imr_reportid
                 FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imridentitypolicyset6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
     DELETE FROM imridentitypolicyusersync6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

     DELETE FROM imridentitypolicyactions6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
-- end of identitypolicy support
-- 12.5 SP1 support
     DELETE FROM imrpolicyxpress125 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
		)
     DELETE FROM imrreversesyncnewaccount125 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
		)
     DELETE FROM imrreversesyncmodifyaccount125 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
		)
     DELETE FROM imremailmanagement125 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
		)
     DELETE FROM imrbulktasks125 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
		)
-- end of 12.5 SP1 support
    DELETE FROM imreport6 WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )

END
go


CREATE PROCEDURE IMRRemoveSnapshot
	@id int
AS
BEGIN
	DELETE FROM imrrole6 WHERE imr_reportid = @id
	DELETE FROM imrrolemember6 WHERE imr_reportid = @id
	DELETE FROM imrroleadmin6 WHERE imr_reportid = @id
	DELETE FROM imrroleowner6 WHERE imr_reportid = @id
	DELETE FROM imrrolerule6 WHERE imr_reportid = @id
	DELETE FROM imrrolescoperule6 WHERE imr_reportid = @id

	DELETE FROM imrroletask6 WHERE imr_reportid = @id
	DELETE FROM imrtask6 WHERE imr_reportid = @id

	DELETE FROM imrorg6 WHERE imr_reportid = @id
	DELETE FROM imrorgattr6 WHERE imr_reportid = @id

	DELETE FROM imruser6 WHERE imr_reportid = @id
	DELETE FROM imruserattr6 WHERE imr_reportid = @id

	DELETE FROM imrgroup6 WHERE imr_reportid = @id
	DELETE FROM imrgroupattr6 WHERE imr_reportid = @id
	DELETE FROM imrgroupmember6 WHERE imr_reportid = @id
	DELETE FROM imrgroupadmin6 WHERE imr_reportid = @id

	DELETE FROM imridentitypolicy6 WHERE imr_reportid = @id
	DELETE FROM imridentitypolicyset6 WHERE imr_reportid = @id
	DELETE FROM imridentitypolicyusersync6 WHERE imr_reportid = @id
	DELETE FROM imridentitypolicyactions6 WHERE imr_reportid = @id

	DELETE FROM imrAttribute WHERE imr_reportid = @id;
	DELETE FROM imrAttributeInfo WHERE imr_reportid = @id;
	DELETE FROM imrObject WHERE imr_reportid = @id;
	DELETE FROM imrObjectCategory WHERE imr_reportid = @id;
	DELETE FROM imrObjectInfo WHERE imr_reportid = @id;
	DELETE FROM imrProvRolesRelationship WHERE imr_reportid = @id;
	DELETE FROM imrProvRoleTemplRelationship WHERE imr_reportid = @id;
	DELETE FROM imrUserAccountRelationship WHERE imr_reportid = @id;
	DELETE FROM imrTemplEndpointRelationship WHERE imr_reportid = @id;
	DELETE FROM imrAccountTemplateRel WHERE imr_reportid = @id;
	DELETE FROM imrCapabilityAttrRel WHERE imr_reportid = @id;
	DELETE FROM imrroleattr12 WHERE imr_reportid = @id;

	-- 12.5 SP1 support
	DELETE FROM imrpolicyxpress125 WHERE imr_reportid = @id
	DELETE FROM imrreversesyncnewaccount125 WHERE imr_reportid = @id
	DELETE FROM imrreversesyncmodifyaccount125 WHERE imr_reportid = @id
	DELETE FROM imremailmanagement125 WHERE imr_reportid = @id
	DELETE FROM imrbulktasks125 WHERE imr_reportid = @id
	-- End of 12.5 SP1 support

	DELETE FROM imreport6 WHERE imr_reportid = @id
END
GO


CREATE PROCEDURE IMRCleanOldSnapshotsProc
	@keep int,
	@name varchar (255),
	@source varchar (255)
AS
BEGIN
	DECLARE @SQLString NVARCHAR(500)
	DECLARE @SQLDef NVARCHAR(100)
	DECLARE @runCursor CURSOR
	DECLARE @reportId int

	-- make temp table to keep the snapshots ids we want to clean
	create table #retired_snpashots (id int)

	SET @SQLString = 'INSERT INTO #retired_snpashots SELECT imr_reportid FROM imreport6 ' +
					'WHERE imr_reportname=@n AND imr_snapshotsource=@s AND imr_snapshotstatus = ''Completed '' ' +
					'AND imr_reportid NOT IN ' +
					'(SELECT TOP ' + STR(@keep) + ' imr_reportid FROM imreport6 ' +
					'WHERE imr_reportname=@n AND imr_snapshotsource=@s AND imr_snapshotstatus = ''Completed '''+
					'ORDER BY imr_reportid DESC)'
	SET @SQLDef = '@n varchar (255),@s varchar (255)'

	EXECUTE sp_executesql @SQLString,@SQLDef,@n=@name,@s=@source

	SET @runCursor = CURSOR FAST_FORWARD FOR
	SELECT id from #retired_snpashots

	OPEN @runCursor
	FETCH NEXT FROM @runCursor INTO @reportId

	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE IMRRemoveSnapshot @reportId
		FETCH NEXT FROM @runCursor INTO @reportId
	END

	CLOSE @runCursor
	DEALLOCATE @runCursor

	DROP TABLE #retired_snpashots
END
GO

CREATE FUNCTION [IM_Reports] (@getid int, @gettaskname varchar, @getenvoid varchar)

RETURNS int
AS
BEGIN

declare @tempid1 int;
      
if @getid = -1
set @tempid1 = 
(SELECT max(imr_reportid)
FROM 
 imreport6
WHERE imr_reportname IN
  (SELECT imr_snapshotname
  FROM imrptsnapshots12
  WHERE imr_rpt_task_tag_name = @gettaskname
  AND imr_envoid = @getenvoid
  )

)
else
set @tempid1 = @getid

return @tempid1
END
GO