-- Copyright (C) 2005, Computer Associates, Inc. All rights reserved.
--
-- This file contains the schema for upgrading a IM 8 schema to a IM R12 schema
--
-------------------------------------------------------------------

-- Add imr_reportdescription field to imreport6 table

IF EXISTS(SELECT name
      FROM   sysobjects
      WHERE  name = 'imreport6'
      AND    type = 'U')
      ALTER TABLE imreport6 ADD
				imr_reportdescription nvarchar(628) ,
				imr_keepsnapshot int ,
				imr_snapshotsource nvarchar (255) ,
				imr_snapshotstatus nvarchar (64) DEFAULT 'Completed' NOT NULL

GO
IF EXISTS (SELECT name FROM sysobjects
    WHERE name = 'PK_01_IMREPORT6' AND type = 'K')
    Alter table imreport6 drop constraint PK_01_IMREPORT6
GO

ALTER TABLE imreport6
    ADD CONSTRAINT PK_01_IMREPORT6 PRIMARY KEY (imr_reportid)
GO

IF EXISTS (SELECT name FROM sysobjects
    WHERE name = 'insertintoimreport' AND type = 'P')
    DROP PROCEDURE insertintoimreport
GO


CREATE PROCEDURE insertintoimreport
    @imr_reportname nvarchar(255) ,
    @imr_reportdescription nvarchar (628),
    @imr_reportuser  nvarchar(628) ,
    @imr_envoid      nvarchar(64) ,
    @imr_envname     nvarchar(255) ,
    @imr_keepsnapshot int ,
    @imr_snapshotsource nvarchar(255) ,
    @imr_snapshotstatus nvarchar (64) ,
    @current_id INTEGER OUTPUT
AS
BEGIN
 insert into imreport6 (
        imr_reportname ,
        imr_reportdescription,
        imr_reportuser ,
        imr_envoid ,
        imr_envname ,
		imr_keepsnapshot ,
		imr_snapshotsource ,
		imr_snapshotstatus)
            values (
        @imr_reportname ,
        @imr_reportdescription,
        @imr_reportuser ,
        @imr_envoid ,
        @imr_envname ,
		@imr_keepsnapshot ,
		@imr_snapshotsource ,
		@imr_snapshotstatus);

    SELECT @current_id=IDENT_CURRENT('imreport6');
END
GO


IF EXISTS (SELECT name FROM sysobjects
    WHERE name = 'insertintousertable' AND type = 'P')
    DROP PROCEDURE insertintousertable
GO

CREATE PROCEDURE insertintousertable
    @imr_reportid integer ,
    @imr_userdn    nvarchar(628),
    @imr_isenabled INTEGER ,
    @imr_userid    nvarchar(255) ,
    @imr_fullname  nvarchar(255) ,
    @imr_lastname  nvarchar(255) ,
    @imr_firstname nvarchar(255) ,
    @imr_parentorgdn nvarchar(628) ,
    @imr_email       nvarchar(255)
AS
BEGIN
    IF NOT EXISTS (Select imr_userdn from imruser6
        WHERE
            imr_reportid= @imr_reportid
            AND imr_userdn = @imr_userdn)
     insert into imruser6 (
          imr_reportid ,
          imr_userdn   ,
          imr_isenabled,
          imr_userid   ,
          imr_fullname ,
          imr_lastname ,
          imr_firstname,
          imr_parentorgdn ,
          imr_email )
                values (
           @imr_reportid ,
           @imr_userdn   ,
           @imr_isenabled,
           @imr_userid   ,
           @imr_fullname ,
           @imr_lastname ,
           @imr_firstname,
           @imr_parentorgdn ,
           @imr_email);
END
GO

-- New Tables
-- Add userAccounts tables

IF EXISTS(SELECT name
      FROM   sysobjects
      WHERE  name = 'imruseraccounts6'
      AND    type = 'U')
    DROP TABLE imruseraccounts6
go

CREATE TABLE imruseraccounts6 (
    imr_reportid 			integer  NOT NULL,
    imr_userdn      		nvarchar(628) NOT NULL,
    imr_username			nvarchar(64) NULL,
    imr_accountdn          	nvarchar(628) NOT NULL,
    imr_accountname      	nvarchar(64) NULL,
    imr_endpoint 			nvarchar(64) NULL,
    imr_accountcontainer   	nvarchar(64) NULL,
    imr_accounttype   		nvarchar(64) NULL,
    imr_accountdomain  		nvarchar(64) NULL,
    imr_accountreason		nvarchar(256) NULL,
    imr_accountcreator		nvarchar(256) NULL,
    imr_accountcreationdate	datetime NULL,
    imr_accountlastmodified datetime NULL,
    imr_accountapprover		nvarchar(256) NULL,
    imr_accountpolicy		nvarchar(256) NULL,
    imr_accountsyncwithroles integer NULL,
    imr_isenabled integer NULL
)
go

CREATE INDEX imruseraccounts6_index_1 ON imruseraccounts6 (imr_reportid)
	INCLUDE (imr_userdn)
go

CREATE INDEX imruseraccounts6_index_2 ON imruseraccounts6 (imr_reportid)
	INCLUDE (imr_userdn, imr_accountdn)
go

CREATE INDEX imruseraccounts6_index_3 ON imruseraccounts6 (imr_reportid, imr_endpoint)
go

CREATE INDEX imruseraccounts6_index_4 ON imruseraccounts6 (imr_reportid, imr_accounttype)
go

CREATE INDEX imruseraccounts6_index_5 ON imruseraccounts6 (imr_reportid, imr_endpoint, imr_accounttype)
go


-- SP to do the insert into the imruseraccounts6 table
IF EXISTS (SELECT name FROM sysobjects
    WHERE name = 'insertintoimruseraccounts' AND type = 'P')
    DROP PROCEDURE insertintoimruseraccounts
GO

CREATE PROCEDURE insertintoimruseraccounts
 	@imr_reportid 			integer,
    @imr_userdn      		nvarchar(628),
    @imr_username			nvarchar(64),
    @imr_accountdn          nvarchar(628),
    @imr_accountname      	nvarchar(64),
    @imr_endpoint 			nvarchar(64),
    @imr_accountcontainer   nvarchar(64),
    @imr_accounttype   		nvarchar(64),
    @imr_accountdomain  	nvarchar(64),
    @imr_accountreason		nvarchar(256),
    @imr_accountcreator		nvarchar(256),
    @imr_accountcreationdate	datetime,
    @imr_accountlastmodified 	datetime,
	@imr_accountapprover	nvarchar(256),
	@imr_accountpolicy		nvarchar(256),
	@imr_accountsyncwithroles  integer
AS
BEGIN
    IF NOT EXISTS (Select imr_userdn, imr_accountdn from imruseraccounts6
        WHERE
            imr_reportid= @imr_reportid
            AND imr_userdn = @imr_userdn
            AND imr_accountdn = @imr_accountdn)
     insert into imruseraccounts6 (
		    imr_reportid,
		    imr_userdn,
		    imr_username,
		    imr_accountdn,
		    imr_accountname,
		    imr_endpoint,
		    imr_accountcontainer,
		    imr_accounttype,
		    imr_accountdomain,
		    imr_accountreason,
		    imr_accountcreator,
		    imr_accountcreationdate,
		    imr_accountlastmodified,
			imr_accountapprover,
			imr_accountpolicy,
			imr_accountsyncwithroles
            )
            values (
        	@imr_reportid,
		    @imr_userdn,
		    @imr_username,
		    @imr_accountdn,
		    @imr_accountname,
		    @imr_endpoint,
		    @imr_accountcontainer,
		    @imr_accounttype,
		    @imr_accountdomain,
		    @imr_accountreason,
		    @imr_accountcreator,
		    @imr_accountcreationdate,
		    @imr_accountlastmodified,
			@imr_accountapprover,
			@imr_accountpolicy,
			@imr_accountsyncwithroles);
END
GO

--
-- IM imraccountattrs6
--

IF EXISTS(SELECT name
      FROM   sysobjects
      WHERE  name = 'imraccountattrs6'
      AND    type = 'U')
    DROP TABLE imraccountattrs6
go

CREATE TABLE imraccountattrs6 (
    imr_reportid 			integer  NOT NULL,
    imr_accountdn          	nvarchar(628) NOT NULL,
    imr_accountattr		   	nvarchar(256) NULL,
    imr_accountattrvalue   	nvarchar(628) NULL
    )
go

CREATE INDEX imraccountattrs6_index_1 ON imraccountattrs6 (imr_reportid)
	INCLUDE (imr_accountdn)
go

--
-- IM imrptsnapshots12
--

IF EXISTS(SELECT name
      FROM   sysobjects
      WHERE  name = 'imrptsnapshots12'
      AND    type = 'U')
    DROP TABLE imrptsnapshots12
go

CREATE TABLE imrptsnapshots12 (
    imr_rptname 		nvarchar(255) NOT NULL,
    imr_snapshotname          	nvarchar(255) NOT NULL,
    imr_envname			nvarchar(255) NOT NULL,
    IMR_RPT_TASK_TAG_NAME nvarchar(255) NOT NULL
)
go

CREATE INDEX imrptsnapshots12_index_1 ON imrptsnapshots12 (imr_rptname)
	INCLUDE (imr_snapshotname, imr_envname)
go

--
-- IMR TASK 6
--
IF EXISTS(SELECT name
      FROM   sysobjects
      WHERE  name = 'imrtask6'
      AND    type = 'U')
      ALTER TABLE imrtask6 ADD
	imr_reserved6 nvarchar(255) NULL,
      	imr_reserved7 nvarchar(255) NULL,
      	imr_reserved8 nvarchar(255) NULL,
      	imr_reserved9 nvarchar(255) NULL,
      	imr_reserved10 nvarchar(255) NULL
GO


-- modify existing cleanup procedure to include userAccounts support

-- *********DEPRICATED DO NOT USE

IF EXISTS (SELECT name FROM sysobjects
    WHERE name = 'IMRCleanupProc6' AND type = 'P')
    DROP PROCEDURE IMRCleanupProc6
GO

CREATE PROCEDURE IMRCleanupProc6 AS
BEGIN


    DELETE FROM imrrole6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrroletask6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrtask6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

    DELETE FROM imrrolerule6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

    DELETE FROM imrrolescoperule6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

    DELETE FROM imrrolemember6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

    DELETE FROM imrroleadmin6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrroleowner6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrorg6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrorgattr6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imruser6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imruserattr6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrgroup6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrgroupattr6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrgroupmember6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imrgroupadmin6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

     DELETE FROM imridentitypolicy6 where imr_reportid IN (
                 SELECT imreport6.imr_reportid
                 FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imridentitypolicyset6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
     DELETE FROM imridentitypolicyusersync6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

     DELETE FROM imridentitypolicyactions6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )

     -- add userAccounts support
     DELETE FROM imruseraccounts6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
               WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
      )
      DELETE FROM imraccountattrs6 where imr_reportid IN (
                SELECT imreport6.imr_reportid
                FROM imreport6
                WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )
        )
    DELETE FROM imreport6 WHERE ( imreport6.imr_reporttime < DATEADD(Month,-1,GETDATE()) )

END
go

IF EXISTS (SELECT name FROM sysobjects
    WHERE name = 'IMRRemoveSnapshot' AND type = 'P')
    DROP PROCEDURE IMRRemoveSnapshot
GO

CREATE PROCEDURE IMRRemoveSnapshot
	@id int
AS
BEGIN
	DELETE FROM imrrole6 WHERE imr_reportid = @id
	DELETE FROM imrrolemember6 WHERE imr_reportid = @id
	DELETE FROM imrroleadmin6 WHERE imr_reportid = @id
	DELETE FROM imrroleowner6 WHERE imr_reportid = @id
	DELETE FROM imrrolerule6 WHERE imr_reportid = @id
	DELETE FROM imrrolescoperule6 WHERE imr_reportid = @id

	DELETE FROM imrroletask6 WHERE imr_reportid = @id
	DELETE FROM imrtask6 WHERE imr_reportid = @id

	DELETE FROM imrorg6 WHERE imr_reportid = @id
	DELETE FROM imrorgattr6 WHERE imr_reportid = @id

	DELETE FROM imruser6 WHERE imr_reportid = @id
	DELETE FROM imruserattr6 WHERE imr_reportid = @id

	DELETE FROM imrgroup6 WHERE imr_reportid = @id
	DELETE FROM imrgroupattr6 WHERE imr_reportid = @id
	DELETE FROM imrgroupmember6 WHERE imr_reportid = @id
	DELETE FROM imrgroupadmin6 WHERE imr_reportid = @id

	DELETE FROM imridentitypolicy6 WHERE imr_reportid = @id
	DELETE FROM imridentitypolicyset6 WHERE imr_reportid = @id
	DELETE FROM imridentitypolicyusersync6 WHERE imr_reportid = @id
	DELETE FROM imridentitypolicyactions6 WHERE imr_reportid = @id

	DELETE FROM imrAttribute WHERE imr_reportid = @id;
	DELETE FROM imrAttributeInfo WHERE imr_reportid = @id;
	DELETE FROM imrObject WHERE imr_reportid = @id;
	DELETE FROM imrObjectCategory WHERE imr_reportid = @id;
	DELETE FROM imrObjectInfo WHERE imr_reportid = @id;
	DELETE FROM imrProvRolesRelationship WHERE imr_reportid = @id;
	DELETE FROM imrProvRoleTemplRelationship WHERE imr_reportid = @id;
	DELETE FROM imrUserAccountRelationship WHERE imr_reportid = @id;
	DELETE FROM imrTemplEndpointRelationship WHERE imr_reportid = @id;
	DELETE FROM imrAccountTemplateRel WHERE imr_reportid = @id;
	DELETE FROM imrCapabilityAttrRel WHERE imr_reportid = @id;
	DELETE FROM imrroleattr12 WHERE imr_reportid = @id;

	DELETE FROM imreport6 WHERE imr_reportid = @id
END
GO


IF EXISTS (SELECT name FROM sysobjects
	WHERE name='IMRCleanOldSnapshotsProc' AND type = 'P')
	DROP PROCEDURE IMRCleanOldSnapshotsProc
GO

CREATE PROCEDURE IMRCleanOldSnapshotsProc
	@keep int,
	@name varchar (255),
	@source varchar (255)
AS
BEGIN
	DECLARE @SQLString NVARCHAR(500)
	DECLARE @SQLDef NVARCHAR(100)
	DECLARE @runCursor CURSOR
	DECLARE @reportId int

	-- make temp table to keep the snapshots ids we want to clean
	create table #retired_snpashots (id int)

	SET @SQLString = 'INSERT INTO #retired_snpashots SELECT imr_reportid FROM imreport6 ' +
					'WHERE imr_reportname=@n AND imr_snapshotsource=@s AND imr_snapshotstatus = ''Completed '' ' +
					'AND imr_reportid NOT IN ' +
					'(SELECT TOP ' + STR(@keep) + ' imr_reportid FROM imreport6 ' +
					'WHERE imr_reportname=@n AND imr_snapshotsource=@s AND imr_snapshotstatus = ''Completed '''+
					'ORDER BY imr_reportid DESC)'
	SET @SQLDef = '@n varchar (255),@s varchar (255)'

	EXECUTE sp_executesql @SQLString,@SQLDef,@n=@name,@s=@source

	SET @runCursor = CURSOR FAST_FORWARD FOR
	SELECT id from #retired_snpashots

	OPEN @runCursor
	FETCH NEXT FROM @runCursor INTO @reportId

	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE IMRRemoveSnapshot @reportId
		FETCH NEXT FROM @runCursor INTO @reportId
	END

	CLOSE @runCursor
	DEALLOCATE @runCursor

	DROP TABLE #retired_snpashots
END
GO

CREATE   VIEW totalExceptionAccounts
AS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'Exception' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME,imr_accountcreationdate as CREATION_DATE, imr_isenabled as IS_ENABLED,
			imr_accountlastmodified as LAST_MODIFIED
FROM         imruseraccounts6
WHERE imr_accountsyncwithroles = 1 AND imr_username NOT IN ( '[default user]','[system user]')
UNION
-- ADD ORPHAN ACCOUNTS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'Orphan' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME,imr_accountcreationdate as CREATION_DATE, imr_isenabled as IS_ENABLED,imr_accountlastmodified as LAST_MODIFIED
FROM         imruseraccounts6
WHERE      imr_username  IN('[default user]')
UNION
-- ADD SYSTEM ACCOUNTS
SELECT     imr_reportid AS REPORT_ID, imr_accounttype AS END_POINT_TYPE, imr_endpoint AS END_POINT, 'System' AS EXCEPTION_TYPE,
           imr_accountname AS ACCOUNT_NAME,imr_accountcreationdate as CREATION_DATE, imr_isenabled as IS_ENABLED,imr_accountlastmodified as LAST_MODIFIED
FROM         imruseraccounts6
WHERE      imr_username  IN('[system user]')
GO


CREATE PROCEDURE insertintoaccountAttributes
    @imr_reportid integer ,
    @imr_accountdn nvarchar(628),
    @imr_accountattr  nvarchar(256) ,
    @imr_accountattrvalue nvarchar(628)
AS
BEGIN
 insert into imraccountattrs6 (
        imr_reportid ,
		imr_accountdn,
		imr_accountattr,
	imr_accountattrvalue  )
            values (
        @imr_reportid,
    @imr_accountdn ,
    @imr_accountattr ,
    @imr_accountattrvalue);
END
GO

