-- COPYRIGHT (C) 2009, CA, INC. ALL RIGHTS RESERVED.
--
-- THIS FILE CONTAINS THE SCHEMA FOR 
-- IAM FRAMEWORK TABLES FOR AUDITING
--
--

--
-- Install Procedures
--

-- The section below is for reference purpoces. Procedures are instaleld/removed through schema initialization code
-- The code below installs IAMProcedures (stored procedures installed with IAM)
-- Correct location of EAR is required for jar
-- CALL sqlj.install_jar(
-- 	'<ear_location>\library\iamderbysp.jar'
-- 	, 'IAMProcedures', 0);
-- CALL SYSCS_UTIL.SYSCS_SET_DATABASE_PROPERTY('derby.database.classpath', CURRENT SCHEMA || '.IAMProcedures');


-- Add the Version table
CREATE TABLE IMSAUDIT_VERSION (
    IMR_KEY VARCHAR(255) NOT NULL,
    IMR_VALUE VARCHAR(255) NOT NULL
    );

INSERT INTO IMSAUDIT_VERSION(IMR_KEY,IMR_VALUE) VALUES ('version','13.0.0.0.0');

CREATE TABLE IMSAUDITTASKSESSION12 (
	ID INTEGER GENERATED BY DEFAULT AS IDENTITY (START WITH 1, INCREMENT BY 1) NOT NULL ,
	PARENT_TS_OID VARCHAR (100),
	PARENT_EVENT_OID VARCHAR (100),
	AUDIT_TIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
	TASKSESSION_OID VARCHAR (100) NOT NULL ,
	ADMIN_DN VARCHAR (512) NOT NULL ,
	ADMIN_NAME VARCHAR (255),
	TASK_NAME VARCHAR (255),
	TASK_TAG VARCHAR (255) NOT NULL ,
	TASK_DESCRIPTION VARCHAR (4000),
	TASK_PRIORITY INTEGER,
	STATE VARCHAR (100) NOT NULL ,
	ENVNAME VARCHAR (100) NOT NULL ,
	ENV_OID VARCHAR (100) NOT NULL ,
	CONSTRAINT PK_01_TS12 PRIMARY KEY  (ID),
	CONSTRAINT AUDIT_UNIQUE_TS12 UNIQUE (TASKSESSION_OID, STATE)
);


CREATE INDEX IMSAUDITTASKSESSION12_INDEX_1 ON IMSAUDITTASKSESSION12 (AUDIT_TIME);

 
CREATE INDEX IMSAUDITTASKSESSION12_INDEX_2 ON IMSAUDITTASKSESSION12 (ADMIN_NAME);


CREATE TABLE IMSAUDITEVENT12 (
	ID INTEGER GENERATED ALWAYS AS IDENTITY(START WITH 1, INCREMENT BY 1) NOT NULL ,
	TASKSESSION_ID INTEGER NOT NULL ,
	TASKSESSION_OID VARCHAR (100),
	PARENT_EVENT_OID VARCHAR (100),
	AUDIT_TIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
	EVENT_OID VARCHAR (100) NOT NULL ,
	ADMIN_DN VARCHAR (512) NOT NULL ,
	ADMIN_NAME VARCHAR (255),
	EVENT_NAME VARCHAR (255) NOT NULL ,
	EVENT_DESCRIPTION VARCHAR (4000),
	EVENT_STATE VARCHAR (100),
	ENVNAME VARCHAR (100) NOT NULL ,
	ENV_OID VARCHAR (100) NOT NULL ,
	CONSTRAINT PK_01_EVT12 PRIMARY KEY  (ID)
) ;


CREATE INDEX IMSAUDITEVENT12_INDEX_1 ON IMSAUDITEVENT12 (AUDIT_TIME);

 
CREATE INDEX IMSAUDITEVENT12_INDEX_2 ON IMSAUDITEVENT12 (ADMIN_NAME);


CREATE INDEX IMSAUDITEVENT12_INDEX_3 ON IMSAUDITEVENT12 (EVENT_NAME,EVENT_STATE);


CREATE TABLE IMSAUDITEVENTOBJECT12 (
	ID INTEGER GENERATED ALWAYS AS IDENTITY(START WITH 1, INCREMENT BY 1) NOT NULL ,
	PARENT_EVENT_ID INTEGER NOT NULL ,
	AUDIT_TIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
	OBJECT_TYPE VARCHAR (100) NOT NULL ,
 	OBJECT_NAME VARCHAR (255),
 	CONSTRAINT PK_01_EVTOBJ12 PRIMARY KEY  (ID)
); 


CREATE INDEX IMSAUDITEVENTOBJECT12_INDEX_1 ON IMSAUDITEVENTOBJECT12 (PARENT_EVENT_ID);


CREATE TABLE IMSAUDITOBJECTATTRIBUTES12 (
	ID INTEGER GENERATED ALWAYS AS IDENTITY(START WITH 1, INCREMENT BY 1) NOT NULL ,
	PARENT_OBJECT_ID INTEGER NOT NULL ,
	AUDIT_TIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
	DISPLAY_NAME VARCHAR (255),
	ATTRIBUTE_NAME VARCHAR (255) NOT NULL ,
	ATTRIBUTE_OLDVALUE VARCHAR (4000),
	ATTRIBUTE_NEWVALUE VARCHAR (4000),
	CONSTRAINT PK_01_OBJATTR12 PRIMARY KEY  (ID)
) ;


CREATE INDEX IMSAUDITOBJECTATTRIB12_INDEX_1 ON IMSAUDITOBJECTATTRIBUTES12 (PARENT_OBJECT_ID);


CREATE TABLE IMSAUDITOBJECTRELATIONSHIP12 (
	ID  INTEGER GENERATED ALWAYS AS IDENTITY(START WITH 1, INCREMENT BY 1) NOT NULL ,
	PARENT_EVENT_ID INTEGER NOT NULL ,
	AUDIT_TIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
	OBJECT_TYPE VARCHAR (100) NOT NULL ,
	OBJECT_DN VARCHAR (512) NOT NULL ,
	CONTAINER_TYPE VARCHAR (100) NOT NULL ,
	OBJECT_NAME VARCHAR  (255)  NOT NULL ,
	CONTAINER_NAME VARCHAR (255)  NOT NULL ,
	CONTAINER_DN VARCHAR (512) NOT NULL ,
	OPERATION VARCHAR  (50)  NOT NULL, 
	CONSTRAINT PK_01_OBJREL12 PRIMARY KEY  (ID)

);


CREATE INDEX IMSAUDITOBJECTREL12_INDEX_1 ON IMSAUDITOBJECTRELATIONSHIP12 (PARENT_EVENT_ID);




-- 
-- Insert procedure for imsAuditTaskSession12
-- Returns the inserted eventid
CREATE PROCEDURE insertImsAuditTasksession125(
	              IN parent_ts_oid VARCHAR(255) ,
		      IN parent_event_oid VARCHAR(255) ,
		      IN tasksession_oid VARCHAR(255) ,      
		      IN ims_admin_dn VARCHAR(628) ,
		      IN ims_adminname VARCHAR(255) ,
		      IN ims_taskname  VARCHAR(255) ,
		      IN ims_task_tag   VARCHAR(255) ,
		      IN ims_task_description    VARCHAR(628) ,
		      IN ims_task_priority   INT,
		      IN ims_state   VARCHAR(255) ,
		      IN ims_envname  VARCHAR(255) ,
		      IN ims_env_oid  VARCHAR(255) ,
		      IN audit_time TIMESTAMP	,
		      OUT current_eventid INTEGER )
                  PARAMETER STYLE JAVA
                  MODIFIES SQL DATA
                  LANGUAGE JAVA
                  EXTERNAL NAME 'com.ca.ims.db.derby.AuditDBProcedures.insertintoimsAuditTaskSession125';
          
-- 
-- Insert procedure for ImsAuditEvent12
-- Returns the inserted eventid
CREATE PROCEDURE insertImsAuditEvent125(	
	IN parent_ts_oid VARCHAR(100)	,
	IN parent_event_oid VARCHAR(100)	,
	IN event_oid VARCHAR(100)	,
	IN admin_dn VARCHAR(512)	,
	IN admin_name	VARCHAR(255),
	IN event_name	VARCHAR(255),
	IN event_description VARCHAR(4000),
	IN event_state VARCHAR(100)	,
	IN envname VARCHAR(100)	,
	IN env_oid VARCHAR(100)	 ,
	IN tasksession_id INT,
	IN audit_time TIMESTAMP	,
	OUT current_eventid INTEGER)
                  PARAMETER STYLE JAVA
                  MODIFIES SQL DATA
                  LANGUAGE JAVA
                  EXTERNAL NAME 'com.ca.ims.db.derby.AuditDBProcedures.insertImsAuditEvent125';

-- 
-- Insert procedure for ImsAuditeventObject12
-- Returns the inserted id
CREATE PROCEDURE insertImsAuditeventObject125(
	IN parent_event_id INT,
	IN object_type VARCHAR(100),
	IN object_name VARCHAR(255),
	IN audit_time TIMESTAMP	,
	OUT current_id INTEGER)
                  PARAMETER STYLE JAVA
                  MODIFIES SQL DATA
                  LANGUAGE JAVA
                  EXTERNAL NAME 'com.ca.ims.db.derby.AuditDBProcedures.insertImsAuditeventObject125';
