-- COPYRIGHT (C) 2009, CA, INC. ALL RIGHTS RESERVED.
--
-- THIS FILE CONTAINS THE SCHEMA FOR
-- IAM FRAMEWORK TABLES FOR AUDITING
--
--

-- INNODB is required for FOREIGN KEY CONSTRAINTS and AUTO_INCREMENT.
SET storage_engine=INNODB;

-- Add the Version table
CREATE TABLE IMSAUDIT_VERSION (
    IMR_KEY     VARCHAR(255) NOT NULL,
    IMR_VALUE   VARCHAR(255) NOT NULL
);

INSERT INTO IMSAUDIT_VERSION(IMR_KEY, IMR_VALUE) VALUES ('version','13.0.0.0.0');


CREATE TABLE IMSAUDITTASKSESSION12 (
    ID                  INT NOT NULL,
    PARENT_TS_OID       VARCHAR (100) NULL,
    PARENT_EVENT_OID    VARCHAR (100) NULL,
    AUDIT_TIME          TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    TASKSESSION_OID     VARCHAR (100) NOT NULL,
    ADMIN_DN            VARCHAR (512) NOT NULL,
    ADMIN_NAME          VARCHAR (255) NULL,
    TASK_NAME           VARCHAR (255) NULL,
    TASK_TAG            VARCHAR (255) NOT NULL,
    TASK_DESCRIPTION    VARCHAR(4000) NULL,
    TASK_PRIORITY       INT NULL,
    STATE               VARCHAR(100) NOT NULL,
    ENVNAME             VARCHAR(100) NOT NULL,
    ENV_OID             VARCHAR(100) NOT NULL
);

ALTER TABLE IMSAUDITTASKSESSION12 ADD
    CONSTRAINT PK_01_TS12 PRIMARY KEY (
        ID
);

ALTER TABLE IMSAUDITTASKSESSION12 MODIFY
    ID INT NOT NULL AUTO_INCREMENT;

ALTER TABLE IMSAUDITTASKSESSION12 ADD
    CONSTRAINT AUDIT_UNIQUE_TS12 UNIQUE (
        TASKSESSION_OID,
        STATE
);

CREATE INDEX IMSAUDITTASKSESSION12_INDEX_1 ON IMSAUDITTASKSESSION12 (AUDIT_TIME);

CREATE INDEX IMSAUDITTASKSESSION12_INDEX_2 ON IMSAUDITTASKSESSION12 (ADMIN_NAME);


CREATE TABLE IMSAUDITEVENT12 (
    ID                  INT NOT NULL,
    TASKSESSION_ID      INT NOT NULL,
    TASKSESSION_OID     VARCHAR(100) NULL,
    PARENT_EVENT_OID    VARCHAR(100) NULL,
    AUDIT_TIME          TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    EVENT_OID           VARCHAR(100) NOT NULL,
    ADMIN_DN            VARCHAR(512) NOT NULL,
    ADMIN_NAME          VARCHAR(255) NULL,
    EVENT_NAME          VARCHAR(255) NOT NULL,
    EVENT_DESCRIPTION   VARCHAR(4000) NULL,
    EVENT_STATE         VARCHAR(100) NULL,
    ENVNAME             VARCHAR(100) NOT NULL,
    ENV_OID             VARCHAR(100) NOT NULL
) ;

ALTER TABLE IMSAUDITEVENT12 ADD
    CONSTRAINT PK_01_EVT12 PRIMARY KEY (
        ID
);

ALTER TABLE IMSAUDITEVENT12 MODIFY
    ID INT NOT NULL AUTO_INCREMENT;

CREATE INDEX IMSAUDITEVENT12_INDEX_1 ON IMSAUDITEVENT12 (AUDIT_TIME);

CREATE INDEX IMSAUDITEVENT12_INDEX_2 ON IMSAUDITEVENT12 (ADMIN_NAME);

CREATE INDEX IMSAUDITEVENT12_INDEX_3 ON IMSAUDITEVENT12 (EVENT_NAME, EVENT_STATE);


CREATE TABLE IMSAUDITEVENTOBJECT12 (
    ID              INT NOT NULL,
    PARENT_EVENT_ID INT NOT NULL,
    AUDIT_TIME      TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    OBJECT_TYPE     VARCHAR(100) NOT NULL,
    OBJECT_NAME     VARCHAR(255) NULL
);

ALTER TABLE IMSAUDITEVENTOBJECT12 ADD
    CONSTRAINT PK_01_EVTOBJ12 PRIMARY KEY (
        ID
);

ALTER TABLE IMSAUDITEVENTOBJECT12 MODIFY
    ID INT NOT NULL AUTO_INCREMENT;

CREATE INDEX IMSAUDITEVENTOBJECT12_INDEX_1 ON IMSAUDITEVENTOBJECT12 (PARENT_EVENT_ID);


CREATE TABLE IMSAUDITOBJECTATTRIBUTES12 (
    ID                  INT NOT NULL,
    PARENT_OBJECT_ID    INT NOT NULL,
    AUDIT_TIME          TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    DISPLAY_NAME        VARCHAR(255) NULL,
    ATTRIBUTE_NAME      VARCHAR(255) NOT NULL,
    ATTRIBUTE_OLDVALUE  VARCHAR(4000) NULL,
    ATTRIBUTE_NEWVALUE  VARCHAR(4000) NULL
);

ALTER TABLE IMSAUDITOBJECTATTRIBUTES12 ADD
    CONSTRAINT PK_01_OBJATTR12 PRIMARY KEY (
        ID
);

ALTER TABLE IMSAUDITOBJECTATTRIBUTES12 MODIFY
    ID INT NOT NULL AUTO_INCREMENT;

CREATE INDEX IMSAUDITOBJECTATTRIB12_INDEX_1 ON IMSAUDITOBJECTATTRIBUTES12 (PARENT_OBJECT_ID);


CREATE TABLE IMSAUDITOBJECTRELATIONSHIP12 (
    ID              INT NOT NULL,
    PARENT_EVENT_ID INT NOT NULL,
    AUDIT_TIME      TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    OBJECT_TYPE     VARCHAR(100) NOT NULL,
    OBJECT_DN       VARCHAR(512) NOT NULL,
    CONTAINER_TYPE  VARCHAR(100) NOT NULL,
    OBJECT_NAME     VARCHAR(255)  NOT NULL,
    CONTAINER_NAME  VARCHAR(255)  NOT NULL,
    CONTAINER_DN    VARCHAR(512) NOT NULL,
    OPERATION       VARCHAR(50)  NOT NULL
);

ALTER TABLE IMSAUDITOBJECTRELATIONSHIP12 ADD
    CONSTRAINT PK_01_OBJREL12 PRIMARY KEY (
        ID
);

ALTER TABLE IMSAUDITOBJECTRELATIONSHIP12 MODIFY
    ID INT NOT NULL AUTO_INCREMENT;

CREATE INDEX IMSAUDITOBJECTREL12_INDEX_1 ON IMSAUDITOBJECTRELATIONSHIP12 (PARENT_EVENT_ID);


-- ***************************************
-- INSERT PROCEDURES

DROP PROCEDURE IF EXISTS INSERTIMSAUDITTASKSESSION125;

DELIMITER $$

CREATE PROCEDURE INSERTIMSAUDITTASKSESSION125(
    IN L_PARENT_TS_OID      VARCHAR(100),
    IN L_PARENT_EVENT_OID   VARCHAR(100),
    IN L_TASKSESSION_OID    VARCHAR(100),
    IN L_ADMIN_DN           VARCHAR(512),
    IN L_ADMIN_NAME         VARCHAR(255),
    IN L_TASK_NAME          VARCHAR(255),
    IN L_TASK_TAG           VARCHAR(255),
    IN L_TASK_DESCRIPTION   VARCHAR(4000),
    IN L_TASK_PRIORITY      INT,
    IN L_STATE              VARCHAR(100),
    IN L_ENVNAME            VARCHAR(100),
    IN L_ENV_OID            VARCHAR(100),
    IN L_AUDIT_TIME         TIMESTAMP,
    OUT CURRENT_ID          INT)
BEGIN
    DECLARE CURRENT_ID INT DEFAULT 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET CURRENT_ID = 0;
    SELECT id INTO CURRENT_ID
    FROM IMSAUDITTASKSESSION12 a
    WHERE (
        a.TASKSESSION_OID = L_TASKSESSION_OID AND
        a.STATE  = L_STATE AND
        a.ENV_OID  = L_ENV_OID );

    IF CURRENT_ID = 0 THEN
        INSERT INTO IMSAUDITTASKSESSION12 (
            ID, PARENT_TS_OID, PARENT_EVENT_OID, TASKSESSION_OID, ADMIN_DN, ADMIN_NAME, TASK_NAME,
            TASK_TAG, TASK_DESCRIPTION, TASK_PRIORITY, STATE, ENVNAME, ENV_OID, AUDIT_TIME)
        VALUES (
            CURRENT_ID, L_PARENT_TS_OID, L_PARENT_EVENT_OID, L_TASKSESSION_OID, L_ADMIN_DN, L_ADMIN_NAME, L_TASK_NAME,
            L_TASK_TAG, L_TASK_DESCRIPTION, L_TASK_PRIORITY, L_STATE, L_ENVNAME, L_ENV_OID, L_AUDIT_TIME);
        SELECT LAST_INSERT_ID() INTO CURRENT_ID;
    END IF;
END$$


DROP PROCEDURE IF EXISTS INSERTIMSAUDITEVENTOBJECT125$$

CREATE PROCEDURE INSERTIMSAUDITEVENTOBJECT125(
    IN PARENT_EVENT_ID  INT,
    IN OBJECT_TYPE      VARCHAR(100),
    IN OBJECT_NAME      VARCHAR(255),
    IN AUDIT_TIME       TIMESTAMP,
    OUT CURRENT_ID      INT)
BEGIN
    INSERT INTO IMSAUDITEVENTOBJECT12 (ID, PARENT_EVENT_ID, OBJECT_TYPE, OBJECT_NAME, AUDIT_TIME)
    VALUES (CURRENT_ID, PARENT_EVENT_ID, OBJECT_TYPE, OBJECT_NAME, AUDIT_TIME);
    SELECT LAST_INSERT_ID() INTO CURRENT_ID;
END$$


DROP PROCEDURE IF EXISTS INSERTIMSAUDITEVENT125$$

CREATE PROCEDURE INSERTIMSAUDITEVENT125(
    IN PARENT_TS_OID        VARCHAR(100),
    IN PARENT_EVENT_OID     VARCHAR(100),
    IN EVENT_OID            VARCHAR(100),
    IN ADMIN_DN             VARCHAR(512),
    IN ADMIN_NAME           VARCHAR(255),
    IN EVENT_NAME           VARCHAR(255),
    IN EVENT_DESCRIPTION    VARCHAR(4000),
    IN EVENT_STATE          VARCHAR(100),
    IN ENVNAME              VARCHAR(100),
    IN ENV_OID              VARCHAR(100),
    IN TASKSESSION_ID       INT,
    IN AUDIT_TIME           TIMESTAMP,
    OUT CURRENT_EVENTID     INT)
BEGIN
    INSERT INTO IMSAUDITEVENT12 (
        ID, TASKSESSION_ID, TASKSESSION_OID, PARENT_EVENT_OID, EVENT_OID, ADMIN_DN, ADMIN_NAME,
        EVENT_NAME, EVENT_DESCRIPTION, EVENT_STATE, ENVNAME, ENV_OID, AUDIT_TIME)
    VALUES (
        CURRENT_EVENTID, TASKSESSION_ID, PARENT_TS_OID, PARENT_EVENT_OID, EVENT_OID, ADMIN_DN, ADMIN_NAME,
        EVENT_NAME, EVENT_DESCRIPTION, EVENT_STATE, ENVNAME, ENV_OID, AUDIT_TIME);
    SELECT LAST_INSERT_ID() INTO CURRENT_EVENTID;
END$$


--**********************
-- GARBAGE COLLECTION

DROP PROCEDURE IF EXISTS GARBAGECOLLECTAUDITING12$$

CREATE PROCEDURE GARBAGECOLLECTAUDITING12(
    IME_NAME    VARCHAR(100),
    CUTOFF_TIME TIMESTAMP)
BEGIN
    SELECT 'Sourcing stale events from imsAuditEvent12 . . .';
    CREATE TEMPORARY TABLE idmstaleeventrecords6 (id INT NOT NULL, tasksession_oid VARCHAR(100) NOT NULL);
    INSERT INTO idmstaleeventrecords6
        SELECT id, tasksession_oid
        FROM imsAuditEvent12
        WHERE
            imsAuditEvent12.audit_time < CUTOFF_TIME AND
            imsAuditEvent12.envname = IME_NAME;

    SELECT 'Sourcing stale event objects from imsAuditEventObject12 . . .';
    CREATE TEMPORARY TABLE idmstaleeventobjectrecords6 (id INT NOT NULL, parent_event_id INT NOT NULL);
    INSERT INTO idmstaleeventobjectrecords6
        SELECT imsAuditEventObject12.id, imsAuditEventObject12.parent_event_id
        FROM imsAuditEventObject12,idmstaleeventrecords6
        WHERE
            imsAuditEventObject12.parent_event_id = idmstaleeventrecords6.id;

    -- clear imsAuditObjectAttributes12
    SELECT 'Deleting stale data from imsAuditObjectAttributes12 . . .';
    DELETE FROM imsAuditObjectAttributes12
        WHERE parent_object_id IN
            (SELECT DISTINCT id from idmstaleeventobjectrecords6);

    -- clear imsAuditTaskSession12
    SELECT 'Deleting stale data from imsAuditTaskSession12 . . .';
    DELETE FROM imsAuditTaskSession12
        WHERE tasksession_oid IN
            (SELECT DISTINCT tasksession_oid from idmstaleeventrecords6);

    -- clear imsAuditObjectRelationship12
    SELECT 'Deleting stale data from imsAuditObjectRelationship12 . . .';
    DELETE FROM imsAuditObjectRelationship12
        WHERE parent_event_id IN
            (SELECT DISTINCT id from idmstaleeventrecords6);

    -- clear imsAuditEventObject12
    SELECT 'Deleting stale data from imsAuditEventObject12 . . .';
    DELETE FROM imsAuditEventObject12
        WHERE parent_event_id IN
            (SELECT DISTINCT id from idmstaleeventrecords6);

    -- clear imsAuditEvent12
    SELECT 'Deleting stale data from imsAuditEvent12 . . .';
    DELETE FROM imsAuditEvent12
        WHERE id IN
            (SELECT DISTINCT id from idmstaleeventrecords6);

    -- clear temporary tables
    DROP TABLE idmstaleeventrecords6;
    DROP TABLE idmstaleeventobjectrecords6;
END$$

DELIMITER ;

