-- COPYRIGHT (C) 2009, CA, INC. ALL RIGHTS RESERVED.
--
-- THIS FILE CONTAINS THE SCHEMA FOR 
-- IAM FRAMEWORK TABLES FOR AUDITING
--
--

-- Add the Version table
CREATE TABLE IMSAUDIT_VERSION (
    IMR_KEY VARCHAR2(255) NOT NULL,
    IMR_VALUE VARCHAR2(255) NOT NULL
    );

INSERT INTO IMSAUDIT_VERSION(IMR_KEY,IMR_VALUE) VALUES ('version','12.6.2.0.0');

CREATE TABLE IMSAUDITTASKSESSION12 (
	ID INTEGER NOT NULL ,
	PARENT_TS_OID VARCHAR2 (100) NULL ,
	PARENT_EVENT_OID VARCHAR2 (100) NULL ,
	AUDIT_TIME TIMESTAMP DEFAULT SYSDATE NOT NULL,
	TASKSESSION_OID VARCHAR2 (100) NOT NULL ,
	ADMIN_DN VARCHAR2 (512) NOT NULL ,
	ADMIN_NAME VARCHAR2 (255) NULL ,
	TASK_NAME VARCHAR2 (255) NULL ,
	TASK_TAG VARCHAR2 (255) NOT NULL ,
	TASK_DESCRIPTION VARCHAR2 (4000) NULL ,
	TASK_PRIORITY INTEGER NULL ,
	STATE VARCHAR2 (100) NOT NULL ,
	ENVNAME VARCHAR2 (100) NOT NULL ,
	ENV_OID VARCHAR2 (100) NOT NULL ,
	CONSTRAINT PK_01_TS12 PRIMARY KEY  (ID),
	CONSTRAINT AUDIT_UNIQUE_TS12 UNIQUE (TASKSESSION_OID, STATE)
);
/

CREATE INDEX IMSAUDITTASKSESSION12_INDEX_1 ON IMSAUDITTASKSESSION12 (AUDIT_TIME);
/
 
CREATE INDEX IMSAUDITTASKSESSION12_INDEX_2 ON IMSAUDITTASKSESSION12 (ADMIN_NAME);
/

CREATE TABLE IMSAUDITEVENT12 (
	ID INTEGER NOT NULL ,
	TASKSESSION_ID INTEGER NOT NULL ,
	TASKSESSION_OID VARCHAR2 (100) NULL ,
	PARENT_EVENT_OID VARCHAR2 (100) NULL ,
	AUDIT_TIME TIMESTAMP DEFAULT SYSDATE NOT NULL,
	EVENT_OID VARCHAR2 (100) NOT NULL ,
	ADMIN_DN VARCHAR2 (512) NOT NULL ,
	ADMIN_NAME VARCHAR2 (255) NULL ,
	EVENT_NAME VARCHAR2 (255) NOT NULL ,
	EVENT_DESCRIPTION VARCHAR2 (4000) NULL ,
	EVENT_STATE VARCHAR2 (100) NULL ,
	ENVNAME VARCHAR2 (100) NOT NULL ,
	ENV_OID VARCHAR2 (100) NOT NULL ,
	CONSTRAINT PK_01_EVT12 PRIMARY KEY  (ID)
) ;
/

CREATE INDEX IMSAUDITEVENT12_INDEX_1 ON IMSAUDITEVENT12 (AUDIT_TIME);
/
 
CREATE INDEX IMSAUDITEVENT12_INDEX_2 ON IMSAUDITEVENT12 (ADMIN_NAME);
/

CREATE INDEX IMSAUDITEVENT12_INDEX_3 ON IMSAUDITEVENT12 (EVENT_NAME,EVENT_STATE);
/

CREATE TABLE IMSAUDITEVENTOBJECT12 (
	ID INTEGER NOT NULL ,
	PARENT_EVENT_ID INTEGER NOT NULL ,
	AUDIT_TIME TIMESTAMP DEFAULT SYSDATE NOT NULL,
	OBJECT_TYPE VARCHAR2 (100) NOT NULL ,
 	OBJECT_NAME VARCHAR2 (255) NULL ,
 	CONSTRAINT PK_01_EVTOBJ12 PRIMARY KEY  (ID)
); 
/

CREATE INDEX IMSAUDITEVENTOBJECT12_INDEX_1 ON IMSAUDITEVENTOBJECT12 (PARENT_EVENT_ID);
/

CREATE TABLE IMSAUDITOBJECTATTRIBUTES12 (
	ID INTEGER NOT NULL ,
	PARENT_OBJECT_ID INTEGER NOT NULL ,
	AUDIT_TIME TIMESTAMP DEFAULT SYSDATE NOT NULL,
	DISPLAY_NAME VARCHAR2 (255) NULL ,
	ATTRIBUTE_NAME VARCHAR2 (255) NOT NULL ,
	ATTRIBUTE_OLDVALUE VARCHAR2 (4000) NULL ,
	ATTRIBUTE_NEWVALUE VARCHAR2 (4000) NULL ,
	CONSTRAINT PK_01_OBJATTR12 PRIMARY KEY  (ID)
) ;
/

CREATE INDEX IMSAUDITOBJECTATTRIB12_INDEX_1 ON IMSAUDITOBJECTATTRIBUTES12 (PARENT_OBJECT_ID);
/

CREATE TABLE IMSAUDITOBJECTRELATIONSHIP12 (
	ID  INTEGER NOT NULL ,
	PARENT_EVENT_ID INTEGER NOT NULL ,
	AUDIT_TIME TIMESTAMP DEFAULT SYSDATE NOT NULL,
	OBJECT_TYPE VARCHAR2 (100) NOT NULL ,
	OBJECT_DN VARCHAR2 (512) NOT NULL ,
	CONTAINER_TYPE VARCHAR2 (100) NOT NULL ,
	OBJECT_NAME VARCHAR2  (255)  NOT NULL ,
	CONTAINER_NAME VARCHAR2 (255)  NOT NULL ,
	CONTAINER_DN VARCHAR2 (512) NOT NULL ,
	OPERATION VARCHAR2  (50)  NOT NULL, 
	CONSTRAINT PK_01_OBJREL12 PRIMARY KEY  (ID)

);
/

CREATE INDEX IMSAUDITOBJECTREL12_INDEX_1 ON IMSAUDITOBJECTRELATIONSHIP12 (PARENT_EVENT_ID);
/

-- TRIGGERS
CREATE SEQUENCE SEQ_AUDITOBJATTR12 INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER "AUDITOBJATTR12_PK" 
BEFORE INSERT ON "IMSAUDITOBJECTATTRIBUTES12"    
FOR EACH ROW 
BEGIN     
	IF INSERTING THEN       
		IF :NEW."ID" IS NULL THEN          
			SELECT SEQ_AUDITOBJATTR12.NEXTVAL INTO :NEW."ID" FROM DUAL;
		END IF;    
	END IF; 
END;
/

ALTER TRIGGER "AUDITOBJATTR12_PK" ENABLE;
/ 

CREATE SEQUENCE SEQ_AUDITOBJREL12 INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER "AUDITOBJECTREL12_PK" 
BEFORE INSERT ON "IMSAUDITOBJECTRELATIONSHIP12"    
FOR EACH ROW 
BEGIN     
	IF INSERTING THEN       
		IF :NEW."ID" IS NULL THEN          
			SELECT SEQ_AUDITOBJREL12.NEXTVAL INTO :NEW."ID" FROM DUAL;
		END IF;
	END IF; 
END;
/
ALTER TRIGGER "AUDITOBJECTREL12_PK" ENABLE;
/ 

-- ***************************************
-- INSERT PROCEDURES

CREATE SEQUENCE SEQ_AUDITTS12 INCREMENT BY 1 START WITH 1 ;
/

create or replace PROCEDURE INSERTIMSAUDITTASKSESSION125(
      L_PARENT_TS_OID IN VARCHAR2 ,
      L_PARENT_EVENT_OID IN VARCHAR2,
      L_TASKSESSION_OID IN VARCHAR2,
      L_ADMIN_DN IN VARCHAR2,
      L_ADMIN_NAME IN VARCHAR2 ,
      L_TASK_NAME IN VARCHAR2,
      L_TASK_TAG IN VARCHAR2,
      L_TASK_DESCRIPTION IN VARCHAR2,      
      L_TASK_PRIORITY IN INTEGER ,
      L_STATE IN VARCHAR2,
      L_ENVNAME IN  VARCHAR2,
      L_ENV_OID IN  VARCHAR2,
      L_AUDIT_TIME IN TIMESTAMP,
      CURRENT_ID OUT INT)
AS
BEGIN
  BEGIN
    CURRENT_ID := 0;
    SELECT id INTO CURRENT_ID FROM IMSAUDITTASKSESSION12 a	WHERE
		   ( a.TASKSESSION_OID = L_TASKSESSION_OID
		    AND a.STATE  = L_STATE
		    AND a.ENV_OID  = L_ENV_OID );
    EXCEPTION
		WHEN NO_DATA_FOUND THEN
		CURRENT_ID := 0;

    IF not (CURRENT_ID > 0) THEN
		   SELECT SEQ_AUDITTS12.NEXTVAL INTO CURRENT_ID
                    FROM DUAL;
      BEGIN                    
        INSERT INTO IMSAUDITTASKSESSION12 (
		    ID,
		    PARENT_TS_OID	, 
		    PARENT_EVENT_OID,
		    TASKSESSION_OID	,
		    ADMIN_DN	,
		    ADMIN_NAME	,
		    TASK_NAME	,
		    TASK_TAG	,
		    TASK_DESCRIPTION	,
		    TASK_PRIORITY	,
		    STATE	,
            ENVNAME	,
            ENV_OID	,
            AUDIT_TIME	) 
		    VALUES ( 
		    CURRENT_ID,
		    L_PARENT_TS_OID	, 
		    L_PARENT_EVENT_OID	, 
		    L_TASKSESSION_OID	,
		    L_ADMIN_DN	,
		    L_ADMIN_NAME	,
		    L_TASK_NAME	,
		    L_TASK_TAG	,
		    L_TASK_DESCRIPTION	,
		    L_TASK_PRIORITY	,
		    L_STATE	,
		    L_ENVNAME	,
		    L_ENV_OID	,
			L_AUDIT_TIME	);
       EXCEPTION
        WHEN DUP_VAL_ON_INDEX THEN
        SELECT id INTO CURRENT_ID FROM IMSAUDITTASKSESSION12 a	WHERE
		   ( a.TASKSESSION_OID = L_TASKSESSION_OID
		    AND a.STATE  = L_STATE
		    AND a.ENV_OID  = L_ENV_OID );
    END;
END IF;
END;
END INSERTIMSAUDITTASKSESSION125;
/

CREATE SEQUENCE SEQ_AUDITEVTOBJ12 INCREMENT BY 1 START WITH 1 ;
/

CREATE OR REPLACE PROCEDURE INSERTIMSAUDITEVENTOBJECT125(
	PARENT_EVENT_ID INTEGER,
	OBJECT_TYPE VARCHAR2,
	OBJECT_NAME VARCHAR2,
	AUDIT_TIME TIMESTAMP,
	CURRENT_ID OUT INTEGER)
AS
BEGIN
	SELECT SEQ_AUDITEVTOBJ12.NEXTVAL INTO CURRENT_ID
        FROM DUAL;
 	INSERT INTO IMSAUDITEVENTOBJECT12 (
	ID,
	PARENT_EVENT_ID ,
	OBJECT_TYPE, 
	OBJECT_NAME,
	AUDIT_TIME  ) 
            VALUES (
	CURRENT_ID,
	PARENT_EVENT_ID ,
	OBJECT_TYPE, 
	OBJECT_NAME,
	AUDIT_TIME
 	) ;
END INSERTIMSAUDITEVENTOBJECT125;
/

CREATE SEQUENCE SEQ_AUDITEVT12 INCREMENT BY 1 START WITH 1 ;
/

CREATE OR REPLACE PROCEDURE INSERTIMSAUDITEVENT125(
	PARENT_TS_OID VARCHAR2	,
	PARENT_EVENT_OID VARCHAR2	,
	EVENT_OID VARCHAR2	,
	ADMIN_DN VARCHAR2	,
	ADMIN_NAME	VARCHAR2,
	EVENT_NAME	VARCHAR2,
	EVENT_DESCRIPTION VARCHAR2,
	EVENT_STATE VARCHAR2	,
	ENVNAME VARCHAR2	,
	ENV_OID VARCHAR2	 ,
	TASKSESSION_ID INTEGER,
	AUDIT_TIME TIMESTAMP	,
	CURRENT_EVENTID OUT INTEGER)
AS
BEGIN
	SELECT SEQ_AUDITEVT12.NEXTVAL INTO CURRENT_EVENTID
        FROM DUAL;
 	INSERT INTO IMSAUDITEVENT12 (
 	ID,
	TASKSESSION_ID,
        TASKSESSION_OID	,
        PARENT_EVENT_OID	,
	EVENT_OID	,
	ADMIN_DN	,
	ADMIN_NAME	,
	EVENT_NAME	,
	EVENT_DESCRIPTION	,
	EVENT_STATE	,
	ENVNAME	,
	ENV_OID	,
	AUDIT_TIME	) 
            VALUES (
         CURRENT_EVENTID,
	TASKSESSION_ID,
	PARENT_TS_OID	,
	PARENT_EVENT_OID	,
	EVENT_OID	,
	ADMIN_DN	,
	ADMIN_NAME	,
	EVENT_NAME	,
	EVENT_DESCRIPTION	,
	EVENT_STATE	,
	ENVNAME	,
	ENV_OID	,
	AUDIT_TIME
	) ;
END INSERTIMSAUDITEVENT125;
/

--**********************
-- GARBAGE COLLECTION

CREATE GLOBAL TEMPORARY TABLE IDMSTALEEVENTRECORDS6 (
	ID NUMBER  NOT NULL,
  TASKSESSION_OID VARCHAR2(100)
) ON COMMIT PRESERVE ROWS;

CREATE GLOBAL TEMPORARY TABLE IDMSTALEEVENTOBJECTRECORDS6 (
	ID NUMBER  NOT NULL,
  PARENT_EVENT_ID NUMBER NOT NULL
) ON COMMIT PRESERVE ROWS;

CREATE OR REPLACE PROCEDURE GARBAGECOLLECTAUDITING12(
	IME_ID VARCHAR2,
	CUTOFF_TIME timestamp)
AS
BEGIN
  dbms_output.put_line('Sourcing stale events from imsAuditEvent12 . . .');
	INSERT INTO  IDMSTALEEVENTRECORDS6 SELECT ID, TASKSESSION_OID FROM IMSAUDITEVENT12 WHERE
    IMSAUDITEVENT12.AUDIT_TIME < CUTOFF_TIME AND IMSAUDITEVENT12.ENVNAME = IME_ID;
  
  dbms_output.put_line('Sourcing stale event objects from imsAuditEventObject12 . . .');
 	INSERT INTO  IDMSTALEEVENTOBJECTRECORDS6 SELECT IMSAUDITEVENTOBJECT12.ID, IMSAUDITEVENTOBJECT12.PARENT_EVENT_ID FROM IMSAUDITEVENTOBJECT12,IDMSTALEEVENTRECORDS6 WHERE
    IMSAUDITEVENTOBJECT12.PARENT_EVENT_ID = IDMSTALEEVENTRECORDS6.ID;

  dbms_output.put_line('Deleting stale data from imsAuditObjectAttributes12 . . .');
  DELETE FROM IMSAUDITOBJECTATTRIBUTES12 WHERE PARENT_OBJECT_ID IN
    (SELECT DISTINCT ID FROM IDMSTALEEVENTOBJECTRECORDS6);
    
  dbms_output.put_line('Deleting stale data from imsAuditTaskSession12 . . .');
  DELETE FROM IMSAUDITTASKSESSION12 WHERE TASKSESSION_OID IN
    (SELECT DISTINCT TASKSESSION_OID FROM IDMSTALEEVENTRECORDS6);
    
  dbms_output.put_line('Deleting stale data from imsAuditObjectRelationship12 . . .');
  DELETE FROM IMSAUDITOBJECTRELATIONSHIP12 WHERE PARENT_EVENT_ID IN
    (SELECT DISTINCT ID FROM IDMSTALEEVENTRECORDS6);

  dbms_output.put_line('Deleting stale data from imsAuditEventObject12 . . .');
  DELETE FROM IMSAUDITEVENTOBJECT12 WHERE PARENT_EVENT_ID IN
    (SELECT DISTINCT ID FROM IDMSTALEEVENTRECORDS6);

  dbms_output.put_line('Deleting stale data from imsAuditEvent12 . . .');
  DELETE FROM IMSAUDITEVENT12 WHERE ID IN
    (SELECT DISTINCT ID FROM IDMSTALEEVENTRECORDS6);
  
  -- Clear Temp Tables
  DELETE FROM IDMSTALEEVENTRECORDS6;
  DELETE FROM IDMSTALEEVENTOBJECTRECORDS6;
END GARBAGECOLLECTAUDITING12;
/

create or replace FUNCTION IM_Audit_Start_Date (rangeVal int, multiplier int, startdatetime DATE)

RETURN DATE is
tempid1 DATE;

BEGIN

if rangeVal = -1
then tempid1 := startdatetime;
else
case rangeVal
  when 1 then tempid1 := sysdate - multiplier; --Last N days
  when 2 then tempid1 := trunc(sysdate, 'DAY') - (7 * multiplier); --weekly
  when 3 then tempid1 := add_months(trunc(sysdate, 'month'), -multiplier); -- first day of the month
  when 4 then tempid1 := add_months(trunc(sysdate, 'month'), -(3 * multiplier)); -- quarter of the month
  when 5 then tempid1 := add_months(trunc(sysdate,'y'), -(12 * multiplier)); --Yearly
  else tempid1 := sysdate;
end case;
end if;
return tempid1;
END;
/

create or replace FUNCTION IM_Audit_End_Date (rangeVal int, enddatetime DATE)

RETURN DATE is
tempid1 DATE;

BEGIN
if rangeVal = -1
then tempid1 := enddatetime;
else
case rangeVal
  when 1 then tempid1 := sysdate; --Last N days
  when 2 then tempid1 := trunc(sysdate, 'day') -1 + (1439/1440); --weekly
  when 3 then tempid1 := trunc(sysdate, 'month') - 1 + (1439/1440); -- last day of the month
  when 4 then tempid1 := trunc(sysdate, 'month') - 1 + (1439/1440); -- quarter of the month
  when 5 then tempid1 := trunc(sysdate,'y')-1 + (1439/1440); --Yearly
else tempid1 := sysdate;
end case;
end if;
return tempid1;
END;
/