-- COPYRIGHT (C) 2010, CA Technologies, INC. ALL RIGHTS RESERVED.

-- This SQL script adds missing Oracle Auditing Garbage collection
-- to Audit databases created before 12.5 SP5

--**********************
-- GARBAGE COLLECTION

DECLARE
	OBJECT_EXIST PLS_INTEGER;
BEGIN
	SELECT COUNT(*)	INTO OBJECT_EXIST FROM USER_TABLES WHERE TABLE_NAME = 'IDMSTALEEVENTRECORDS6';
		IF OBJECT_EXIST = 0 THEN
			EXECUTE IMMEDIATE 'CREATE GLOBAL TEMPORARY TABLE IDMSTALEEVENTRECORDS6 (ID NUMBER NOT NULL, TASKSESSION_OID VARCHAR2(100)) ON COMMIT PRESERVE ROWS';
	END IF;

	SELECT COUNT(*)	INTO OBJECT_EXIST FROM USER_TABLES WHERE TABLE_NAME = 'IDMSTALEEVENTOBJECTRECORDS6';
		IF OBJECT_EXIST = 0 THEN
			EXECUTE IMMEDIATE 'CREATE GLOBAL TEMPORARY TABLE IDMSTALEEVENTOBJECTRECORDS6 (ID NUMBER NOT NULL, PARENT_EVENT_ID NUMBER NOT NULL) ON COMMIT PRESERVE ROWS';
	END IF;

	SELECT COUNT(*) INTO OBJECT_EXIST FROM USER_SEQUENCES WHERE SEQUENCE_NAME = 'SEQ_AUDITTS12';
		IF OBJECT_EXIST = 0 THEN
			EXECUTE IMMEDIATE 'CREATE SEQUENCE SEQ_AUDITTS12 INCREMENT BY 1 START WITH 1';
	END IF;

	SELECT COUNT(*) INTO OBJECT_EXIST FROM USER_SEQUENCES WHERE SEQUENCE_NAME = 'SEQ_AUDITEVTOBJ12';
		IF OBJECT_EXIST = 0 THEN
			EXECUTE IMMEDIATE 'CREATE SEQUENCE SEQ_AUDITEVTOBJ12 INCREMENT BY 1 START WITH 1';
	END IF;

	SELECT COUNT(*) INTO OBJECT_EXIST FROM USER_SEQUENCES WHERE SEQUENCE_NAME = 'SEQ_AUDITEVT12';
		IF OBJECT_EXIST = 0 THEN
			EXECUTE IMMEDIATE 'CREATE SEQUENCE SEQ_AUDITEVT12 INCREMENT BY 1 START WITH 1';
	END IF;

	SELECT COUNT(*) INTO OBJECT_EXIST FROM USER_PROCEDURES WHERE OBJECT_NAME = 'INSERTIMSAUDITTASKSESSION12';
		IF OBJECT_EXIST = 1 THEN
			EXECUTE IMMEDIATE 'DROP PROCEDURE INSERTIMSAUDITTASKSESSION12';
	END IF;

	SELECT COUNT(*) INTO OBJECT_EXIST FROM USER_PROCEDURES WHERE OBJECT_NAME = 'INSERTIMSAUDITEVENTOBJECT12';
		IF OBJECT_EXIST = 1 THEN
			EXECUTE IMMEDIATE 'DROP PROCEDURE INSERTIMSAUDITEVENTOBJECT12';
	END IF;

	SELECT COUNT(*) INTO OBJECT_EXIST FROM USER_PROCEDURES WHERE OBJECT_NAME = 'INSERTIMSAUDITEVENT12';
		IF OBJECT_EXIST = 1 THEN
			EXECUTE IMMEDIATE 'DROP PROCEDURE INSERTIMSAUDITEVENT12';
	END IF;
END;
/

CREATE OR REPLACE PROCEDURE GARBAGECOLLECTAUDITING12(
	IME_ID VARCHAR2,
	CUTOFF_TIME timestamp)
AS
BEGIN
  dbms_output.put_line('Sourcing stale events from imsAuditEvent12 . . .');
	INSERT INTO  IDMSTALEEVENTRECORDS6 SELECT ID, TASKSESSION_OID FROM IMSAUDITEVENT12 WHERE
    IMSAUDITEVENT12.AUDIT_TIME < CUTOFF_TIME AND IMSAUDITEVENT12.ENVNAME = IME_ID;
  
  dbms_output.put_line('Sourcing stale event objects from imsAuditEventObject12 . . .');
 	INSERT INTO  IDMSTALEEVENTOBJECTRECORDS6 SELECT IMSAUDITEVENTOBJECT12.ID, IMSAUDITEVENTOBJECT12.PARENT_EVENT_ID FROM IMSAUDITEVENTOBJECT12,IDMSTALEEVENTRECORDS6 WHERE
    IMSAUDITEVENTOBJECT12.PARENT_EVENT_ID = IDMSTALEEVENTRECORDS6.ID;

  dbms_output.put_line('Deleting stale data from imsAuditObjectAttributes12 . . .');
  DELETE FROM IMSAUDITOBJECTATTRIBUTES12 WHERE PARENT_OBJECT_ID IN
    (SELECT DISTINCT ID FROM IDMSTALEEVENTOBJECTRECORDS6);
    
  dbms_output.put_line('Deleting stale data from imsAuditTaskSession12 . . .');
  DELETE FROM IMSAUDITTASKSESSION12 WHERE TASKSESSION_OID IN
    (SELECT DISTINCT TASKSESSION_OID FROM IDMSTALEEVENTRECORDS6);
    
  dbms_output.put_line('Deleting stale data from imsAuditObjectRelationship12 . . .');
  DELETE FROM IMSAUDITOBJECTRELATIONSHIP12 WHERE PARENT_EVENT_ID IN
    (SELECT DISTINCT ID FROM IDMSTALEEVENTRECORDS6);

  dbms_output.put_line('Deleting stale data from imsAuditEventObject12 . . .');
  DELETE FROM IMSAUDITEVENTOBJECT12 WHERE PARENT_EVENT_ID IN
    (SELECT DISTINCT ID FROM IDMSTALEEVENTRECORDS6);

  dbms_output.put_line('Deleting stale data from imsAuditEvent12 . . .');
  DELETE FROM IMSAUDITEVENT12 WHERE ID IN
    (SELECT DISTINCT ID FROM IDMSTALEEVENTRECORDS6);
  
  -- Clear Temp Tables
  DELETE FROM IDMSTALEEVENTRECORDS6;
  DELETE FROM IDMSTALEEVENTOBJECTRECORDS6;
END GARBAGECOLLECTAUDITING12;
/

create or replace PROCEDURE INSERTIMSAUDITTASKSESSION125(
      L_PARENT_TS_OID IN VARCHAR2 ,
      L_PARENT_EVENT_OID IN VARCHAR2,
      L_TASKSESSION_OID IN VARCHAR2,
      L_ADMIN_DN IN VARCHAR2,
      L_ADMIN_NAME IN VARCHAR2 ,
      L_TASK_NAME IN VARCHAR2,
      L_TASK_TAG IN VARCHAR2,
      L_TASK_DESCRIPTION IN VARCHAR2,      
      L_TASK_PRIORITY IN INTEGER ,
      L_STATE IN VARCHAR2,
      L_ENVNAME IN  VARCHAR2,
      L_ENV_OID IN  VARCHAR2,
      L_AUDIT_TIME IN TIMESTAMP,
      CURRENT_ID OUT INT)
AS
BEGIN
  BEGIN
    CURRENT_ID := 0;
    SELECT id INTO CURRENT_ID FROM IMSAUDITTASKSESSION12 a	WHERE
		   ( a.TASKSESSION_OID = L_TASKSESSION_OID
		    AND a.STATE  = L_STATE
		    AND a.ENV_OID  = L_ENV_OID );
    EXCEPTION
		WHEN NO_DATA_FOUND THEN
		CURRENT_ID := 0;

    IF not (CURRENT_ID > 0) THEN
		   SELECT SEQ_AUDITTS12.NEXTVAL INTO CURRENT_ID
                    FROM DUAL;
      BEGIN                    
        INSERT INTO IMSAUDITTASKSESSION12 (
		    ID,
		    PARENT_TS_OID	, 
		    PARENT_EVENT_OID,
		    TASKSESSION_OID	,
		    ADMIN_DN	,
		    ADMIN_NAME	,
		    TASK_NAME	,
		    TASK_TAG	,
		    TASK_DESCRIPTION	,
		    TASK_PRIORITY	,
		    STATE	,
            ENVNAME	,
            ENV_OID	,
            AUDIT_TIME	) 
		    VALUES ( 
		    CURRENT_ID,
		    L_PARENT_TS_OID	, 
		    L_PARENT_EVENT_OID	, 
		    L_TASKSESSION_OID	,
		    L_ADMIN_DN	,
		    L_ADMIN_NAME	,
		    L_TASK_NAME	,
		    L_TASK_TAG	,
		    L_TASK_DESCRIPTION	,
		    L_TASK_PRIORITY	,
		    L_STATE	,
		    L_ENVNAME	,
		    L_ENV_OID	,
			L_AUDIT_TIME	);
       EXCEPTION
        WHEN DUP_VAL_ON_INDEX THEN
        SELECT id INTO CURRENT_ID FROM IMSAUDITTASKSESSION12 a	WHERE
		   ( a.TASKSESSION_OID = L_TASKSESSION_OID
		    AND a.STATE  = L_STATE
		    AND a.ENV_OID  = L_ENV_OID );
    END;
END IF;
END;
END INSERTIMSAUDITTASKSESSION125;
/

CREATE OR REPLACE PROCEDURE INSERTIMSAUDITEVENTOBJECT125(
	PARENT_EVENT_ID INTEGER,
	OBJECT_TYPE VARCHAR2,
	OBJECT_NAME VARCHAR2,
	AUDIT_TIME TIMESTAMP,
	CURRENT_ID OUT INTEGER)
AS
BEGIN
	SELECT SEQ_AUDITEVTOBJ12.NEXTVAL INTO CURRENT_ID
        FROM DUAL;
 	INSERT INTO IMSAUDITEVENTOBJECT12 (
	ID,
	PARENT_EVENT_ID ,
	OBJECT_TYPE, 
	OBJECT_NAME,
	AUDIT_TIME  ) 
            VALUES (
	CURRENT_ID,
	PARENT_EVENT_ID ,
	OBJECT_TYPE, 
	OBJECT_NAME,
	AUDIT_TIME
 	) ;
END INSERTIMSAUDITEVENTOBJECT125;
/

CREATE OR REPLACE PROCEDURE INSERTIMSAUDITEVENT125(
	PARENT_TS_OID VARCHAR2	,
	PARENT_EVENT_OID VARCHAR2	,
	EVENT_OID VARCHAR2	,
	ADMIN_DN VARCHAR2	,
	ADMIN_NAME	VARCHAR2,
	EVENT_NAME	VARCHAR2,
	EVENT_DESCRIPTION VARCHAR2,
	EVENT_STATE VARCHAR2	,
	ENVNAME VARCHAR2	,
	ENV_OID VARCHAR2	 ,
	TASKSESSION_ID INTEGER,
	AUDIT_TIME TIMESTAMP	,
	CURRENT_EVENTID OUT INTEGER)
AS
BEGIN
	SELECT SEQ_AUDITEVT12.NEXTVAL INTO CURRENT_EVENTID
        FROM DUAL;
 	INSERT INTO IMSAUDITEVENT12 (
 	ID,
	TASKSESSION_ID,
        TASKSESSION_OID	,
        PARENT_EVENT_OID	,
	EVENT_OID	,
	ADMIN_DN	,
	ADMIN_NAME	,
	EVENT_NAME	,
	EVENT_DESCRIPTION	,
	EVENT_STATE	,
	ENVNAME	,
	ENV_OID	,
	AUDIT_TIME	) 
            VALUES (
         CURRENT_EVENTID,
	TASKSESSION_ID,
	PARENT_TS_OID	,
	PARENT_EVENT_OID	,
	EVENT_OID	,
	ADMIN_DN	,
	ADMIN_NAME	,
	EVENT_NAME	,
	EVENT_DESCRIPTION	,
	EVENT_STATE	,
	ENVNAME	,
	ENV_OID	,
	AUDIT_TIME
	) ;
END INSERTIMSAUDITEVENT125;
/

UPDATE IMSAUDIT_VERSION SET IMR_VALUE = '13.0.0.0.0' WHERE IMR_KEY = 'version';
