-- Copyright (C) 2009, CA, Inc. All rights reserved.
--
-- This file contains the schema for 
-- IAM Framework Tables for auditing
--
--
-- Add the Version table
CREATE TABLE imsAudit_version (
    imr_key nvarchar(255) NOT NULL,
    imr_value nvarchar(255) NOT NULL
    )
go

insert into imsAudit_version(imr_key,imr_value) values ('version','12.6.2.0.0')
go

CREATE TABLE [imsAuditTaskSession12] (
	[id] [int] IDENTITY (1, 1) NOT NULL ,
	[parent_ts_oid] [nvarchar] (100) NULL ,
	[parent_event_oid] [nvarchar] (100) NULL ,
	[audit_time] [datetime] NOT NULL CONSTRAINT [DF_imsAuditTaskSession12_audit_time] DEFAULT (getdate()),
	[tasksession_oid] [nvarchar] (100) NOT NULL ,
	[admin_dn] [nvarchar] (512) NOT NULL ,
	[admin_name] [nvarchar] (255) NULL ,
	[task_name] [nvarchar] (255) NULL ,
	[task_tag] [nvarchar] (255) NOT NULL ,
	[task_description] [nvarchar] (4000) NULL ,
	[task_priority] [int] NULL ,
	[state] [nvarchar] (100) NOT NULL ,
	[envname] [nvarchar] (100) NOT NULL ,
	[env_oid] [nvarchar] (100) NOT NULL 
) ON [PRIMARY]
GO

CREATE INDEX imsAuditTaskSession12_index_1 ON imsAuditTaskSession12 (audit_time)
go
 
CREATE INDEX imsAuditTaskSession12_index_2 ON imsAuditTaskSession12 (admin_name)
go


CREATE TABLE [imsAuditEvent12] (
	[id] [int] IDENTITY (1, 1) NOT NULL ,
	[tasksession_id] [int] NOT NULL ,
	[tasksession_oid] [nvarchar] (100) NULL ,
	[parent_event_oid] [nvarchar] (100) NULL ,
	[audit_time] [datetime] NOT NULL CONSTRAINT [DF_imsAuditEvent12_audit_time] DEFAULT (getdate()),
	[event_oid] [nvarchar] (100) NOT NULL ,
	[admin_dn] [nvarchar] (512) NOT NULL ,
	[admin_name] [nvarchar] (255) NULL ,
	[event_name] [nvarchar] (255) NOT NULL ,
	[event_description] [nvarchar] (4000) NULL ,
	[event_state] [nvarchar] (100) NULL ,
	[envname] [nvarchar] (100) NOT NULL ,
	[env_oid] [nvarchar] (100) NOT NULL 
) ON [PRIMARY]
GO

CREATE INDEX imsAuditEvent12_index_1 ON imsAuditEvent12 (audit_time)
go
 
CREATE INDEX imsAuditEvent12_index_2 ON imsAuditEvent12 (admin_name)
go

CREATE INDEX imsAuditEvent12_index_3 ON imsAuditEvent12 (event_name,event_state)
go

CREATE TABLE [imsAuditEventObject12] (
	[id] [int] IDENTITY (1, 1) NOT NULL ,
	[parent_event_id] [int] NOT NULL ,
	[audit_time] [datetime] NOT NULL CONSTRAINT [DF_imsAuditEventObject12_audit_time] DEFAULT (getdate()),
	[object_type] [nvarchar] (100) NOT NULL ,
 	[object_name] [nvarchar] (255) NULL ,
) ON [PRIMARY]
GO

CREATE INDEX imsAuditEventObject12_index_1 ON imsAuditEventObject12 (parent_event_id)
go


CREATE TABLE [imsAuditObjectAttributes12] (
	[id] [int] IDENTITY (1, 1) NOT NULL ,
	[parent_object_id] [int] NOT NULL ,
	[audit_time] [datetime] NOT NULL CONSTRAINT [DF_imsAuditObjectAttributes12_audit_time] DEFAULT (getdate()),
	[display_name] [nvarchar] (255) NULL ,
	[attribute_name] [nvarchar] (255) NOT NULL ,
	[attribute_oldvalue] [nvarchar] (4000) NULL ,
	[attribute_newvalue] [nvarchar] (4000) NULL 
) ON [PRIMARY]
GO

CREATE INDEX imsAuditObjectAttributes12_index_1 ON imsAuditObjectAttributes12 (parent_object_id)
go


CREATE TABLE [imsAuditObjectRelationship12] (
	[id] [int] IDENTITY (1, 1) NOT NULL ,
	[parent_event_id] [int] NOT NULL ,
	[audit_time] [datetime] NOT NULL CONSTRAINT [DF_imsAuditObjectRelationship12_audit_time] DEFAULT (getdate()),
	[object_type] [nvarchar] (100)  NOT NULL ,
	[object_dn] [nvarchar] (512)  NOT NULL ,
	[object_name] [nvarchar] (255)  NOT NULL ,
	[container_type] [nvarchar] (100)  NOT NULL ,
	[container_dn] [nvarchar] (512)  NOT NULL ,
	[container_name] [nvarchar] (255)  NOT NULL ,
	[operation] [nvarchar] (50)  NOT NULL 
) ON [PRIMARY]
GO


CREATE INDEX imsAuditObjectRelationship12_index_1 ON imsAuditObjectRelationship12 (parent_event_id)
go

-- ***************************************
-- Insert procedures

CREATE PROCEDURE insertImsAuditTasksession125
      @parent_ts_oid nvarchar(100) ,
      @parent_event_oid nvarchar(100) ,
      @tasksession_oid nvarchar(100) ,
      @admin_dn  nvarchar(512) ,
      @admin_name nvarchar(255) ,
      @task_name  nvarchar(255) ,
      @task_tag  nvarchar(255) ,
      @task_description nvarchar(4000) ,      
      @task_priority int ,
      @state nvarchar(100) ,
      @envname   nvarchar(100) ,
      @env_oid    nvarchar(100),
      @audit_time	datetime,
      @current_id INTEGER OUTPUT
AS
BEGIN
SET @current_id = 0;
Select @current_id =id  from imsAuditTaskSession12 WITH (NOLOCK) 
        WHERE
           ( tasksession_oid= @tasksession_oid
            AND state  = @state
            AND env_oid  = @env_oid ) ;

  IF not  @current_id >0
       BEGIN
 	INSERT into imsAuditTaskSession12 WITH (ROWLOCK) (
            parent_ts_oid	, 
            parent_event_oid,
            tasksession_oid	,
            admin_dn	,
            admin_name	,
            task_name	,
            task_tag	,
            task_description	,
            task_priority	,
            state	,
            envname	,
            env_oid	,
            audit_time	) 
            values ( 
            @parent_ts_oid	, 
            @parent_event_oid	, 
            @tasksession_oid	,
            @admin_dn	,
            @admin_name	,
            @task_name	,
            @task_tag	,
            @task_description	,
            @task_priority	,
            @state	,
            @envname	,
            @env_oid	,
            @audit_time	) ;
 	SELECT @current_id=IDENT_CURRENT('imsAuditTaskSession12');
    END
END
GO

CREATE PROCEDURE insertImsAuditEventObject125
	@parent_event_id int,
	@object_type nvarchar(100),
	@object_name nvarchar(255),
	@audit_time datetime,
	@current_id INTEGER OUTPUT
AS
BEGIN
 	INSERT into imsAuditEventObject12 WITH (ROWLOCK) (
	parent_event_id ,
	object_type, 
	object_name, 
	audit_time  ) 
            values (
	@parent_event_id ,
	@object_type, 
	@object_name,
	@audit_time
 ) ;
    SELECT @current_id=IDENT_CURRENT('imsAuditEventObject12');
END
GO

CREATE PROCEDURE insertImsAuditEvent125
	@parent_ts_oid nvarchar	(100)	,
	@parent_event_oid nvarchar	(100)	,
	@event_oid nvarchar	(100)	,
	@admin_dn nvarchar	(512)	,
	@admin_name	nvarchar(255),
	@event_name	nvarchar(255),
	@event_description nvarchar(4000)	,
	@event_state nvarchar(100)	,
	@envname nvarchar(100)	,
	@env_oid nvarchar(100)	 ,
	@tasksession_id int,
	@audit_time datetime	,
	@current_eventid INTEGER OUTPUT
AS
BEGIN
 	INSERT into imsAuditEvent12 WITH (ROWLOCK) (
	tasksession_id,
        tasksession_oid	,
        parent_event_oid	,
	event_oid	,
	admin_dn	,
	admin_name	,
	event_name	,
	event_description	,
	event_state	,
	envname	,
	env_oid	,
	audit_time	) 
            values (
	@tasksession_id,
	@parent_ts_oid	,
	@parent_event_oid	,
	@event_oid	,
	@admin_dn	,
	@admin_name	,
	@event_name	,
	@event_description	,
	@event_state	,
	@envname	,
	@env_oid	,
	@audit_time
 ) ;
    SELECT @current_eventid=IDENT_CURRENT('imsAuditEvent12');
END
GO



--**********************
-- Garbage collection

CREATE PROCEDURE garbageCollectAuditing125
@ime_name nvarchar(100),
@cutoff_time datetime

AS

BEGIN

	PRINT 'Sourcing stale events from imsAuditEvent12 . . .'
	CREATE TABLE idmstaleeventrecords6 ([id] [int] NOT NULL, [tasksession_oid] [nvarchar] (100) NOT NULL)
	INSERT INTO  idmstaleeventrecords6 SELECT id, tasksession_oid from imsAuditEvent12 where
	imsAuditEvent12.audit_time < @cutoff_time AND imsAuditEvent12.envname = @ime_name
	
	PRINT 'Sourcing stale event objects from imsAuditEventObject12 . . .'
	CREATE TABLE idmstaleeventobjectrecords6 ([id] [int] NOT NULL, [parent_event_id] [int] NOT NULL)
	INSERT INTO  idmstaleeventobjectrecords6 SELECT imsAuditEventObject12.id, imsAuditEventObject12.parent_event_id from imsAuditEventObject12,idmstaleeventrecords6 where
	imsAuditEventObject12.parent_event_id = idmstaleeventrecords6.id

    -- clear imsAuditObjectAttributes12
    PRINT 'Deleting stale data from imsAuditObjectAttributes12 . . .'
    DELETE FROM imsAuditObjectAttributes12 WHERE parent_object_id in
        (SELECT DISTINCT id from idmstaleeventobjectrecords6)
        
      -- clear imsAuditTaskSession12
    PRINT 'Deleting stale data from imsAuditTaskSession12 . . .'
    DELETE FROM imsAuditTaskSession12 WHERE tasksession_oid in
        (SELECT DISTINCT tasksession_oid from idmstaleeventrecords6)    
        
	  -- clear imsAuditObjectRelationship12
    PRINT 'Deleting stale data from imsAuditObjectRelationship12 . . .'
    DELETE FROM imsAuditObjectRelationship12 WHERE parent_event_id in
        (SELECT DISTINCT id from idmstaleeventrecords6)    

	  -- clear imsAuditEventObject12
    PRINT 'Deleting stale data from imsAuditEventObject12 . . .'
    DELETE FROM imsAuditEventObject12 WHERE parent_event_id in
        (SELECT DISTINCT id from idmstaleeventrecords6)    

	  -- clear imsAuditEvent12
    PRINT 'Deleting stale data from imsAuditEvent12 . . .'
    DELETE FROM imsAuditEvent12 WHERE id in
        (SELECT DISTINCT id from idmstaleeventrecords6)    

	  -- clear temporary tables	
	DROP TABLE idmstaleeventrecords6
	DROP TABLE idmstaleeventobjectrecords6

END

GO

CREATE FUNCTION [IM_Audit_Start_Date](@rangeVal integer, @multiplier integer, @startdatetime datetime)
RETURNS datetime
AS
BEGIN
DECLARE @tempid1 datetime, @tempid2 datetime

if @rangeVal = -1 
SET @tempid1 = @startdatetime;
else
if @rangeVal = 1 --days
SET @tempid1 = dateadd(day, -@multiplier, SYSDATETIME());
else
if @rangeVal = 2 --weekly
SET @tempid2 = dateadd (day, (-7 * @multiplier) + 1, dateadd(DAY, (-DATEPART(WEEKDAY, sysdatetime())), sysdatetime()))
if @rangeVal = 3 -- monthly
SET @tempid1 = dateadd(DAY,-(day(sysdatetime())- 1), DATEADD(MONTH, -@multiplier, SYSDATETIME()));
if @rangeVal = 4 --quarter
SET @tempid1 = dateadd(month, -(3 *@multiplier), DATEADD(DAY, -(DATEPART(day, sysdatetime())-1), SYSDATETIME()))
if @rangeVal = 5 -- yearly
SET @tempid1 = dateadd(day, -(DATEPART(day, sysdatetime())-1), DATEADD(MONTH, -(DATEPART(month, sysdatetime()) - 1), DATEADD(year, -@multiplier, SYSDATETIME())))

RETURN @tempid1

END
GO

CREATE FUNCTION [IM_Audit_End_Date](@rangeVal integer, @enddatetime datetime)
RETURNS datetime
AS
BEGIN
DECLARE @tempid1 datetime, @tempid2 datetime

if @rangeVal = -1 
SET @tempid1 = @enddatetime;
else
if @rangeVal = 1 --days
SET @tempid1 = SYSDATETIME();
else
if @rangeVal = 2 --weekly
SET @tempid2 = DATEADD(day, -DATEPART(WEEKDAY, sysdatetime()), sysdatetime())
if @rangeVal = 3 -- monthly
SET @tempid1 = dateadd(DAY,-(day(sysdatetime())), SYSDATETIME());
if @rangeVal = 4 --quarter
SET @tempid1 = dateadd(DAY,-(day(sysdatetime())), SYSDATETIME());
if @rangeVal = 5 -- yearly
SET @tempid1 = dateadd(day, -(DATEPART(day, sysdatetime())), DATEADD(MONTH, -(DATEPART(month, sysdatetime()) - 1), SYSDATETIME()))

RETURN @tempid1

END
GO
