--
-- Runtime status detail related schema objects.
--

-- INNODB is required for FOREIGN KEY CONSTRAINTS and AUTO_INCREMENT.
SET storage_engine=INNODB;


CREATE TABLE runtimeStatusDetail12 (
        id                  INT NOT NULL,
        originationTime     DATETIME NOT NULL,
        eventID             VARCHAR (255) NOT NULL,
        parentID            VARCHAR (255) NULL,
        sourceIdentifier    VARCHAR (255) NOT NULL,
        serverName          VARCHAR (255) NOT NULL,
        provDomainName      VARCHAR (255) NULL,
        displayName         VARCHAR (255) NOT NULL,
        description         VARCHAR (2000) NULL,
        descriptionType     VARCHAR (255) NULL,
        descriptionParam    VARCHAR (1024) NULL,
        admindn             VARCHAR (628) NOT NULL,
        adminName           VARCHAR (255) NOT NULL,
        importance          INT NULL
);

ALTER TABLE runtimeStatusDetail12 ADD
        CONSTRAINT PK_runtimeStatusDetail12 PRIMARY KEY (
        id
);

ALTER TABLE runtimeStatusDetail12 MODIFY
        id INT NOT NULL AUTO_INCREMENT;


-- may have to revisit this... can't auto_increment
-- composite primary key, so create single primary key
-- and composite unique key...

CREATE TABLE runtimeStatusDetailAttribute12 (
        id              INT NOT NULL,
        rsdid           INT NOT NULL,
        profiletype     INT NOT NULL,
        profilename     VARCHAR (255) NOT NULL,
        attrname        VARCHAR (255) NOT NULL,
        attrvalue       VARCHAR (1024) NULL,
        status          VARCHAR (255) NULL,
        attrType        VARCHAR (255) NULL,
        additionalInfo  VARCHAR (255) NULL
);

ALTER TABLE runtimeStatusDetailAttribute12 ADD
        CONSTRAINT PK_rsdAttribute12 PRIMARY KEY (
            -- rsdid,
            id
        );

ALTER TABLE runtimeStatusDetailAttribute12 MODIFY
        id INT NOT NULL AUTO_INCREMENT;

ALTER TABLE runtimeStatusDetailAttribute12 ADD
        CONSTRAINT U_rsdAttribute12 UNIQUE (
            rsdid,
            id
        );


DROP PROCEDURE IF EXISTS insertRuntimeStatusDetail;

DELIMITER ++

CREATE PROCEDURE insertRuntimeStatusDetail (
        IN originationTime  DATETIME,
        IN eventID          VARCHAR (255),
        IN parentID         VARCHAR (255),
        IN sourceIdentifier VARCHAR (255),
        IN serverName       VARCHAR (255),
        IN provDomainName   VARCHAR (255),
        IN displayName      VARCHAR (255),
        IN description      VARCHAR (2000),
        IN descriptionType  VARCHAR (255),
        IN descriptionParam VARCHAR (1024),
        IN admindn          VARCHAR (628),
        IN adminName        VARCHAR (255),
        IN importance       INT,
        OUT current_id      INT
)
BEGIN
        INSERT INTO runtimeStatusDetail12 (
                originationTime,
                eventID,
                parentID,
                sourceIdentifier,
                serverName,
                provDomainName,
                displayName,
                description,
                descriptionType,
                descriptionParam,
                admindn,
                adminName,
                importance
        )
        VALUES (
                originationTime,
                eventID,
                parentID,
                sourceIdentifier,
                serverName,
                provDomainName,
                displayName,
                description,
                descriptionType,
                descriptionParam,
                admindn,
                adminName,
                importance
        );
        SELECT LAST_INSERT_ID() INTO current_id;
END++

DROP PROCEDURE IF EXISTS insertRuntimeStatusDetailAttr++

CREATE PROCEDURE insertRuntimeStatusDetailAttr(
        IN rsdid            INT,
        IN profiletype      INT,
        IN profilename      VARCHAR (255),
        IN attrname         VARCHAR (255),
        IN attrvalue        VARCHAR (1024),
        IN status           VARCHAR (255),
        IN attrType         VARCHAR (255),
        IN additionalInfo   VARCHAR (255)
)
BEGIN
        INSERT INTO runtimeStatusDetailAttribute12 (
                rsdid,
                profiletype,
                profilename,
                attrname,
                attrvalue,
                status,
                attrType,
                additionalInfo
        )
        VALUES (
                rsdid,
                profiletype,
                profilename,
                attrname,
                attrvalue,
                status,
                attrType,
                additionalInfo
        );
END++

DELIMITER ;


CREATE TABLE AuxData12 (
        Id          INT NOT NULL,
        EnvOID      VARCHAR(64) NOT NULL,
        ParentId    INT NULL,
        Precedence  INT NULL,
        ProviderId  INT NULL,
        Name        VARCHAR(255) NOT NULL,
        Type        VARCHAR(255) NOT NULL,
        DisplayName VARCHAR(255) NULL
);

ALTER TABLE AuxData12 ADD 
        CONSTRAINT PK_AuxData12 PRIMARY KEY (
                Id
        );

ALTER TABLE AuxData12 MODIFY 
        Id INT NOT NULL AUTO_INCREMENT;


DROP PROCEDURE IF EXISTS insertAuxData;

DELIMITER ++

CREATE PROCEDURE insertAuxData (
        IN envOID       VARCHAR(64),
        IN parentID     INT,
        IN precedence   INT,
        IN providerID   INT,
        IN name         VARCHAR(255),
        IN type         VARCHAR(255),
        IN displayName  VARCHAR(255),
        OUT currentID   INT
)
BEGIN
        INSERT INTO AuxData12 (
                EnvOID,
                ParentId,
                Precedence,
                ProviderId,
                Name,
                Type,
                DisplayName
        )
        VALUES (
                envOID,
                parentID,
                precedence,
                providerID,
                name,
                type,
                displayName
        );
        SELECT LAST_INSERT_ID() INTO currentID;
END++

DELIMITER ;


CREATE TABLE AuxDataProviders12 (
        Id          INT NOT NULL,
        EnvOID      VARCHAR(64) NOT NULL,
        Type        INT NOT NULL,
        DisplayName VARCHAR(255) NOT NULL,
        Precedence  INT NOT NULL,
        Description VARCHAR(255) NULL,
        NodeId      INT NULL,
        FileName    VARCHAR(255) NULL,
        ClassName   VARCHAR(255) NULL
);

ALTER TABLE AuxDataProviders12 ADD 
        CONSTRAINT PK_AuxDataProviders12 PRIMARY KEY
        (
                Id
        );

ALTER TABLE AuxDataProviders12 MODIFY
        Id INT NOT NULL AUTO_INCREMENT;


DROP PROCEDURE IF EXISTS insertAuxDataProvider;

DELIMITER ++

CREATE PROCEDURE insertAuxDataProvider (
        IN envOID VARCHAR(64),
        IN type VARCHAR(255),
        IN displayName VARCHAR(255),
        IN precedence INT,
        IN description VARCHAR(255),
        IN nodeID INT,
        IN fileName VARCHAR(255),
        IN className VARCHAR(255),
        OUT currentID INT
)
BEGIN
        INSERT INTO AuxDataProviders12 (
                EnvOID,
                Type,
                DisplayName,
                Precedence,
                Description,
                NodeId,
                FileName,
                ClassName
        )
        VALUES (
                envOID,
                type,
                displayName,
                precedence,
                description,
                nodeID,
                fileName,
                className
        );
        SELECT LAST_INSERT_ID() INTO currentID;
END++

DELIMITER ;


-- with charset=utf8, MySQL requires 3x the number of chars and
-- consquently throws a ERROR 1071: Specified key was too long; max length
-- is 767 bytes

CREATE TABLE lock12_5 (
        lockid      VARCHAR (50)   NOT NULL,
        -- lockinfo    VARCHAR (400)  NOT NULL
        lockinfo    VARCHAR (200)  NOT NULL
);

ALTER TABLE lock12_5 ADD
        CONSTRAINT PK_lock12_5 PRIMARY KEY (
                lockid,
                lockinfo
        );


CREATE TABLE object12_5 (
        objectid    VARCHAR (50) NOT NULL,
        objecttype  INT NOT NULL,
        -- BLOB wasn't long enough for object column
        object      LONGBLOB NULL
);

ALTER TABLE object12_5 ADD
        CONSTRAINT PK_object12_5 PRIMARY KEY (
                objectid,
                objecttype
        );


CREATE TABLE tasksession12_5 (
        tasksessionid       VARCHAR (50) NOT NULL,
        state               VARCHAR(50) NOT NULL,
        created_time        DATETIME NOT NULL,
        last_access_time    DATETIME NOT NULL,
        user_dn             VARCHAR (512)NOT NULL,
        org_dn              VARCHAR (512) NOT NULL,
        environmentid       VARCHAR (50) NOT NULL,
        name                VARCHAR (512) NOT NULL,
        description         VARCHAR (2000) NOT NULL,
        acknowledged        CHAR NOT NULL,
        parent_of_wf_id     VARCHAR (50) NULL,
        action_type         INT NOT NULL,
        priority            INT NULL,
        initiatorid         VARCHAR (50) NULL,
        nesting_type        INT NULL,
        next_state          INT NULL
);

ALTER TABLE tasksession12_5 ADD
        CONSTRAINT PK_tasksession12_5 PRIMARY KEY (
                tasksessionid
        );


CREATE TABLE event12_5 (
        eventid             VARCHAR (50) NOT NULL,
        state               INT NOT NULL,
        tasksessionid       VARCHAR (50) NOT NULL,
        created_on          DATETIME NOT NULL,
        last_access_time    DATETIME NOT NULL,
        name                VARCHAR (512) NOT NULL,
        description         VARCHAR (2000) NOT NULL,
        type                INT NOT NULL,
        next_state          INT NULL
);

ALTER TABLE event12_5 ADD
        CONSTRAINT PK_event12_5 PRIMARY KEY   (
                eventid
        );


-- with charset=utf8, MySQL requires 3x the number of chars and
-- consquently throws a ERROR 1071: Specified key was too long; max length
-- is 767 bytes

CREATE TABLE event_object12_5 (
        tasksessionid       VARCHAR (50) NOT NULL,
        eventid             VARCHAR (50) NOT NULL,
        -- object_type         VARCHAR (128) NOT NULL,
        object_type         VARCHAR (100) NOT NULL,
        isprimary           CHAR NOT NULL,
        -- object_uniquename   VARCHAR (512) NOT NULL
        object_uniquename   VARCHAR (250) NOT NULL
);

ALTER TABLE event_object12_5 ADD
        CONSTRAINT PK_event_object12_5 PRIMARY KEY (
                eventid,
                object_type,
                object_uniquename
        );


CREATE TABLE tp_version (
        tp_key      VARCHAR (255) NOT NULL,
        tp_value    VARCHAR(255) NOT NULL
);

ALTER TABLE tp_version ADD
        CONSTRAINT PK_tp_version PRIMARY KEY (
                tp_key
        );

INSERT INTO tp_version(tp_key,tp_value) VALUES ('version','12.6.2.0.0');


ALTER TABLE event12_5 ADD
        CONSTRAINT FK_event12_5_tasksession12_5 FOREIGN KEY (
                tasksessionid
        )
        REFERENCES tasksession12_5 (
                tasksessionid
        );


ALTER TABLE tasksession12_5 ADD
        CONSTRAINT FK_tasksession12_5_tasksession12_5 FOREIGN KEY (
                initiatorid
        ) REFERENCES tasksession12_5 (
                tasksessionid
        );


ALTER TABLE event_object12_5 ADD
        CONSTRAINT FK_event_object12_5_event12_5 FOREIGN KEY (
                eventid
        ) REFERENCES event12_5 (
                eventid
        );


ALTER TABLE AuxData12 ADD
        CONSTRAINT FK_AuxData12_parent FOREIGN KEY (
                ParentId
        ) REFERENCES AuxData12 (
                Id
        );


ALTER TABLE AuxData12 ADD
        CONSTRAINT FK_AuxData12_provider FOREIGN KEY (
                ProviderId
        ) REFERENCES AuxDataProviders12 (
                Id
        );


ALTER TABLE AuxDataProviders12 ADD
        CONSTRAINT FK_AuxDataProviders12_rootNode FOREIGN KEY (
                NodeId
        ) REFERENCES AuxData12 (
                Id
        );


create index idx_ts12_5_initiatorid on tasksession12_5(initiatorid);
create index idx_event12_5_tasksessionid on event12_5(tasksessionid);
create index idx_rtmstatusdetail12_eventid on runtimestatusdetail12(eventid);


-- Quartz 1.7.3 tables begin


CREATE TABLE qrtz_job_details (
        JOB_NAME            VARCHAR(200) NOT NULL,
        JOB_GROUP           VARCHAR(200) NOT NULL,
        DESCRIPTION         VARCHAR(250) NULL,
        JOB_CLASS_NAME      VARCHAR(250) NOT NULL,
        IS_DURABLE          VARCHAR(1) NOT NULL,
        IS_VOLATILE         VARCHAR(1) NOT NULL,
        IS_STATEFUL         VARCHAR(1) NOT NULL,
        REQUESTS_RECOVERY   VARCHAR(1) NOT NULL,
        JOB_DATA            LONGBLOB NULL,

        PRIMARY KEY (JOB_NAME, JOB_GROUP)
);

-- The inline REFERENCES specifications where the references are defined as part of the column specification
-- are silently ignored by InnoDB. InnoDB only accepts REFERENCES clauses defined as part of a separate
-- FOREIGN KEY specification.

CREATE TABLE qrtz_job_listeners (
        JOB_NAME        VARCHAR(200) NOT NULL,
        JOB_GROUP       VARCHAR(200) NOT NULL,
        JOB_LISTENER    VARCHAR(200) NOT NULL,

        PRIMARY KEY (JOB_NAME, JOB_GROUP, JOB_LISTENER)
        -- FOREIGN KEY (JOB_NAME, JOB_GROUP)
        -- REFERENCES qrtz_job_details(JOB_NAME, JOB_GROUP)
        -- ON DELETE CASCADE
);

ALTER TABLE qrtz_job_listeners ADD
        CONSTRAINT FK_qrtz_job_listeners_qrtz_job_details FOREIGN KEY (
                JOB_NAME,
                JOB_GROUP
        )
        REFERENCES qrtz_job_details (
                JOB_NAME,
                JOB_GROUP
        ) ON DELETE CASCADE;


CREATE TABLE qrtz_triggers (
        TRIGGER_NAME    VARCHAR(200) NOT NULL,
        TRIGGER_GROUP   VARCHAR(200) NOT NULL,
        JOB_NAME        VARCHAR(200) NOT NULL,
        JOB_GROUP       VARCHAR(200) NOT NULL,
        IS_VOLATILE     VARCHAR(1) NOT NULL,
        DESCRIPTION     VARCHAR(250) NULL,
        NEXT_FIRE_TIME  BIGINT NULL,
        PREV_FIRE_TIME  BIGINT NULL,
        PRIORITY        INT NULL,
        TRIGGER_STATE   VARCHAR(16) NOT NULL,
        TRIGGER_TYPE    VARCHAR(8) NOT NULL,
        START_TIME      BIGINT NOT NULL,
        END_TIME        BIGINT NULL,
        CALENDAR_NAME   VARCHAR(200) NULL,
        MISFIRE_INSTR   SMALLINT NULL,
        JOB_DATA        LONGBLOB NULL,

        PRIMARY KEY (TRIGGER_NAME, TRIGGER_GROUP)
        -- FOREIGN KEY (JOB_NAME, JOB_GROUP)
        -- REFERENCES qrtz_job_details(JOB_NAME, JOB_GROUP)
);

ALTER TABLE qrtz_triggers ADD
        CONSTRAINT FK_qrtz_triggers_qrtz_job_details FOREIGN KEY (
                JOB_NAME,
                JOB_GROUP
        )
        REFERENCES qrtz_job_details (
                JOB_NAME,
                JOB_GROUP
        );


CREATE TABLE qrtz_simple_triggers (
        TRIGGER_NAME    VARCHAR(200) NOT NULL,
        TRIGGER_GROUP   VARCHAR(200) NOT NULL,
        REPEAT_COUNT    BIGINT NOT NULL,
        REPEAT_INTERVAL BIGINT NOT NULL,
        TIMES_TRIGGERED BIGINT NOT NULL,

        PRIMARY KEY (TRIGGER_NAME, TRIGGER_GROUP)
        -- FOREIGN KEY (TRIGGER_NAME, TRIGGER_GROUP)
        -- REFERENCES qrtz_triggers(TRIGGER_NAME, TRIGGER_GROUP)
        -- ON DELETE CASCADE
);

ALTER TABLE qrtz_simple_triggers ADD
        CONSTRAINT FK_qrtz_simple_triggers_qrtz_triggers FOREIGN KEY (
                TRIGGER_NAME,
                TRIGGER_GROUP
        )
        REFERENCES qrtz_triggers (
                TRIGGER_NAME,
                TRIGGER_GROUP
        ) ON DELETE CASCADE;


CREATE TABLE qrtz_cron_triggers (
        TRIGGER_NAME    VARCHAR(200) NOT NULL,
        TRIGGER_GROUP   VARCHAR(200) NOT NULL,
        CRON_EXPRESSION VARCHAR(120) NOT NULL,
        TIME_ZONE_ID    VARCHAR(80) NULL,

        PRIMARY KEY (TRIGGER_NAME, TRIGGER_GROUP)
        -- FOREIGN KEY (TRIGGER_NAME, TRIGGER_GROUP)
        -- REFERENCES qrtz_triggers(TRIGGER_NAME, TRIGGER_GROUP)
        -- ON DELETE CASCADE
);

ALTER TABLE qrtz_cron_triggers ADD
        CONSTRAINT FK_qrtz_cron_triggers_qrtz_triggers FOREIGN KEY (
                TRIGGER_NAME,
                TRIGGER_GROUP
        )
        REFERENCES qrtz_triggers (
                TRIGGER_NAME,
                TRIGGER_GROUP
        ) ON DELETE CASCADE;


CREATE TABLE qrtz_blob_triggers (
        TRIGGER_NAME    VARCHAR(200) NOT NULL,
        TRIGGER_GROUP   VARCHAR(200) NOT NULL,
        BLOB_DATA       LONGBLOB NULL,

        PRIMARY KEY (TRIGGER_NAME, TRIGGER_GROUP)
        -- FOREIGN KEY (TRIGGER_NAME, TRIGGER_GROUP)
        -- REFERENCES qrtz_triggers(TRIGGER_NAME, TRIGGER_GROUP)
);

ALTER TABLE qrtz_blob_triggers ADD
        CONSTRAINT FK_qrtz_blob_triggers_qrtz_triggers FOREIGN KEY (
                TRIGGER_NAME,
                TRIGGER_GROUP
        )
        REFERENCES qrtz_triggers (
                TRIGGER_NAME,
                TRIGGER_GROUP
        );


CREATE TABLE qrtz_trigger_listeners (
        TRIGGER_NAME        VARCHAR(200) NOT NULL,
        TRIGGER_GROUP       VARCHAR(200) NOT NULL,
        TRIGGER_LISTENER    VARCHAR(200) NOT NULL,

        PRIMARY KEY (TRIGGER_NAME, TRIGGER_GROUP, TRIGGER_LISTENER)
        -- FOREIGN KEY (TRIGGER_NAME, TRIGGER_GROUP)
        -- REFERENCES qrtz_triggers(TRIGGER_NAME, TRIGGER_GROUP)
        -- ON DELETE CASCADE
);

ALTER TABLE qrtz_trigger_listeners ADD
        CONSTRAINT FK_qrtz_trigger_listeners_qrtz_triggers FOREIGN KEY (
                TRIGGER_NAME,
                TRIGGER_GROUP
        )
        REFERENCES qrtz_triggers (
                TRIGGER_NAME,
                TRIGGER_GROUP
        ) ON DELETE CASCADE;


CREATE TABLE qrtz_calendars (
        CALENDAR_NAME   VARCHAR(200) NOT NULL,
        CALENDAR        LONGBLOB NOT NULL,

        PRIMARY KEY (CALENDAR_NAME)
);


CREATE TABLE qrtz_paused_trigger_grps (
        TRIGGER_GROUP   VARCHAR(200) NOT NULL,

        PRIMARY KEY (TRIGGER_GROUP)
);


CREATE TABLE qrtz_fired_triggers (
        ENTRY_ID            VARCHAR(95) NOT NULL,
        TRIGGER_NAME        VARCHAR(200) NOT NULL,
        TRIGGER_GROUP       VARCHAR(200) NOT NULL,
        IS_VOLATILE         VARCHAR(1) NOT NULL,
        INSTANCE_NAME       VARCHAR(200) NOT NULL,
        FIRED_TIME          BIGINT NOT NULL,
        PRIORITY            INT NOT NULL,
        STATE               VARCHAR(16) NOT NULL,
        JOB_NAME            VARCHAR(200) NULL,
        JOB_GROUP           VARCHAR(200) NULL,
        IS_STATEFUL         VARCHAR(1) NULL,
        REQUESTS_RECOVERY   VARCHAR(1) NULL,

        PRIMARY KEY (ENTRY_ID)
);


CREATE TABLE qrtz_scheduler_state (
        INSTANCE_NAME       VARCHAR(200) NOT NULL,
        LAST_CHECKIN_TIME   BIGINT NOT NULL,
        CHECKIN_INTERVAL    BIGINT NOT NULL,

        PRIMARY KEY (INSTANCE_NAME)
);


CREATE TABLE qrtz_locks (
        LOCK_NAME   VARCHAR(40) NOT NULL,

        PRIMARY KEY (LOCK_NAME)
);


INSERT INTO qrtz_locks values('TRIGGER_ACCESS');
INSERT INTO qrtz_locks values('JOB_ACCESS');
INSERT INTO qrtz_locks values('CALENDAR_ACCESS');
INSERT INTO qrtz_locks values('STATE_ACCESS');
INSERT INTO qrtz_locks values('MISFIRE_ACCESS');


create index idx_qrtz_j_req_recovery on qrtz_job_details(REQUESTS_RECOVERY);
create index idx_qrtz_t_next_fire_time on qrtz_triggers(NEXT_FIRE_TIME);
create index idx_qrtz_t_state on qrtz_triggers(TRIGGER_STATE);
create index idx_qrtz_t_nft_st on qrtz_triggers(NEXT_FIRE_TIME, TRIGGER_STATE);
create index idx_qrtz_t_volatile on qrtz_triggers(IS_VOLATILE);
create index idx_qrtz_ft_trig_name on qrtz_fired_triggers(TRIGGER_NAME);
create index idx_qrtz_ft_trig_group on qrtz_fired_triggers(TRIGGER_GROUP);
create index idx_qrtz_ft_trig_nm_gp on qrtz_fired_triggers(TRIGGER_NAME, TRIGGER_GROUP);
create index idx_qrtz_ft_trig_volatile on qrtz_fired_triggers(IS_VOLATILE);
create index idx_qrtz_ft_trig_inst_name on qrtz_fired_triggers(INSTANCE_NAME);
create index idx_qrtz_ft_job_name on qrtz_fired_triggers(JOB_NAME);
create index idx_qrtz_ft_job_group on qrtz_fired_triggers(JOB_GROUP);
create index idx_qrtz_ft_job_stateful on qrtz_fired_triggers(IS_STATEFUL);
create index idx_qrtz_ft_job_req_recovery on qrtz_fired_triggers(REQUESTS_RECOVERY);



-- Garbage Collection start


DELIMITER ++

DROP PROCEDURE IF EXISTS garbageCollectById++

CREATE PROCEDURE garbageCollectById (
        IN workingId    VARCHAR(255)
)
BEGIN
        SELECT CONCAT('GC ID ', workingId);

        -- garbage collect locks
        DELETE FROM lock12_5
                WHERE lockid = workingId;

        -- garbage collect runtme status details attributes
        DELETE FROM runtimeStatusDetailAttribute12
                WHERE rsdid IN (SELECT runtimeStatusDetail12.id FROM
                        runtimeStatusDetail12 WHERE eventID = workingId);

        -- garbage collect runtme status details 
        DELETE FROM runtimeStatusDetail12 
                WHERE eventID = workingId;

        -- garbage collect object12_5
        DELETE FROM object12_5
                WHERE objectid = workingId;
END++


DROP PROCEDURE IF EXISTS garbageCollectByEventId++

CREATE PROCEDURE garbageCollectByEventId (
        IN event_id VARCHAR(255)
)
BEGIN
        SELECT CONCAT('GC event ID ', event_id);

        CALL garbageCollectById(@event_id);

        -- garbage collect event_object12_5
        DELETE FROM event_object12_5 
        WHERE eventid = event_id;

        -- garbage colelct event_object12_5
        DELETE FROM event12_5 
        WHERE eventid = event_id;
END++


DROP PROCEDURE IF EXISTS garbageCollectByTaskId++

CREATE PROCEDURE garbageCollectByTaskId (
        IN task_id  VARCHAR(255)
)
BEGIN
	DECLARE workingId VARCHAR(255);
        DECLARE done INT DEFAULT 0;

	-- nested tasks for removal
	DECLARE tasksCursor CURSOR FOR
		SELECT tasksessionid 
		FROM tasksession12_5 
		WHERE initiatorid = task_id AND tasksessionid != initiatorid;

	-- events for removal
	DECLARE eventCursor CURSOR FOR
		SELECT eventid
		FROM event12_5
		WHERE tasksessionid = task_id;

        DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

	SELECT CONCAT('GC task ID ', task_id);
	
	-- garbage collect nested tasks first
	OPEN tasksCursor;
        tasks_loop: LOOP
                FETCH tasksCursor INTO workingId;
                IF done THEN
                        LEAVE tasks_loop;  -- exit loop if condition is true
                END IF;
                CALL garbageCollectByTaskId(@workingId);
	END LOOP;
	CLOSE tasksCursor;

        SET done = 0;   -- reset continue handler
	OPEN eventCursor;
        events_loop: LOOP
                FETCH eventCursor INTO workingId;
                IF done THEN
                        LEAVE events_loop;  -- exit loop if condition is true
                END IF;
		CALL garbageCollectByEventId(@workingId);
	END LOOP;
	CLOSE eventcursor;
	
	CALL garbageCollectById(@task_id);
	
	-- garbage collect from tasksession12_5;
        DELETE FROM tasksession12_5
                WHERE tasksessionid = task_id;
END++


DROP PROCEDURE IF EXISTS garbageCollectTaskPersistence++

-- MySQL doesn't support default procedure variables so use COALESCE
-- to return the default value if passed in parameter is NULL.

CREATE PROCEDURE garbageCollectTaskPersistence (
        IN ime_id               VARCHAR(255),
        IN cutoff_time          DATETIME,
        -- IN audit_timeout_days   FLOAT DEFAULT 1,
        IN audit_timeout_days   FLOAT,
        -- IN chunk_rec_count      INT DEFAULT 500
        IN chunk_rec_count      INT
)
BEGIN
        DECLARE gcTaskId VARCHAR(255);
        DECLARE counter INT;
        DECLARE done INT DEFAULT 0;

        -- top level tasks for removal
        DECLARE gcTasksCursor CURSOR FOR
        SELECT tasksessionid
        FROM tasksession12_5
        WHERE environmentid = ime_id
        AND (
                -- final states cut-off
                (
                    (
                        (tasksession12_5.state = 512) --   mark for deletion 
                        OR (tasksession12_5.state = 128) --  completed  
                        OR (tasksession12_5.state = 256) --  cancelled   
                        OR (tasksession12_5.state = 32) -- rejected   
                        OR (tasksession12_5.state = 2) -- invalid   
                        OR (tasksession12_5.state = 1048576) --  task aborted 
                    )
                    AND last_access_time < cutoff_time 
                )
                -- audit state conditional cut-off
                OR
                (
                    tasksession12_5.state = 16384 -- audit state
                    AND last_access_time < 
                            LEAST(cutoff_time, SYSDATE() -
                    -- if audit_timeout_days is NULL COALESCE 'defaults' to 1
                                    COALESCE(audit_timeout_days, 1))
                )
            )
        AND (initiatorid IS NULL OR initiatorid = tasksessionid);

        DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;


        SET counter = 0;
        OPEN gcTasksCursor;
        gcTasks_loop: LOOP
                FETCH gcTasksCursor INTO gcTaskId;
                IF done THEN
                        LEAVE gcTasks_loop;  -- exit loop if condition is true
                END IF;
                SET counter = counter + 1;
                CALL garbageCollectByTaskId(@gcTaskId);
                -- if chunk_rec_count is NULL COALESCE 'defaults' to 500
                IF counter >= COALESCE(chunk_rec_count, 500) THEN
                        COMMIT;
                        SET counter = 0;
                        SELECT CONCAT('GC Commited ', counter);
                END IF;
        END LOOP;
        CLOSE gcTasksCursor;
        COMMIT;
END++

DELIMITER ;

