-- INNODB is required for FOREIGN KEY CONSTRAINTS and AUTO_INCREMENT.
SET storage_engine=INNODB;
DROP PROCEDURE IF EXISTS remove_table_constraints;

DELIMITER ++

CREATE PROCEDURE remove_table_constraints ()
BEGIN
    DECLARE table_count INT;

    -- foreign key constraints must be deleted first
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'event12_5' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE event12_5 DROP FOREIGN KEY FK_event12_5_tasksession12_5;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'tasksession12_5' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE tasksession12_5 DROP FOREIGN KEY FK_tasksession12_5_tasksession12_5;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'event_object12_5' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE event_object12_5 DROP FOREIGN KEY FK_event_object12_5_event12_5;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'AuxData12' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE AuxData12 DROP FOREIGN KEY FK_AuxData12_parent;
            ALTER TABLE AuxData12 DROP FOREIGN KEY FK_AuxData12_provider;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'AuxDataProviders12' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE AuxDataProviders12 DROP FOREIGN KEY FK_AuxDataProviders12_rootNode;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_simple_triggers' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_simple_triggers DROP FOREIGN KEY FK_qrtz_simple_triggers_qrtz_triggers;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_cron_triggers' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_cron_triggers DROP FOREIGN KEY FK_qrtz_cron_triggers_qrtz_triggers;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_blob_triggers' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_blob_triggers DROP FOREIGN KEY FK_qrtz_blob_triggers_qrtz_triggers;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_trigger_listeners' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_trigger_listeners DROP FOREIGN KEY FK_qrtz_trigger_listeners_qrtz_triggers;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_triggers' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_triggers DROP FOREIGN KEY FK_qrtz_triggers_qrtz_job_details;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_job_listeners' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_job_listeners DROP FOREIGN KEY FK_qrtz_job_listeners_qrtz_job_details;
    END IF;

    -- followed by any primary key, auto_increment, unique constraints or indexes
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'runtimeStatusDetail12' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE runtimeStatusDetail12 MODIFY id INT NOT NULL;
            ALTER TABLE runtimeStatusDetail12 DROP PRIMARY KEY;
            DROP INDEX idx_rtmstatusdetail12_eventid on runtimestatusdetail12;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'runtimeStatusDetailAttribute12' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE runtimeStatusDetailAttribute12 MODIFY id INT NOT NULL;
            ALTER TABLE runtimeStatusDetailAttribute12 DROP PRIMARY KEY;
            -- drop unique contraint as an index as that's how mysql implements it
            ALTER TABLE runtimeStatusDetailAttribute12 DROP INDEX U_rsdAttribute12;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'lock12_5' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE lock12_5 DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'object12_5' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE object12_5 DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'tasksession12_5' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE tasksession12_5 DROP PRIMARY KEY;
            DROP INDEX idx_ts12_5_initiatorid on tasksession12_5;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'event12_5' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE event12_5 DROP PRIMARY KEY;
            DROP INDEX idx_event12_5_tasksessionid on event12_5;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'event_object12_5' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE event_object12_5 DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'tp_version' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE tp_version DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'AuxData12' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE AuxData12 MODIFY Id INT NOT NULL;
            ALTER TABLE AuxData12 DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'AuxDataProviders12' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE AuxDataProviders12 MODIFY Id INT NOT NULL;
            ALTER TABLE AuxDataProviders12 DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_simple_triggers' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_simple_triggers DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_cron_triggers' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_cron_triggers DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_blob_triggers' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_blob_triggers DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_trigger_listeners' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_trigger_listeners DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_job_listeners' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_job_listeners DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_triggers' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_triggers DROP PRIMARY KEY;
            DROP INDEX idx_qrtz_t_next_fire_time on qrtz_triggers;
            DROP INDEX idx_qrtz_t_state on qrtz_triggers;
            DROP INDEX idx_qrtz_t_nft_st on qrtz_triggers;
            DROP INDEX idx_qrtz_t_volatile on qrtz_triggers;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_job_details' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_job_details DROP PRIMARY KEY;
            DROP INDEX idx_qrtz_j_req_recovery on qrtz_job_details;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_calendars' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_calendars DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_paused_trigger_grps' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_paused_trigger_grps DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_fired_triggers' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_fired_triggers DROP PRIMARY KEY;
            DROP INDEX idx_qrtz_ft_trig_name on qrtz_fired_triggers;
            DROP INDEX idx_qrtz_ft_trig_group on qrtz_fired_triggers;
            DROP INDEX idx_qrtz_ft_trig_nm_gp on qrtz_fired_triggers;
            DROP INDEX idx_qrtz_ft_trig_volatile on qrtz_fired_triggers;
            DROP INDEX idx_qrtz_ft_trig_inst_name on qrtz_fired_triggers;
            DROP INDEX idx_qrtz_ft_job_name on qrtz_fired_triggers;
            DROP INDEX idx_qrtz_ft_job_group on qrtz_fired_triggers;
            DROP INDEX idx_qrtz_ft_job_stateful on qrtz_fired_triggers;
            DROP INDEX idx_qrtz_ft_job_req_recovery on qrtz_fired_triggers;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_scheduler_state' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_scheduler_state DROP PRIMARY KEY;
    END IF;
    SELECT COUNT(*) INTO table_count FROM information_schema.tables WHERE table_name = 'qrtz_locks' AND table_schema = DATABASE();
    IF table_count > 0 THEN
            ALTER TABLE qrtz_locks DROP PRIMARY KEY;
    END IF;

END++

DELIMITER ;

-- Call it...
CALL remove_table_constraints();

-- ...then drop it.
DROP PROCEDURE IF EXISTS remove_table_constraints;

DROP TABLE IF EXISTS runtimeStatusDetail12;

DROP TABLE IF EXISTS runtimeStatusDetailAttribute12;

DROP PROCEDURE IF EXISTS insertRuntimeStatusDetail;

DROP PROCEDURE IF EXISTS insertRuntimeStatusDetailAttr;

DROP TABLE IF EXISTS AuxData12;

DROP TABLE IF EXISTS AuxDataProviders12;

DROP PROCEDURE IF EXISTS insertAuxData;

DROP PROCEDURE IF EXISTS insertAuxDataProvider;

DROP TABLE IF EXISTS lock12_5;

DROP TABLE IF EXISTS object12_5;

DROP TABLE IF EXISTS tasksession12_5;

DROP TABLE IF EXISTS event12_5;

DROP TABLE IF EXISTS event_object12_5;

DROP TABLE IF EXISTS tp_version;

-- drop Quartz 1.5.2 db sql begin

DROP TABLE IF EXISTS qrtz_simple_triggers;

DROP TABLE IF EXISTS qrtz_cron_triggers;

DROP TABLE IF EXISTS qrtz_blob_triggers;

DROP TABLE IF EXISTS qrtz_trigger_listeners;

DROP TABLE IF EXISTS qrtz_job_listeners;

DROP TABLE IF EXISTS qrtz_triggers;

DROP TABLE IF EXISTS qrtz_job_details;

DROP TABLE IF EXISTS qrtz_calendars;

DROP TABLE IF EXISTS qrtz_paused_trigger_grps;

DROP TABLE IF EXISTS qrtz_fired_triggers;

DROP TABLE IF EXISTS qrtz_scheduler_state;

DROP TABLE IF EXISTS qrtz_locks;

-- drop Quartz 1.5.2 db sql end 

-- drop garbage collection start

DROP PROCEDURE IF EXISTS garbageCollectById;

DROP PROCEDURE IF EXISTS garbageCollectByEventId;

DROP PROCEDURE IF EXISTS garbageCollectByTaskId;

DROP PROCEDURE IF EXISTS garbageCollectTaskPersistence;

-- drop garbage collection end
