-- Copyright (C) 2009, Computer Associates, Inc. All rights reserved.
--
-- This file contains the schema for upgrading a IM 12 schema to a IM R12.5
-- Task Persistence schema
--
-------------------------------------------------------------------

-- Add the Version table
CREATE TABLE tp_version (
        tp_key NVARCHAR2(255) NOT NULL,
        tp_value NVARCHAR2(255) NOT NULL
);

ALTER TABLE  tp_version ADD 
        CONSTRAINT PK_tp_version PRIMARY KEY
        (
                tp_key
        );

insert into tp_version(tp_key,tp_value) values ('version','12.6.2.0.0');

ALTER TABLE PX_POLICY MODIFY 
(
	ACTIONRULES		NVARCHAR2(2000)
);

CREATE TABLE event12_5 (
        eventid NVARCHAR2 (50)   NOT NULL ,
        state NUMBER(16,0) NOT NULL ,
        tasksessionid NVARCHAR2 (50)   NOT NULL ,
        created_on DATE NOT NULL ,
        last_access_time DATE NOT NULL ,
        name NVARCHAR2 (512)   NOT NULL ,
        description NVARCHAR2 (2000)   NOT NULL ,
        type NUMBER(16,0) NOT NULL ,
        next_state NUMBER(16,0)
); 

CREATE TABLE event_object12_5 (
        tasksessionid NVARCHAR2 (50)   NOT NULL ,
        eventid NVARCHAR2 (50)   NOT NULL ,
        object_type NVARCHAR2 (128) NOT NULL ,
        isprimary CHAR NOT NULL ,
        object_uniquename NVARCHAR2 (256) NOT NULL 
);

CREATE TABLE lock12_5 (
        lockid NVARCHAR2 (50)   NOT NULL ,
        lockinfo NVARCHAR2 (400)  NOT NULL
);

CREATE TABLE object12_5 (
        objectid NVARCHAR2 (50)   NOT NULL ,
        objecttype NUMBER(16,0) NOT NULL ,
        object BLOB
);

CREATE TABLE tasksession12_5 (
        tasksessionid NVARCHAR2 (50)   NOT NULL ,
        state NUMBER(16,0) NOT NULL ,
        created_time DATE NOT NULL ,
        last_access_time DATE NOT NULL ,
        user_dn NVARCHAR2 (512)   NOT NULL ,
        org_dn NVARCHAR2 (512)   NOT NULL ,
        environmentid NVARCHAR2 (50)   NOT NULL ,
        name NVARCHAR2 (512)   NOT NULL ,
        description NVARCHAR2 (2000)   NOT NULL ,
        acknowledged CHAR NOT NULL ,
        parent_of_wf_id NVARCHAR2 (50) ,
        action_type NUMBER(16,0) NOT NULL ,
        priority NUMBER(16,0),
        initiatorid NVARCHAR2 (50) ,
        nesting_type NUMBER(16,0),
        next_state NUMBER(16,0)
);

ALTER TABLE event12_5 ADD 
        CONSTRAINT PK_event12_5 PRIMARY KEY   
        (
                eventid
        )  ;

ALTER TABLE event_object12_5 ADD 
        CONSTRAINT PK_event_object12_5 PRIMARY KEY
        (
                eventid,
                object_type,
                object_uniquename
        );

ALTER TABLE lock12_5 ADD 
        CONSTRAINT PK_lock12_5 PRIMARY KEY
        (
                lockid,
                lockinfo
        );

ALTER TABLE  object12_5 ADD 
        CONSTRAINT PK_object12_5 PRIMARY KEY
        (
                objectid,
                objecttype
        );

ALTER TABLE tasksession12_5 ADD 
        CONSTRAINT PK_tasksession12_5 PRIMARY KEY
        (
                tasksessionid
        );

ALTER TABLE event12_5 ADD 
        CONSTRAINT FK_event12_5_tasksession12_5 FOREIGN KEY 
        (
                tasksessionid
        ) REFERENCES tasksession12_5 (
                tasksessionid
        );

ALTER TABLE tasksession12_5 ADD 
        CONSTRAINT FK_tsession12_5_tsession12_5 FOREIGN KEY 
        (
                initiatorid
        ) REFERENCES tasksession12_5 (
                tasksessionid
        );

ALTER TABLE event_object12_5 ADD 
        CONSTRAINT FK_event_object12_5_event12_5 FOREIGN KEY 
        (
                eventid
        ) REFERENCES event12_5 (
                eventid
        );
        
create index idx_ts12_5_initiatorid on tasksession12_5(initiatorid);
create index idx_event12_5_tasksessionid on event12_5(tasksessionid);

Alter TABLE runtimeStatusDetail12 MODIFY (descriptionParam VARCHAR2 (1024));
Alter TABLE runtimeStatusDetail12 MODIFY (description VARCHAR2 (2000));

DECLARE
	idx_rtmstatus12_eventid_exist PLS_INTEGER;
BEGIN
  select count(*)   into idx_rtmstatus12_eventid_exist from user_indexes where index_name = 'IDX_RTMSTATUSDETAIL12_EVENTID';
	if idx_rtmstatus12_eventid_exist = 0 then
		execute immediate 'create index idx_rtmstatusdetail12_eventid on runtimestatusdetail12(eventid)';
	end if;
END;
/

-- Upgrade stored procedure

--
-- garbageCollectById
--
CREATE OR REPLACE PROCEDURE garbageCollectById
(workingId IN VARCHAR2)
IS
BEGIN
    dbms_output.put_line('GC ID ' || workingId);

    -- garbage collect locks
    DELETE FROM lock12_5
    WHERE lockid = workingId;

    -- garbage collect runtme status details attributes
    DELETE FROM runtimeStatusDetailAttribute12
    WHERE rsdid IN (SELECT runtimeStatusDetail12.id FROM runtimeStatusDetail12 WHERE eventID = workingId);

    -- garbage collect runtme status details 
    DELETE FROM runtimeStatusDetail12 
    WHERE eventID = workingId;

    -- garbage collect object12_5
    DELETE FROM object12_5
    WHERE objectid = workingId;

END garbageCollectById;
/

--
-- garbageCollectByEventId
--
CREATE OR REPLACE PROCEDURE garbageCollectByEventId
(event_id IN VARCHAR2)
IS
BEGIN
    dbms_output.put_line('GC event ID ' || event_id);

    garbageCollectById(event_id);
    
    -- garbage collect event_object12_5
    DELETE FROM event_object12_5 
    WHERE eventid = event_id;
    
    -- garbage colelct event_object12_5
    DELETE FROM event12_5 
    WHERE eventid = event_id;

END garbageCollectByEventId;
/

--
-- garbageCollectByTaskId
--
CREATE OR REPLACE PROCEDURE garbageCollectByTaskId
(task_id IN VARCHAR2)
IS
    workingId VARCHAR2(255);

    -- nested tasks for removal
    CURSOR tasksCoursor IS
        SELECT tasksessionid 
        FROM tasksession12_5 
        WHERE initiatorid = task_id AND tasksessionid != initiatorid;

    -- events for removal
    CURSOR eventCoursor IS
        SELECT eventid
        FROM event12_5
        WHERE tasksessionid = task_id;
BEGIN

    dbms_output.put_line('GC task ID ' || task_id);
    
    -- garbage collect nested tasks first
    OPEN tasksCoursor;
    LOOP 
        FETCH tasksCoursor INTO workingId;
        EXIT WHEN tasksCoursor%NOTFOUND;  -- exit loop if condition is true
        garbageCollectByTaskId(workingId);
    END LOOP;
    CLOSE tasksCoursor;
    
    OPEN eventCoursor;
    LOOP 
        FETCH eventCoursor INTO workingId;
        EXIT WHEN eventCoursor%NOTFOUND;  -- exit loop if condition is true
        garbageCollectByEventId(workingId);
    END LOOP;
    CLOSE eventCoursor;
    
    garbageCollectById(task_id);
    
    -- garbage collect from tasksession12_5;
    DELETE FROM tasksession12_5
    WHERE tasksessionid = task_id;
    

END garbageCollectByTaskId;
/
--
--  garbageCollectTaskPersistence
--
CREATE OR REPLACE PROCEDURE garbageCollectTaskPersistence
(ime_id IN VARCHAR2,
cutoff_time IN DATE,
audit_timeout_days IN FLOAT DEFAULT 1, chunk_rec_count IN number DEFAULT 500)
IS
	gcTaskId VARCHAR2(255);
	counter number;
	

	-- top level tasks for removal
	CURSOR gcTasksCoursor IS
		SELECT tasksessionid 
		FROM tasksession12_5 
		WHERE environmentid = ime_id 


			AND (
					-- final states cut off
					(
						(
						
							(tasksession12_5.state = 512) --   mark for deletion 
							OR (tasksession12_5.state = 128) --  completed  
							OR (tasksession12_5.state = 256) --  cancelled   
							OR (tasksession12_5.state = 32) -- rejected   
							OR (tasksession12_5.state = 2) -- invalid   
							OR (tasksession12_5.state = 1048576) --  task aborted 
						)
						AND last_access_time < cutoff_time 
					)
					-- audit state conditional cut-off
					OR
					(
						tasksession12_5.state = 16384 -- audit state
						AND last_access_time < 
							LEAST(cutoff_time, SYSDATE - audit_timeout_days)
					)
				)
			AND (initiatorid IS NULL OR initiatorid = tasksessionid);


BEGIN
	counter:=0;
	OPEN gcTasksCoursor;
LOOP
		FETCH gcTasksCoursor INTO gcTaskId;
		
		EXIT WHEN gcTasksCoursor%NOTFOUND;
		-- exit loop if condition is true
		counter:=counter+1;
		garbageCollectByTaskId(gcTaskId);
		if counter >= chunk_rec_count then
		   COMMIT;
		   counter := 0;
		   dbms_output.put_line('GC Commited ' || counter);
		end if;
		 
END LOOP;
CLOSE gcTasksCoursor;
COMMIT;
END garbageCollectTaskPersistence;
/
-- 
-- Garbage Collection - end
--


-- Upgrade to Quartz 1.7.3 db sql begin

DECLARE
	col_exists PLS_INTEGER;
BEGIN
  SELECT  count(*) INTO col_exists FROM USER_TAB_COLUMNS WHERE table_name = 'QRTZ_FIRED_TRIGGERS' AND column_name = 'PRIORITY';
  if col_exists = 0 then
	EXECUTE IMMEDIATE 'ALTER TABLE qrtz_fired_triggers ADD PRIORITY NUMBER(13)';
  end if;
  
END;
/

DECLARE
	col_exists PLS_INTEGER;
BEGIN
  SELECT  count(*) INTO col_exists FROM USER_TAB_COLUMNS WHERE table_name = 'QRTZ_TRIGGERS' AND column_name = 'PRIORITY';
  if col_exists = 0 then
	EXECUTE IMMEDIATE 'ALTER TABLE qrtz_triggers ADD PRIORITY NUMBER(13)';
  end if;
  
END;
/

update qrtz_triggers set PRIORITY = 5 where PRIORITY IS NULL;
update qrtz_fired_triggers set PRIORITY = 5 where PRIORITY IS NULL;


-- Upgrade to Quartz 1.7.3 db sql end

