
DECLARE
	idx_event12_5_tsid_exist PLS_INTEGER;
BEGIN
  select count(*)   into idx_event12_5_tsid_exist from user_indexes where index_name = 'IDX_EVENT12_5_TASKSESSIONID';
	if idx_event12_5_tsid_exist = 0 then
		execute immediate 'create index idx_event12_5_tasksessionid on event12_5(tasksessionid);';
	end if;
END;
/
DECLARE
	idx_ts12_5_initiatorid_exist PLS_INTEGER;
BEGIN
  select count(*)   into idx_ts12_5_initiatorid_exist from user_indexes where index_name = 'IDX_TS12_5_INITIATORID';
	if idx_ts12_5_initiatorid_exist = 0 then
		execute immediate 'create index idx_ts12_5_initiatorid on tasksession12_5(initiatorid);';
	end if;
END;
/
DECLARE
	idx_rtmstatus12_eventid_exist PLS_INTEGER;
BEGIN
  select count(*)   into idx_rtmstatus12_eventid_exist from user_indexes where index_name = 'IDX_RTMSTATUSDETAIL12_EVENTID';
	if idx_rtmstatus12_eventid_exist = 0 then
		execute immediate 'create index idx_rtmstatusdetail12_eventid on runtimestatusdetail12(eventid)';
	end if;
END;
/
delete from tp_version;
insert into tp_version(tp_key,tp_value) values ('version','12.6.2.0.0');

ALTER TABLE PX_POLICY MODIFY 
(
	ACTIONRULES		NVARCHAR2(2000)
);

Alter TABLE runtimeStatusDetail12 MODIFY (descriptionParam VARCHAR2 (1024));
Alter TABLE runtimeStatusDetail12 MODIFY (description VARCHAR2 (2000));
--
-- garbageCollectById
--
CREATE OR REPLACE PROCEDURE garbageCollectById
(workingId IN VARCHAR2)
IS
BEGIN
	dbms_output.put_line('GC ID ' || workingId);

	-- garbage collect locks
	DELETE FROM lock12_5
	WHERE lockid = workingId;

	-- garbage collect runtme status details attributes
	DELETE FROM runtimeStatusDetailAttribute12
	WHERE rsdid IN (SELECT runtimeStatusDetail12.id FROM runtimeStatusDetail12 WHERE eventID = workingId);

	-- garbage collect runtme status details 
	DELETE FROM runtimeStatusDetail12 
	WHERE eventID = workingId;

	-- garbage collect object12_5
	DELETE FROM object12_5
	WHERE objectid = workingId;

END garbageCollectById;
/

--
-- garbageCollectByEventId
--
CREATE OR REPLACE PROCEDURE garbageCollectByEventId
(event_id IN VARCHAR2)
IS
BEGIN
	dbms_output.put_line('GC event ID ' || event_id);

	garbageCollectById(event_id);
	
	-- garbage collect event_object12_5
	DELETE FROM event_object12_5 
	WHERE eventid = event_id;
	
	-- garbage colelct event_object12_5
	DELETE FROM event12_5 
	WHERE eventid = event_id;

END garbageCollectByEventId;
/

--
-- garbageCollectByTaskId
--
CREATE OR REPLACE PROCEDURE garbageCollectByTaskId
(task_id IN VARCHAR2)
IS
	workingId VARCHAR2(255);

	-- nested tasks for removal
	CURSOR tasksCoursor IS
		SELECT tasksessionid 
		FROM tasksession12_5 
		WHERE initiatorid = task_id AND tasksessionid != initiatorid;

	-- events for removal
	CURSOR eventCoursor IS
		SELECT eventid
		FROM event12_5
		WHERE tasksessionid = task_id;
BEGIN

	dbms_output.put_line('GC task ID ' || task_id);
	
	-- garbage collect nested tasks first
	OPEN tasksCoursor;
	LOOP 
		FETCH tasksCoursor INTO workingId;
		EXIT WHEN tasksCoursor%NOTFOUND;  -- exit loop if condition is true
		garbageCollectByTaskId(workingId);
	END LOOP;
	CLOSE tasksCoursor;
	
	OPEN eventCoursor;
	LOOP 
		FETCH eventCoursor INTO workingId;
		EXIT WHEN eventCoursor%NOTFOUND;  -- exit loop if condition is true
		garbageCollectByEventId(workingId);
	END LOOP;
	CLOSE eventCoursor;
	
	garbageCollectById(task_id);
	
	-- garbage collect from tasksession12_5;
	DELETE FROM tasksession12_5
	WHERE tasksessionid = task_id;
	

END garbageCollectByTaskId;
/


/
--
--	garbageCollectTaskPersistence
--
CREATE OR REPLACE PROCEDURE garbageCollectTaskPersistence
(ime_id IN VARCHAR2,
cutoff_time IN DATE,
audit_timeout_days IN FLOAT DEFAULT 1, chunk_rec_count IN number DEFAULT 500)
IS
	gcTaskId VARCHAR2(255);
	counter number;
	

	-- top level tasks for removal
	CURSOR gcTasksCoursor IS
		SELECT tasksessionid 
		FROM tasksession12_5 
		WHERE environmentid = ime_id 


			AND (
					-- final states cut off
					(
						(
						
							(tasksession12_5.state = 512) --   mark for deletion 
							OR (tasksession12_5.state = 128) --  completed  
							OR (tasksession12_5.state = 256) --  cancelled   
							OR (tasksession12_5.state = 32) -- rejected   
							OR (tasksession12_5.state = 2) -- invalid   
							OR (tasksession12_5.state = 1048576) --  task aborted 
						)
						AND last_access_time < cutoff_time 
					)
					-- audit state conditional cut-off
					OR
					(
						tasksession12_5.state = 16384 -- audit state
						AND last_access_time < 
							LEAST(cutoff_time, SYSDATE - audit_timeout_days)
					)
				)
			AND (initiatorid IS NULL OR initiatorid = tasksessionid);


BEGIN
	counter:=0;
	OPEN gcTasksCoursor;
LOOP
		FETCH gcTasksCoursor INTO gcTaskId;
		
		EXIT WHEN gcTasksCoursor%NOTFOUND;
		-- exit loop if condition is true
		counter:=counter+1;
		garbageCollectByTaskId(gcTaskId);
		if counter >= chunk_rec_count then
		   COMMIT;
		   counter := 0;
		   dbms_output.put_line('GC Commited ' || counter);
		end if;
		 
END LOOP;
CLOSE gcTasksCoursor;
COMMIT;
END garbageCollectTaskPersistence;
/

-- Upgrade to Quartz 1.7.3 db sql begin

DECLARE
	col_exists PLS_INTEGER;
BEGIN
  SELECT  count(*) INTO col_exists FROM USER_TAB_COLUMNS WHERE table_name = 'QRTZ_FIRED_TRIGGERS' AND column_name = 'PRIORITY';
  if col_exists = 0 then
	EXECUTE IMMEDIATE 'ALTER TABLE qrtz_fired_triggers ADD PRIORITY NUMBER(13)';
  end if;
  
END;
/

DECLARE
	col_exists PLS_INTEGER;
BEGIN
  SELECT  count(*) INTO col_exists FROM USER_TAB_COLUMNS WHERE table_name = 'QRTZ_TRIGGERS' AND column_name = 'PRIORITY';
  if col_exists = 0 then
	EXECUTE IMMEDIATE 'ALTER TABLE qrtz_triggers ADD PRIORITY NUMBER(13)';
  end if;
  
END;
/

update qrtz_triggers set PRIORITY = 5 where PRIORITY IS NULL;
update qrtz_fired_triggers set PRIORITY = 5 where PRIORITY IS NULL;


-- Upgrade to Quartz 1.7.3 db sql end
