-- Copyright (C) 2009, Computer Associates, Inc. All rights reserved.
--
-- This file contains the schema for upgrading IM 12 schema
--
-------------------------------------------------------------------

CREATE TABLE [dbo].[event12_5] (
    [eventid] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [state] [int] NOT NULL ,
    [tasksessionid] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [created_on] [datetime] NOT NULL ,
    [last_access_time] [datetime] NOT NULL ,
    [name] [nvarchar] (512) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [description] [nvarchar] (2000) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [type] [int] NOT NULL ,
    [next_state] [int] NULL 
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[event_object12_5] (
    [tasksessionid] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [eventid] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [object_type] [nvarchar] (128) NOT NULL ,
    [isprimary] [char] NOT NULL ,
    [object_uniquename] [nvarchar] (256) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL 
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[lock12_5] (
    [lockid] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [lockinfo] [nvarchar] (400) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL 
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[object12_5] (
    [objectid] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [objecttype] [int] NOT NULL ,
    [object] [varbinary](max) NULL
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[tasksession12_5] (
    [tasksessionid] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [state] [int] NOT NULL ,
    [created_time] [datetime] NOT NULL ,
    [last_access_time] [datetime] NOT NULL ,
    [user_dn] [nvarchar] (512) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [org_dn] [nvarchar] (512) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [environmentid] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [name] [nvarchar] (512) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [description] [nvarchar] (2000) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    [acknowledged] [char] NOT NULL ,
    [parent_of_wf_id] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL ,
    [action_type] [int] NOT NULL ,
    [priority] [int] NULL ,
    [initiatorid] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL ,
    [nesting_type] [int] NOT NULL ,
    [next_state] [int] NULL 
) ON [PRIMARY]
GO

ALTER TABLE [dbo].[event12_5] WITH NOCHECK ADD 
    CONSTRAINT [PK_event12_5] PRIMARY KEY  CLUSTERED 
    (
        [eventid]
    )  ON [PRIMARY]
GO 

ALTER TABLE [dbo].[event_object12_5] WITH NOCHECK ADD 
    CONSTRAINT [PK_event_object12_5] PRIMARY KEY  CLUSTERED 
    (
        [eventid],
        [object_type],
        [object_uniquename]
    )  ON [PRIMARY]
GO

ALTER TABLE [dbo].[lock12_5] WITH NOCHECK ADD 
    CONSTRAINT [PK_lock12_5] PRIMARY KEY  CLUSTERED 
    (
        [lockid],
        [lockinfo]
    )  ON [PRIMARY]
GO 

ALTER TABLE [dbo].[object12_5] WITH NOCHECK ADD 
    CONSTRAINT [PK_object12_5] PRIMARY KEY  CLUSTERED 
    (
        [objectid],
        [objecttype]
    )  ON [PRIMARY]
GO

ALTER TABLE [dbo].[tasksession12_5] WITH NOCHECK ADD 
    CONSTRAINT [PK_tasksession12_5] PRIMARY KEY  CLUSTERED 
    (
        [tasksessionid]
    )  ON [PRIMARY]
GO 

ALTER TABLE [dbo].[event12_5] ADD 
    CONSTRAINT [FK_event12_5_tasksession12_5] FOREIGN KEY 
    (
        [tasksessionid]
    ) REFERENCES [dbo].[tasksession12_5] (
        [tasksessionid]
    )
GO

ALTER TABLE [dbo].[tasksession12_5] ADD 
    CONSTRAINT [FK_tasksession12_5_tasksession12_5] FOREIGN KEY 
    (
        [initiatorid]
    ) REFERENCES [dbo].[tasksession12_5] (
        [tasksessionid]
    )
GO

ALTER TABLE [dbo].[event_object12_5] ADD 
    CONSTRAINT [FK_event_object12_5_event12_5] FOREIGN KEY 
    (
        [eventid]
    ) REFERENCES [dbo].[event12_5] (
        [eventid]
    )
GO

-- Add the Version table
CREATE TABLE [dbo].[tp_version] (
    [tp_key] nvarchar(255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
    [tp_value] nvarchar(255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL
) ON [PRIMARY]
GO

ALTER TABLE [dbo].[tp_version] WITH NOCHECK ADD 
    CONSTRAINT [PK_tp_version] PRIMARY KEY  CLUSTERED 
    (
        [tp_key]
    )  ON [PRIMARY]
GO

insert into [dbo].[tp_version]([tp_key],[tp_value]) values ('version','12.6.2.0.0')
GO

create index [idx_ts12_5_initiatorid] on [dbo].[tasksession12_5] ([initiatorid])
GO
create index [idx_event12_5_tasksessionid] on [dbo].[event12_5] ([tasksessionid])
GO
if not exists (select name from dbo.sysindexes where name = 'idx_rtmstatusdetail12_eventid')
	create index [idx_rtmstatusdetail12_eventid] on [dbo].[runtimestatusdetail12] ([eventid])
GO

ALTER TABLE runtimeStatusDetail12 
	ALTER COLUMN description nvarchar(2000)
GO

ALTER TABLE runtimeStatusDetail12 
	ALTER COLUMN descriptionParam nvarchar(1024)
GO

ALTER TABLE px_policy alter column ACTIONRULES nvarchar(2000) 
Go

-- Upgrade stored procedure


--
-- garbageCollectById
--
IF EXISTS (SELECT name FROM sysobjects
      WHERE name = 'garbageCollectById' AND type = 'P')
   DROP PROCEDURE garbageCollectById
GO

CREATE PROCEDURE
garbageCollectById 
    @workingId VARCHAR(255)
AS
BEGIN
    PRINT 'GC ID ' + @workingId;
 
    -- garbage collect locks
    DELETE FROM lock12_5 
    WHERE lockid = @workingId;

    -- garbage collect runtme status details attributes
    DELETE FROM runtimeStatusDetailAttribute12
    WHERE rsdid IN (SELECT runtimeStatusDetail12.id FROM runtimeStatusDetail12 WHERE eventID = @workingId);

    -- garbage collect runtme status details 
    DELETE FROM runtimeStatusDetail12 
    WHERE eventID = @workingId;

    -- garbage collect object12_5
    DELETE FROM object12_5
    WHERE objectid = @workingId;

END --garbageCollectById
GO

--
-- garbageCollectByEventId
--
IF EXISTS (SELECT name FROM sysobjects
      WHERE name = 'garbageCollectByEventId' AND type = 'P')
   DROP PROCEDURE garbageCollectByEventId
GO

CREATE PROCEDURE
garbageCollectByEventId 
@event_id VARCHAR(255)
AS
BEGIN
    PRINT 'GC event ID ' + @event_id;

    EXEC garbageCollectById @event_id;
    
    -- garbage collect event_object12_5
    DELETE FROM event_object12_5 
    WHERE eventid = @event_id;
    
    -- garbage colelct event_object12_5
    DELETE FROM event12_5 
    WHERE eventid = @event_id;

END --garbageCollectByEventId
GO

--
-- garbageCollectByTaskId
--
IF EXISTS (SELECT name FROM sysobjects
      WHERE name = 'garbageCollectByTaskId' AND type = 'P')
   DROP PROCEDURE garbageCollectByTaskId
GO

CREATE PROCEDURE
garbageCollectByTaskId 
@task_id VARCHAR(255)
AS
    DECLARE @workingId VARCHAR(255);

    -- nested tasks for removal
    DECLARE tasksCoursor CURSOR LOCAL FOR
        SELECT tasksessionid 
        FROM tasksession12_5 
        WHERE initiatorid = @task_id AND tasksessionid != initiatorid;

    -- events for removal
    DECLARE eventCoursor CURSOR LOCAL FOR
        SELECT eventid
        FROM event12_5
        WHERE tasksessionid = @task_id;
BEGIN

    PRINT 'GC task ID ' + @task_id;
    
    -- garbage collect nested tasks first
    OPEN tasksCoursor;
    WHILE 1 = 1 
    BEGIN
        FETCH NEXT FROM tasksCoursor INTO @workingId;
        IF @@FETCH_STATUS <> 0 BREAK;  -- exit loop if condition is true
        EXEC garbageCollectByTaskId @workingId;
    END
    CLOSE tasksCoursor;
    DEALLOCATE tasksCoursor;

    OPEN eventCoursor;
    WHILE 1 = 1 
    BEGIN 
        FETCH NEXT FROM eventCoursor INTO @workingId;
        IF @@FETCH_STATUS <> 0 BREAK;  -- exit loop if condition is true
        EXEC garbageCollectByEventId @workingId;
    END
    CLOSE eventCoursor;
    DEALLOCATE eventCoursor;
    
    EXEC garbageCollectById @task_id;
    
    -- garbage collect from tasksession12_5;
    DELETE FROM tasksession12_5
    WHERE tasksessionid = @task_id;
    

END --garbageCollectByTaskId
GO
--
--  garbageCollectTaskPersistence
--
IF EXISTS (SELECT name FROM sysobjects
      WHERE name = 'garbageCollectTaskPersistence' AND type = 'P')
   DROP PROCEDURE garbageCollectTaskPersistence
GO

CREATE PROCEDURE garbageCollectTaskPersistence
@ime_id VARCHAR(50),
@cutoff_time DATETIME,
@audit_timeout_days FLOAT = 1,
@chunk_rec_count int =500
AS
	DECLARE @gcTaskId VARCHAR(255);
	DECLARE @counter int;
	DECLARE @transactionsCount int;
	declare @Error int

	DECLARE gcTasksCoursor CURSOR LOCAL FOR
			SELECT tasksessionid 
		FROM tasksession12_5 
		WHERE environmentid = @ime_id 
			AND (
					(
						(
						
							(tasksession12_5.state = 512) --   mark for deletion 
							OR (tasksession12_5.state = 128) --  completed  
							OR (tasksession12_5.state = 256) --  cancelled   
							OR (tasksession12_5.state = 32) -- rejected   
							OR (tasksession12_5.state = 2) -- invalid   
							OR (tasksession12_5.state = 1048576) --  task aborted 
						)
						AND last_access_time < @cutoff_time 
					)
					OR
					(
						tasksession12_5.state = 16384 -- audit state
						AND last_access_time < 
							CASE WHEN @cutoff_time < (GETDATE() - @audit_timeout_days) THEN @cutoff_time ELSE GETDATE() -@audit_timeout_days END
					)
				)
			AND (initiatorid IS NULL OR initiatorid = tasksessionid);

BEGIN
	set @counter = 0;
	OPEN gcTasksCoursor;
	begin transaction;
	WHILE 1 = 1 
	BEGIN
		FETCH NEXT FROM gcTasksCoursor INTO @gcTaskId;
		IF @@FETCH_STATUS <> 0 BREAK;  -- exit loop if condition is true
		set @counter = @counter +1;
		EXEC garbageCollectByTaskId @gcTaskId;
	    set @Error = @@ERROR
	    if @Error <> 0 --if error is raised
  		  goto LogError

		IF (@counter = @chunk_rec_count)
		BEGIN
			commit;
			set @counter = 0;
			begin transaction;
		END
	END

	if (@counter > 0)
		commit;
	set @transactionsCount = @@trancount;
	if (@transactionsCount >0)
		commit;
	CLOSE gcTasksCoursor;
	DEALLOCATE gcTasksCoursor;
	goto ProcEnd
LogError:
	set @transactionsCount = @@trancount;
	if (@transactionsCount >0)
		rollback transaction

ProcEnd:


END --garbageCollectTaskPersistence
GO
-- 
-- Garbage Collection - end
--
IF EXISTS (SELECT name FROM sysobjects WHERE name = 'insertRuntimeStatusDetail' AND type = 'P')
   DROP PROCEDURE insertRuntimeStatusDetail
GO

CREATE PROCEDURE insertRuntimeStatusDetail(
        @originationTime datetime,
                  @eventID varchar(255),
                  @parentID varchar(255),
                  @sourceIdentifier nvarchar(255),
                  @serverName nvarchar(255),
                  @provDomainName nvarchar(255),
                  @displayName nvarchar(255),
                  @description nvarchar(2000),
                  @descriptionType varchar(255),
                  @descriptionParam nvarchar(1024),
                  @admindn nvarchar(628),
                  @adminName nvarchar(255),
		        @importance integer,
         @current_id INTEGER OUTPUT)
AS
BEGIN
 insert into runtimeStatusDetail12 (
           originationTime,
                     eventID,
                     parentID,
                     sourceIdentifier,
                     serverName,
                     provDomainName,
                     displayName,
                     description,
					 					 descriptionType,
					 					 descriptionParam,
                     admindn,
                     adminName,
					importance
			)

            values (
           @originationTime,
                     @eventID,
                     @parentID,
                     @sourceIdentifier,
                     @serverName,
                     @provDomainName,
                     @displayName,
                     @description,
                     @descriptionType,
					           @descriptionParam,
                     @admindn,
                     @adminName,
		            @importance
			);

    SELECT @current_id=IDENT_CURRENT('runtimeStatusDetail12');
END
GO

-- Upgrade to Quartz 1.7.3 db sql begin

IF NOT EXISTS (SELECT * FROM dbo.syscolumns WHERE id = OBJECT_ID(N'[dbo].[QRTZ_FIRED_TRIGGERS]') AND name=N'PRIORITY')
ALTER TABLE [dbo].[QRTZ_FIRED_TRIGGERS] ADD PRIORITY INTEGER 
GO

IF NOT EXISTS (SELECT * FROM dbo.syscolumns WHERE id = OBJECT_ID(N'[dbo].[QRTZ_TRIGGERS]') AND name=N'PRIORITY')
ALTER TABLE [dbo].[QRTZ_TRIGGERS] ADD PRIORITY INTEGER 
GO

UPDATE [dbo].[QRTZ_FIRED_TRIGGERS] SET PRIORITY=5 WHERE PRIORITY IS NULL
UPDATE [dbo].[QRTZ_TRIGGERS] SET PRIORITY=5 WHERE PRIORITY IS NULL





-- Upgrade to Quartz 1.7.3 db sql end

